// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id
//
// Module  :  ClientReplicableMap.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

import com.ibm.websphere.objectgrid.plugins.ReplicationMapListener;
import com.ibm.websphere.objectgrid.security.ServerMapPermission;

/**
 * This interface represents a replicable client map. A replicable client map can be
 * a continuous replica or a snapshot replica of the server map.
 * <p>
 * If the client is a continuous replica of the server map, the data in the server will be
 * replicated to the client continuously in an asynchronous manner.
 * <p>
 * If the client is a snapshot replica of the server map, a snapshot on the data in the server
 * will be taken and the snapshot will be replicated to the client in an asynchronous manner.
 * A snapshot replication is a one-time replication.
 * <p>
 * A ReplicationMapListener can be used to listen for the data changes as well as
 * the replication lifecycle events.
 *
 * @see com.ibm.websphere.objectgrid.plugins.ReplicationMapListener
 *
 * @since WAS XD 6.1, XC10
 * @deprecated The client replicated map function is deprecated in version 8.6.  Use the {@link com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryManager} function.
 */
public interface ClientReplicableMap {

    /**
     * Client Replication mode
     */
    final class Mode {
        int mode = 0;

        Mode(int mode) {
            this.mode = mode;
        }

        public String toString() {
            StringBuffer sb = new StringBuffer("Mode[");
            switch (mode) {
            case 0:
                sb.append("NONE");
                break;
            case 1:
                sb.append("CONTINUOUS");
                break;
            case 2:
                sb.append("SNAPSHOT");
                break;
            }

            sb.append("]");
            return new String(sb);
        }
    }

    /**
     * No replication mode, aka normal mode.
     */
    public static final Mode NONE = new Mode(0);

    /**
     * Full replication mode. Data in the server map will be replicated to the client continuously.
     */
    public static final Mode CONTINUOUS_REPLICATION = new Mode(1);

    /**
     *
     * Snapshot replication mode. A snapshot on the data in the server
     * will be taken and the snapshot will be replicated to the client. A snapshot replication
     * is a one-time replication.
     */
    public static final Mode SNAPSHOT_REPLICATION = new Mode(2);

    /**
     * Make the client map a replica of the server side map.
     * <p>
     * When security is enabled, this method requires a {@link com.ibm.websphere.objectgrid.security.ServerMapPermission}
     * with action "replicate". Refer to {@link com.ibm.websphere.objectgrid.security.ServerMapPermission} for more
     * permission details.
     * <p>
     *
     * Required Client Permission: {@link ServerMapPermission#REPLICATE}
     *
     * @param mode The replication mode.
     * @param partitions
     *            The array of partition IDs represent which partitions the data should be replicated
     *            from. If the value is null or an empty array, it indicates the data should be
     *            replicated from all partitions.
     * @param listener a listener to receive client replication events
     *
     * @throws IllegalArgumentException if mode is not {@link #CONTINUOUS_REPLICATION} or {@link #SNAPSHOT_REPLICATION} or
     *                                  the map isn't currently in the mode specified or is not in {@link #NONE} mode
     * @throws IllegalStateException if this method is invoked on a map other than a client map
     * @throws ObjectGridException if an error occurs during processing this request
     *
     * @see ReplicationMapListener
     * @see #CONTINUOUS_REPLICATION
     * @see #SNAPSHOT_REPLICATION
     * @see #getReplicationMode()
     * @see BackingMap#CLIENT
     */
    void enableClientReplication(Mode mode, int[] partitions, ReplicationMapListener listener) throws ObjectGridException;

    /**
     * Returns the current replication mode
     *
     * @return the replication mode
     * @see #NONE
     * @see #CONTINUOUS_REPLICATION
     * @see #SNAPSHOT_REPLICATION
     */
    Mode getReplicationMode();

    /**
     * Disables the replication for this client. If it is not in
     * a replication mode, this method will be a no-op.
     * <p>
     * When security is enabled, this method requires a {@link com.ibm.websphere.objectgrid.security.ServerMapPermission}
     * with action "replicate". Refer to {@link com.ibm.websphere.objectgrid.security.ServerMapPermission} for more
     * permission details.
     *
     * @throws IllegalStateException if this method is invoked on a map other than a client map
     * @throws ObjectGridException if an error occurs during processing this request
     *
     * @see BackingMap#CLIENT
     */
    void disableClientReplication() throws ObjectGridException ;

}
