// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $HeadURL$
// $Id$
//
// Module  :  SessionHandleHelper.java
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import com.ibm.ws.objectgrid.SessionHandleImpl;
import com.ibm.ws.xs.util.TypeConversion;

/**
 * A helper class to import and export SessionHandle instances in different formats.
 * 
 * @since WAS XD 6.1.0.4
 */
public final class SessionHandleTransformer {
    /**
     * Converts the sessionHandle instance into a byte array.
     * 
     * @param sessionHandle
     *            The instance to be converted into a byte array.
     * @return The byte array holding the state of the session handle.
     */
    public static byte[] toByteArray(SessionHandle sessionHandle) {
        if (sessionHandle == null)
            throw new IllegalArgumentException("The SessionHandle argument must not be null.");
        byte[] bytes = new byte[8];
        TypeConversion.intToBytes(0, bytes, 0); // version
        final int partition = ((SessionHandleImpl) sessionHandle).getPartitionId();
        TypeConversion.intToBytes(partition, bytes, 4);
        return bytes;
    }

    /**
     * Converts the byte array representation of the handle back into instance form.
     * 
     * @param serializedHandle
     *            The byte array that was derived from {@link SessionHandleTransformer#fromByteArray(byte[])}.
     * @return A new SessionHandle instance.
     */
    public static SessionHandle fromByteArray(byte[] serializedHandle) {
        if (serializedHandle == null)
            throw new IllegalArgumentException("The byte[] argument must not be null.");
        if (serializedHandle.length < 8)
            throw new IllegalArgumentException("The byte[] argument did not contain enough information.");
        final int partition = TypeConversion.bytesToInt(serializedHandle, 4);
        return new SessionHandleImpl(partition);
    }

    /**
     * Converts the sessionHandle instance into a small stringified representation.
     * 
     * @param sessionHandle
     *            The instance to be stringified, this value should not be null.
     * @return The string representation.
     */
    public static String toString(SessionHandle sessionHandle) {
        if (sessionHandle == null)
            throw new IllegalArgumentException("The SessionHandle argument should not be null.");
        int partition = ((SessionHandleImpl) sessionHandle).getPartitionId();
        long encoded = TypeConversion.intsToLong(new int[] { 0, partition });
        String result = Long.toHexString(encoded);
        return result;
    }

    /**
     * Converts a stringified session handle back into a SessionHandle instance.
     * 
     * @param stringifiedHandle
     *            A string version of the session handle which was derived from
     *            {@link SessionHandleTransformer#toString(SessionHandle)}.
     * @return A new SessionHandle instance.
     */
    public static SessionHandle fromString(String stringifiedHandle) {
        if (stringifiedHandle == null)
            throw new IllegalArgumentException("The String argument should not be null.");
        long encoded = Long.parseLong(stringifiedHandle, 16);
        int partition = TypeConversion.longToInts(encoded)[1];

        return new SessionHandleImpl(partition);
    }

    /**
     * Writes the SessionHandle contents into the output stream. This method may be used with ObjectOutput as well since
     * it extends DataOutput.
     * 
     * @param oo
     *            The DataOutput stream that the sessionHandle instance will be written into, must not be null.
     * @param sessionHandle
     *            The session handle instance to write into the stream, must not be null.
     * @throws IOException
     */
    public static void writeSessionHandle(DataOutput oo, SessionHandle sessionHandle) throws IOException {
        if (oo == null)
            throw new IllegalArgumentException("The DataOutput argument must not be null.");
        if (sessionHandle == null)
            throw new IllegalArgumentException("The SessionHandle argument must not be null.");
        SessionHandleImpl impl = (SessionHandleImpl) sessionHandle;
        oo.writeByte(0); // version
        oo.writeInt(impl.getPartitionId());
    }

    /**
     * Reads and returns a SessionHandle instance from the contents of the input stream. This method may be used with
     * ObjectInput as well since it extends DataInput.
     * 
     * @param oi
     *            The DataInput stream that the session handle instance will be read from, must not be null.
     * @return A new SessionHandle instance instantiated up from the content of the stream.
     * @throws IOException
     */
    public static SessionHandle readSessionHandle(DataInput oi) throws IOException {
        if (oi == null)
            throw new IllegalArgumentException("The DataInput argument must not be null.");
        oi.readByte(); // version
        int partition = oi.readInt();

        SessionHandleImpl result = new SessionHandleImpl(partition);
        return result;
    }
}
