// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  QueryRelationship.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b834         6.1       04/27/2006   jian.tang Single query configuration
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.config;

import java.io.Serializable;

/**
 * A QueryRelationship represents a relationship between two BackingMap value classes.  A BackingMap
 * must have one class type defined in the value part of the map.  A relationship can be established
 * between two maps by mapping the source and target map's value classes.<p>
 * 
 * The cardinality of the relationship is automatically determined by the type of the attribute field.<p>
 *   
 * A relationship requires two classes, a relationship field, and optionally an inverse 
 * realtionship field. 
 * <p>
 * For example:
 * Two entities; Department and Employee, have the following bi-directional relationship:
 * <ol>
 * <li>One department has many employees. the collection field in the Department class is "emps"</li>
 * <li>An employee belongs to one department</li>
 * </ol>
 * 
 * <pre>
 * public class Department {
 *     private int id;
 *     private Collection emps;
 *     
 *     public void setEmps(Collection emps) {
 *         this.emps = emps;
 *     }
 *     
 *     public Collection getEmps() {
 *         return emps;
 *     }
 *     ...
 * }
 * 
 * public class Employee {
 *     private int id;
 *     private Department dept;
 *     
 *     public void setDept(Department dept) {
 *         this.dept = dept;
 *     }
 *     
 *     public Department getDept() {
 *         return dept;
 *     }
 * }
 * </pre>
 * Use the following method call to establish this bi-directional relationship.<p>
 *  
 * <pre>
 * queryConfig.addRelationship(new QueryRelationship(
 *     Department.class.getName(), Employee.class.getName(), "emps", "dept"));
 * </pre>
 * 
 * @since WAS XD 6.1, XC10
 *
 */
public class QueryRelationship implements Serializable {

    private static final long serialVersionUID = 1L;

    private String sourceClass;

    private String targetClass;

    private String relationshipField;

    /** optional */
    private String invRelationshipField;

    /**
     * 
     * Constructor for creating a QueryRelationship instance.<p> 
     * 
     * The sourceClass, targetClass, and relationshipField must not be null.
     * 
     * @param sourceClass the source class of the relationship
     * @param targetClass the target class of the relationship
     * @param relationshipField the attribute in the source class that references the key of the 
     * target class.
     * @param invRelationshipField the attribute in the target class that references the key of the 
     * source class.  This value is null if a bi-directional relationship does not exist.
     */
    public QueryRelationship(String sourceClass, String targetClass, String relationshipField,
            String invRelationshipField) {
        super();
        this.sourceClass = sourceClass;
        this.targetClass = targetClass;
        this.relationshipField = relationshipField;
        this.invRelationshipField = invRelationshipField;
        if (this.invRelationshipField != null && this.invRelationshipField.trim().length() == 0) {
            this.invRelationshipField = null;
        }
    }

    /**
     * Retrieve the inverse relationship attribute name of a bi-directional relationship.
     * @return the attribute name of the inverse side of a bi-directional relationship or null if
     * the relationship is uni-directional.
     */
    public String getInvRelationshipField() {
        return invRelationshipField;
    }

    /**
     * Set the inverse relationship attribute name of a bi-directional relationship.
     * @param invRelationshipField the attribute name of the inverse side of a bi-directional relationship or null if
     * the relationship is uni-directional.
     */
    public void setInvRelationshipField(String invRelationshipField) {
        this.invRelationshipField = invRelationshipField;
    }

    /**
     * Retrieve the name of the attribute in the source class that references the key of the target class.
     * @return the name of the relationship attribute.
     */
    public String getRelationshipField() {
        return relationshipField;
    }

    /**
     * Set the name of the attribute in the source class that references the key of the target class.
     * @param relationshipField the name of the relationship attribute.
     */
    public void setRelationshipField(String relationshipField) {
        this.relationshipField = relationshipField;
    }

    /**
     * Retrieve the name of the class representing the source of a relationship.
     * @return the source class
     */
    public String getSourceClass() {
        return sourceClass;
    }

    /**
     * Set the name of the class representing the source of a relationship.
     * @param sourceClass the source class
     */
    public void setSourceClass(String sourceClass) {
        this.sourceClass = sourceClass;
    }

    /**
     * Retrieve the name of the class representing the target of a relationship.
     * @return the target class
     */
    public String getTargetClass() {
        return targetClass;
    }

    /**
     * Set the name of the class representing the target of a relationship.
     * @param targetClass the target class
     */
    public void setTargetClass(String targetClass) {
        this.targetClass = targetClass;
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (o instanceof QueryRelationship) {
            QueryRelationship other = (QueryRelationship) o;

            return sourceClass.equals(other.sourceClass)
                    && targetClass.equals(other.targetClass)
                    && relationshipField.equals(other.relationshipField)
                    && (invRelationshipField == null ? other.invRelationshipField == null
                            : invRelationshipField.equals(other.invRelationshipField));
        }

        return false;

    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return sourceClass.hashCode() + targetClass.hashCode() + relationshipField.hashCode()
                + (invRelationshipField == null ? 0 : invRelationshipField.hashCode());
    }

    public String toString() {
        return super.toString() + "[" + sourceClass + ", " + targetClass + ", " + relationshipField
                + ", " + invRelationshipField + "]";
    }
}
