// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006-2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  EntityManager.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b834         6.1       05/22/2006   rajeshk   New part.
// b2404        6.1       10/19/2006   cheng1    disable isOpen() and close()
// b11518       6.1.0.3   04/08/2008   cdjohnson add invalidate support
// b11433       6.1.0.3   05/05/2008   cdjohnson EntityManager.merge() Feature
// b13153       6.1.0.5   10/15/2008   jian.tang Add Feature - FetchPlan
// ---------------------------------------------------------------------------------------

package com.ibm.websphere.objectgrid.em;

import com.ibm.websphere.objectgrid.security.ObjectGridPermission;
import com.ibm.websphere.projector.FetchPlan;
import com.ibm.websphere.projector.Projector;



/**
 * An EntityManager allows transactional introspection and manipulation of entities.<p>
 *
 * Each EntityManager is associated with an ObjectGrid <code>Session</code>.  Use
 * the {@link com.ibm.websphere.objectgrid.Session#getEntityManager()} method to retrieve the Session's
 * EntityManager instance.  Simultaneous interaction with the EntityManager and the owning
 * ObjectGrid Session is allowed.<p>
 *
 * All interactions with the EntityManager must be from a single thread.  Multi-thread access
 * to the EntityManager is not supported.<p>
 *
 * The EntityManager manages each entity within it's context scope.  Entity operations will
 * not be synchronized to the underlying ObjectGrid cache until the transaction is committed or the
 * EntityManager context is flushed to the cache.<p>
 *
 * Each entity is backed by an ObjectGrid map with the same name as the entity and utilizes the
 * inherent map configuration for locking, evicting, indexing and synchronizing with a loader.
 * See {@link com.ibm.websphere.objectgrid.BackingMap} for details on how entity a map can
 * be configured and optimized for each entity.<p>
 *
 * The EntityManager converts each entity instance to key and value Tuple instances using
 * the entity's persistent fields or properties and stores the Tuples in the entity's associated
 * ObjectMap.  The metadata that describes the Tuple can
 * be accessed using the {@link com.ibm.websphere.objectgrid.ObjectMap#getEntityMetadata()} or
 * {@link com.ibm.websphere.objectgrid.BackingMap#getEntityMetadata()} methods. <p>
 *
 * Tuples can be manually extracted from entities using the {@link Projector} which is obtained using the
 * {@link ProjectorFactory#getProjector(EntityManager)} factory method.<p>
 *
 * Accessing entities and Tuples simultaneously in a single transaction is not supported and may
 * cause unexpected results.
 *
 * @since WAS XD 6.1
 * @see com.ibm.websphere.projector.Tuple
 * @see com.ibm.websphere.projector.md.EntityMetadata
 */
public interface EntityManager {

    /**
     * Make an entity instance managed and persistent in the ObjectGrid cache.
     *
     * @param entity
     *            the entity object instance to make persistent.
     * @throws EntityExistsException
     *             if the entity already exists. (The EntityExistsException may be thrown when the persist operation is
     *             invoked, or the EntityExistsException or another PersistenceException may be thrown at flush or
     *             commit time.)
     * @throws IllegalArgumentException
     *             if not a valid entity
     * @throws TransactionRequiredException
     *             if invoked without an active transaction.
     * @see com.ibm.websphere.objectgrid.ObjectMap#insert(Object, Object)
     */
    public void persist(Object entity);

    /**
     * Remove the entity instance from the ObjectGrid.
     *
     * @param entity
     *            the managed entity instance to remove from the ObjectGrid cache.
     * @throws IllegalArgumentException
     *             if instance is not an entity or is a detached entity
     * @throws TransactionRequiredException
     *             if invoked without an active transaction.
     * @see com.ibm.websphere.objectgrid.ObjectMap#remove(Object)
     */
    public void remove(Object entity);

    /**
     * Invalidate the entity instance from the ObjectGrid cache, including
     * the transactional, client and server caches.<p>
     *
     * This method is similar to remove, but does not flow the changes to the entity's Loader.
     *
     * @param entity
     *            the managed entity instance to evict from the ObjectGrid cache.
     * @throws IllegalArgumentException
     *             if instance is not an entity or is a detached entity
     * @throws TransactionRequiredException
     *             if invoked without an active transaction.
     * @see com.ibm.websphere.objectgrid.ObjectMap#invalidate(Object, boolean)
     * @since WAS XD 6.1.0.3
     */
    public void invalidate(Object entity);

    /**
     * Find an entity by primary key in the ObjectGrid cache.
     *
     * @param entityClass
     *            the class representing the entity to find.
     * @param primaryKey
     *            an object instance representing the key of the entity to find.  The key may be a single
     *            Object (for single value keys), an IdClass or Entity instance (for composite keys)
     *            or a Tuple instance.
     * @return the entity instance or null if the entity does not exist
     * @throws IllegalArgumentException
     *             if the first argument does not denote an entity type or the second argument is not a valid type for
     *             that entity's primary key
     * @see com.ibm.websphere.projector.Tuple
     * @see com.ibm.websphere.objectgrid.ObjectMap#get(Object)
     */
    public Object find(Class entityClass, Object primaryKey);

    /**
     * Find an entity by primary key with the intent to be updated.<p>
     *
     * If the underlying map is configured with pessimistic locking strategy, an upgradeable
     * lock mode is obtained for this entity instance's map entry.
     * Any associated entities that are loaded are not locked for update.  If the associates
     * are need to be locked in the same manner, then <code>findForUpdate</code> should be
     * called for each associated entity.
     *
     * @param entityClass
     *            the class representing the entity to find.
     * @param primaryKey
     *            an object instance representing the key of the entity to find.
     * @return the found entity instance or null if the entity does not exist
     * @throws IllegalArgumentException
     *             if the first argument does not denote an entity type or the second argument is not a valid type for
     *             that entity's primary key
     * @see com.ibm.websphere.objectgrid.ObjectMap#getForUpdate(Object)
     */
    public Object findForUpdate(Class entityClass, Object primaryKey);

    /**
     * Synchronize the EntityManager to the ObjectGrid cache.<p>
     *
     * All managed entities in the current transaction are synchronized
     * with the ObjectGrid cache and the associated ObjectGrid session is
     * flushed.
     *
     * @throws TransactionRequiredException
     *             if there is no transaction
     * @throws PersistenceException
     *             if the flush fails
     * @see com.ibm.websphere.objectgrid.Session#flush()
     */
    public void flush();

    /**
     * Set the flush mode that applies to all objects contained
     * in the EntityManager's context.<p>
     *
     * If not set, the default is <tt>FlushModeType.AUTO</tt>.
     *
     * @param flushMode the FlushModeType.
     */
    public void setFlushMode(FlushModeType flushMode);

    /**
     * Get the flush mode that applies to all objects contained
     * in the EntityManager's context.
     * @return flushMode
     */
    public FlushModeType getFlushMode();

    /**
     * Return the resource-level transaction object. The EntityTransaction instance may be used serially to begin and
     * commit multiple transactions.<p>
     *
     * Each EntityManager is associated with one EntityTransaction.  Repeated calls to this method for
     * the same EntityManager instance will result in the same EntityTransaction instance.
     *
     * @return the EntityTransaction instance
     */
    public EntityTransaction getTransaction();

    /**
     * Create a Query instance for executing an ObjectGrid query language statement.
     * <p>
     * When security is enabled, this method requires an com.ibm.websphere.objectgrid.security.ObjectGridPermission
     * with action "query".<p>
     *
     * Required Permission: {@link ObjectGridPermission#QUERY}
     *
     * @param qlString
     *            an ObjectGrid query string
     * @return the new query instance
     * @throws IllegalArgumentException
     *             if query string is not valid
     */
    public Query createQuery(String qlString);

    /**
     * Clear the EntityManager's context.<p>
     *
     * Causes all managed entities to become detached.
     * Any changes made to entities that
     * have not been flushed to the cache will not be persisted.
     */
    public void clear();

    /**
     * Create a QueryQueue for an entity map.
     * <p>
     * An entity QueryQueue will select all the entities which match the query filter's <TT>WHERE</TT>
     * condition. Users can call {@link QueryQueue#getNextEntity(long)} or
     * {@link QueryQueue#getNextEntities(int, long)} to get one or more entities at one time.
     * <p>
     * A QueryQueue is useful for retrieving select entities in an iterative manner.
     * <p>
     * The query string's <TT>SELECT</TT> clause must specify one and only one entity
     * identification variable and no other functions or attributes.<p>
     * For example:
     * <pre>
     * "SELECT t FROM Tasks t WHERE t.type=?1 AND t.status='PENDING' ORDER BY t.createTime"
     * </pre>
     * <p>
     * An entity class can be optionally specified to indicate which class the result(s) will be projected
     * into. The specified entity class must have the same entity name as specified in the query string.
     * If a null value is used as the entity class, then the result will not be projected; the key tuple(s)
     * will be returned by the{@link QueryQueue#getNextEntity(long)} or
     * {@link QueryQueue#getNextEntities(int, long)} methods.
     *
     * @param qlString the query SQL string
     * @param entityClass the entity class the result will be projected to; null if the results
     *        are not projected.
     * @return a {@link QueryQueue} instance
     * @see QueryQueue
     * @since WAS XD 6.1 FIX3
     */
    public QueryQueue createQueryQueue(String qlString, Class entityClass);

    /**
     * Merge the entity instance with the ObjectGrid.  The entity to merge can be a new, detached or managed entity.
     * Once merged, the input entity will remain in it's original state is not managed.  The entity returned by
     * this method is the managed result of the merge operation.
     *
     * @param entity
     *            the managed, new or detached entity instance to merge with the ObjectGrid cache.
     * @return a managed entity representing the result of the merge.
     * @throws IllegalArgumentException
     *             if instance is not an entity or a removed entity
     * @throws TransactionRequiredException
     *             if invoked without an active transaction.
     * @throws OptimisticLockException
     *             if there is a mismatch between the version of the entity and a managed entity.  This exception may
     *             not be thrown until commit or flush time.
     * @since WAS XD 6.1.0.3
     */
    public Object merge(Object entity);

    /**
     * Get the current mutable FetchPlan.
     * <p>
     * A FetchPlan is used to customize how the eager relations are fetched with the fetch operation.
     * A fetch operation could be any of the following: {@link #find(Class, Object)}, {@link #findForUpdate(Class, Object)},
     * {@link Query} operations, or {@link QueryQueue} operations.
     * <p>
     * A FetchPlan object is normally used on the client side to access for a distributed ObjectGrid. However,
     * when used with a local ObjectGrid, the FetchPlan object can be used to customize the relations to be
     * projected when the fetch operation is executed.
     * <p>
     * The FetchPlan object is mutable, and once changed, the changed value will be applied to the
     * fetch operations executed afterwards.
     *
     * @return the current FetchPlan object
     * @since WAS XD 6.1.0.5
     */
    FetchPlan getFetchPlan();
}
