// IBM Confidential OCO Source Material
// 5733-W70 (C) COPYRIGHT International Business Machines Corp. 2008, 2010
//
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  ObjectGridHibernateCacheProvider.java
//
// Source File Description:
//
// This class is Hibernate CacheProvider implementation and will be used by user to be specified in persistence.xml
// or hibernate.cfg.xml.
//   Start() - will be invoked by hibernate to prepare cache.
//   buildCache() - will be called per regionName. A regionName should mapeed to a BackingMap.
//                  However, it is invoked on demand, even after ObjectGrid has been initialized.
//                  Should create BackingMap per known regionName before ObjectGrid initialization, and
//                  create a defaultCacheMap to handle all regionNames known after ObjectGrid initlaization.
//                  This method should return a hibernate Cache instance back to hibernate. Hibernate will invoke on
//                  the returned Cache instance for corresponding regionName.
//                  In summary, for each regionName, should have a Cache instance that is mapped to BackingMap.
//
// Ideally, should put off ObjectGrid initialization as late as possible in order to know most regionNames.
// The ObjectGrid should be initialized when hibernate use the cache at the first time.
// If the hibernate never uses the cache, then the gird won't be initialized.
//
// During ObjectGrid initialization, create configurations based on all known regionNames plus defaultCacheMap.
//
// Change Activity:
//
// Reason          Version   Date     Userid    Change Description
// --------------- --------- -------- --------- --------------------------------
// 11400           XD61x     20080423 cheng1    new file
// b15969          7.0       20090430 cheng1    improve performance and enhance cache statistics
// b19719          7.1       20100503 cheng1    write behind support
// --------------- --------- -------- --------- --------------------------------

package com.ibm.websphere.objectgrid.hibernate.cache;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.hibernate.cache.Cache;
import org.hibernate.cache.CacheException;
import org.hibernate.cache.CacheProvider;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.objectgrid.hibernate.cache.CacheProviderMode;
import com.ibm.ws.objectgrid.hibernate.cache.CommonHibernateCacheProvider;
import com.ibm.ws.objectgrid.hibernate.cache.ObjectGridHibernateCache;
import com.ibm.ws.objectgrid.jpa.cache.common.JPAPMIModule;
import com.ibm.ws.xs.NLSConstants;
import com.ibm.ws.xs.util.MethodHelper;
import com.ibm.ws.xs.util.dopriv.DoPrivUtil;

/**
 * The ObjectGridHibernateCacheProvider is the implementation of Hibernate CacheProvider. Users can enable ObjectGrid as
 * level 2 cache for Hibernate by configuring cache provider property in the Hibernate configuration file.
 * <P>
 * If using Hibernate as a JPA provider, the cache provider property name in <tt>persistence.xml</tt> is
 * <tt>hibernate.cache.provider_class</tt>:
 * <pre>
 * &lt;property
 *   name="hibernate.cache.provider_class"
 *   value="com.ibm.websphere.objectgrid.hibernate.cache.ObjectGridHibernateCacheProvider" /&gt;
 * </pre>
 *
 * <P>
 * If using the traditional Hibernate configuration, the cache provider property name is <tt>cache.provider_class</tt>:
 * <pre>
 * &lt;property
 *   name="cache.provider_class"&gt;
 *     com.ibm.websphere.objectgrid.hibernate.cache.ObjectGridHibernateCacheProvider
 * &lt;/property&gt;
 * </pre>
 *
 * <P>
 * The following additional properties apply to the ObjectGrid:
 * <ol>
 * <li><tt><strong>objectgrid.configuration</strong></tt>: Comma-delimited ObjectGrid configuration properties.
 *      <ul>
 *      <li> ObjectGridName: the uniquely identified ObjectGrid name of the ObjectGrid to use.
 *      <li> ObjectGridType: (EMBEDDED | EMBEDDED_PARTITION | REMOTE) the type of ObjectGrid.
 *              <ul>
 *              <li> EMBEDDED: This is the default ObjectGrid type. There will be only 1 partition with SYNC replica mode and the maximum number of repicas set to 47 by default.
 *                             With the EMBEDDED type, users can specify the ReplicaMode and MaxNumberOfReplicas to determine the type of replication and
 *                             the maximum number of replicas. The ReplicaReadEnabled will be always true.
 *                             If a system has more than 47 JVMs, users should set the MaxNumberOfReplicas to be equal to the number rof JVMs.
 *                             This EMBEDDED mode has two limitations:
 *                             <ul>
 *                             <li>limited cache size: because there is only one partition so the cahce size is limited to the heap size of the JVM.
 *                                 If the system need to cache 100GB, it should use EMBEDDED_PARTITION type.
 *                             <li>All write opeations will go to one JVM.
 *                             </ul>
 *              <li> EMBEDDED_PARTITION: This type should be used when the system would like to cache large amount of data.
 *                   It is default to have 47 partitions with NONE replica mode. In a small system that has only few JVMs, users should set the NumberOfPartitions to be the number of JVMs.
 *                   Users can specify ReplicaMode, NumberOfPartitions, MaxNumberOfReplicas, and ReplicaReadEnabled to tune the system
 *              <li> REMOTE: With this REMOTE type, the cache simply try to obtain a distributed ObjectGrid from catalog service. Users should place objectGridServer.properties file in classpath.
 *                   The objectGridServer.properties should have the catalogServiceEndPoints property. It is users' responsibility to setup the remote ObjectGrid system.
 *              </ul>
 *      <li> NumberOfPartitions: (>=1) the number of partitions to be used for the cache. The default value is 47 for EMBEDDED-PARTITION. Only apply on EMBEDDED-PARTITION ObjectGrid type.
 *      <li> ReplicaMode: (SYNC/ASYNC/NONE; not case sensitive) how cache is replicated to replica. The default value is NONE. Only apply on EMBEDDED and EMBEDDED-PARTITION ObjectGrid type.
 *      <li> ReplicaReadEnabled: (TRUE/FALSE; not case sensitive) to enable reading data from replica or not. The default value is FALSE. Only apply on EMBEDDED_PARTITION ObjectGrid type.
 *      <li> MaxUsedMemory: (TRUE/FALSE; not case sensitive) to enable memory-based evictors on maps. The default value is TRUE.
 *      <li> MaxNumberOfReplicas: (>=1) the maximum number of replicas to be used for the cache. For EMBEDDED ObjectGrid type, this number should be equal or greater than the number of JVMs in a system.
 *
 *      </ul>
 * <li><tt><strong>objectgrid.hibernate.regionNames</strong></tt>: Comma-delimited Hibernate region names in use. This is optional property and should be specified when regionNames are defined after ObjectGrid cache is initialized.
 * For example, if the Query.setCacheRegion(String regionName) method is executed after ObjectGrid cache initialization. This regionName could be a new regionName to cache and there won't be a corresponding BackingMap for it.
 * To have better performance, these regionNames should be included in this property. If not, all regionNames that are determined dynamically will use a shared default BackingMap.
 *
 * </ol>
 *
 * <P>
 * The following is a configuration example of JPA persistence.xml file.
 * <pre>
 * &lt;property
 *   name="hibernate.cache.provider_class"
 *   value="com.ibm.websphere.objectgrid.hibernate.cache.ObjectGridHibernateCacheProvider" /&gt;
 * &lt;property
 *   name="hibernate.cache.use_query_cache" value="true" /&gt;
 * &lt;property
 *   name="objectgrid.configuration" value="ObjectGridName=MyObjectGridName, ObjectGridType=EMBEDDED , MaxNumberOfReplicas=47" /&gt;
 * &lt;property
 *   name="objectgrid.hibernate.regionNames" value="queryRegion1, queryRegion2" />
 *
 * </pre>
 *
 * @since WAS XD 6.1.0.3
 *
 */
public final class ObjectGridHibernateCacheProvider extends CommonHibernateCacheProvider implements CacheProvider {
	private static final String CLASS_NAME = ObjectGridHibernateCacheProvider.class.getName();

    private static final TraceComponent tc = Tr.register(CLASS_NAME, Constants.TR_GROUP_NAME, 
            Constants.TR_RESOURCE_BUNDLE_NAME);
    
    private final Map cacheList = new HashMap();
    

    /**
     * @see org.hibernate.cache.CacheProvider#buildCache(java.lang.String, java.util.Properties)
     */
    public synchronized Cache buildCache(String regionName, Properties props) throws CacheException {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "buildCache", regionName);
        }

        // Entity class that relies on annotation need to annotated with @Cache so teh buildCache() will be called for
        // that entity. The default entity name is the class name, unless specified with @Entity(name="") attribute.
        //
        // The regionName could be entity class name or identity
        // hibernate will call this method for each regionName before or after using the cache
        // that means even after ObjectGrid has initialized, this method could be called.
        // What must be done:
        // 1. cache all regionNames as many as possible for creating BackingMaps during ObjectGrid initialization.
        // 2. The ObjectGrid should have a defaultCacheMap to hanlde regionName that is known afer initialization.
        // if the hibernate never uses the cache, then the gird won't be initialized.
        addRegionName(regionName);

        Cache rc = (Cache) cacheList.get(regionName);
        if (rc != null) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "buildCache", rc);
            }
            return rc;
        }

        ObjectGridHibernateCache cache = new ObjectGridHibernateCache(this, getObjectGrid(), regionName);
        if(isWriteBehind()) {
            // the agentHelper is created in start method.
            cache.setAgentOperator(getAgentOperator());
            cache.setMaxBatchSize(getWriteBehindMaxBatchSize());
            cache.setBatchPut(isWriteBehind());
        }

        synchronized (cacheList) {
            cacheList.put(regionName, cache);
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "buildCache", cache);
        }

        return cache;

    }
    
    /**
     * Overrides
     */
    protected void setPreLoadMode(boolean preLoadMode) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "set PreLoad Mode to " + preLoadMode + " for the PU [" + getPuName() + "] with ObjectGrid [" + getObjectGridName() + "]");
        }
        //System.out.println("ObjectGridHibernateCacheProvider.setPreLoadMode(): " + preLoadMode);
        CacheProviderMode.setMode(preLoadMode);
        if (preLoadMode == false) {
            // When setPreLoadMode(false), it is the time to flush preload batch map to ObjectGrid.
            // if the objectgrid is not initialized, then this will also trigger og initialization.
            // System.out.println("cacheList size:" + cacheList.size());
            Iterator iter = cacheList.values().iterator();
            while (iter.hasNext()) {
                ObjectGridHibernateCache cache = (ObjectGridHibernateCache) iter.next();
                //System.out.println("cache region name: " + cache.getRegionName());
                cache.flushPreLoadIfNeeded(null, null, 0);
            }
        }
    }
    
    /**
     * Overrides
     */
    protected void processWriteBehind(boolean checkSize){
        super.processWriteBehind(checkSize); // increments the counter
        
        if(getObjectGrid() == null) {
            // the objectGrid is not initialized yet.
            return;
        }

        try {
            final List currentCacheList = new ArrayList();
            synchronized (cacheList) {
                currentCacheList.addAll(cacheList.values());
            }
            int cacheListSize = currentCacheList.size();
            for(int i=0; i< cacheListSize; i++) {
                ObjectGridHibernateCache cache = (ObjectGridHibernateCache)currentCacheList.get(i);
                try {
                    cache.processBatchPutMap(checkSize);
                } catch (Throwable t) {
                    // during processing batch, can not throw error to caller.
                    // ffdc the error for debug.
                    FFDCFilter.processException(t, CLASS_NAME + ".processWriteBehind", "1096", this);
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isWarningEnabled()) {
                        Tr.warning(tc, NLSConstants.WRITE_BEHIND_FAILED_CWOBJ5050, t);
                    }
                }
            }
        } catch (Throwable t) {
            FFDCFilter.processException(t, CLASS_NAME + ".processWriteBehind", "1103", this);
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isWarningEnabled()) {
                Tr.warning(tc, NLSConstants.WRITE_BEHIND_FAILED_CWOBJ5050, t);
            }
        }
    }
    
    /**
     * Note:  This method is accessed reflection and used for testing and debugging.  Do not remove.
     */
    protected String getCacheStatistics() {
        StringBuilder buffer = new StringBuilder();
        buffer.append(super.getCacheStatistics());

        long totalGet = 0;
        //long totalGetAll = 0;
        long totalPut = 0;
        long totalRemove = 0;
        long totalClear = 0;
        //long totalCommit = 0;
        long totalPutAll = 0;

        try {
            Method m = MethodHelper.getDeclaredMethod(ObjectGridHibernateCache.class, "getJPAPMIModule", new Class[] {});
            DoPrivUtil.setMethodAccessible(m, true);

            if (m != null) {
                Iterator iter = cacheList.values().iterator();
                while (iter.hasNext()) {
                    ObjectGridHibernateCache cache = (ObjectGridHibernateCache) iter.next();
                    JPAPMIModule pmiModule = (JPAPMIModule)m.invoke(cache, new Object[]{});
                    buffer.append(pmiModule.toString());

                    totalGet = totalGet + pmiModule.counterGet;
                    //totalGetAll = totalGetAll + pmiModule.counterGetAll;
                    totalPut = totalPut + pmiModule.counterPut;
                    totalRemove = totalRemove + pmiModule.counterRemove;
                    totalClear = totalClear + pmiModule.counterClear;
                    //totalCommit = totalCommit + pmiModule.counterCommit;
                    totalPutAll = totalPutAll + pmiModule.counterPutAll;
                }
            } else {
                buffer.append("no such method");
            }

        } catch (Exception e) {
            e.printStackTrace(System.out);
            buffer.append("Exception: " + e);
        }

        buffer.append(lineBreak);
        buffer.append("-------------------------------------------------" + lineBreak);
        buffer.append("totalGet    : " + totalGet + lineBreak);
        //buffer.append("totalGetAll : " + totalGetAll + lineBreak);
        buffer.append("totalPut    : " + totalPut + lineBreak);
        buffer.append("totalRemove : " + totalRemove + lineBreak);
        buffer.append("totalClear  : " + totalClear + lineBreak);
        //buffer.append("totalCommit : " + totalCommit + lineBreak);
        buffer.append("totalPutAll : " + totalPutAll + lineBreak);
        buffer.append("-------------------------------------------------" + lineBreak);
        if(isWriteBehind()) {
            buffer.append("writeBehindPoolSize=" + getWriteBehindPoolSize());
            buffer.append(", writeBehindInterval=" + getWriteBehindInterval());
            buffer.append(", writeBehindMaxBatchSize=" + getWriteBehindMaxBatchSize() + lineBreak);
            buffer.append("counterProcessWriteBehind : " + getCounterProcessWriteBehind() + lineBreak);
            buffer.append("-------------------------------------------------" + lineBreak);
        }
        return buffer.toString();
    }
}
