/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 */

package com.ibm.websphere.objectgrid.management;


/**
 * This MBean interface allows a client process to perform operations on and get status from an ObjectGrid container
 * running in a dynamic environment. The object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=ObjectGridContainer,name=&lt;server&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since WAS XD 6.1 FIX3, XC10
 */

public interface ContainerMBean {
    /**
     * QUIESCE_COMPLETE is the MBean notification type for a completed quiesce.
     * @see #quiesceContainer(Boolean)
     */
    public static final String QUIESCE_COMPLETE = "og.container.quiesce.complete";

    /**
     * RESERVATION_SUCCESSFUL indicates that the attempt to reserve the shard was successful. The shard is reserved by
     * the requesting container.
     *
     * @see #reserve(String, String, String, String)
     * @since 7.0.0.0 FIX1
     */
    public final static String RESERVATION_SUCCESSFUL = "RESERVATION_SUCCESSFUL";

    /**
     * RESERVATION_PRIOR_TO_INITIAL_PLACEMENT indicates that the attempt to reserve the shard was processed
     * successfully. However, since initial placement has not yet occurred, the reserved shard is not immediately moved
     * to the requesting container. The shard will be placed on the container when initial placement is triggered.
     *
     * @see #reserve(String, String, String, String)
     * @since 7.0.0.0 FIX1
     */
    public final static String RESERVATION_PRIOR_TO_INITIAL_PLACEMENT = "RESERVATION_PRIOR_TO_INITIAL_PLACEMENT";

    /**
     * SHARD_ALREADY_RESERVED indicates that the shard is already reserved elsewhere and cannot be reserved on the
     * specified container. The shard must be released from the owning container before it can be reserved again.
     *
     * @see #reserve(String, String, String, String)
     * @since 7.0.0.0 FIX1
     */
    public final static String SHARD_ALREADY_RESERVED = "SHARD_ALREADY_RESERVED";

    /**
     * INVALID_PARTITION indicates that no partition was found for the requested shard.
     *
     * @see #reserve(String, String, String, String)
     * @since 7.0.0.0 FIX1
     */
    public final static String INVALID_PARTITION = "INVALID_PARTITION";

    /**
     * RESERVE_UNSUPPORTED_WITH_PER_CONTAINER indicates that the shard is part of a map set using the PER_CONTAINER
     * placement strategy. Shard reservation is not supported with this placement strategy.
     *
     * @see #reserve(String, String, String, String)
     * @since 7.0.0.0 FIX1
     */
    public final static String RESERVE_UNSUPPORTED_WITH_PER_CONTAINER = "RESERVE_UNSUPPORTED_WITH_PER_CONTAINER";

    /**
     * RELEASE_SUCCESSFUL indicates that the attempt to release the shard was successful. The shard is no longer
     * reserved by this container. The shard is free to migrate, but it is not forced to migrate.
     *
     * @see #release(String, String, String)
     * @since 7.0.0.0 FIX1
     */
    public final static String RELEASE_SUCCESSFUL = "RELEASE_SUCCESSFUL";

    /**
     * SHARD_NOT_RESERVED_ON_CONTAINER indicates that the attempt to release the shard from the requesting container
     * failed because the specified shard was not found to be reserved by the requesting container. Only the container
     * owning the reservation may release a shard.
     *
     * @see #release(String, String, String)
     * @since 7.0.0.0 FIX1
     */
    public final static String SHARD_NOT_RESERVED_ON_CONTAINER = "SHARD_NOT_RESERVED_ON_CONTAINER";

    /**
     * RELEASE_UNSUPPORTED_WITH_PER_CONTAINER indicates that the shard is part of a map set using the PER_CONTAINER
     * placement strategy. Shard release is not supported with this placement strategy.
     *
     * @see #release(String, String, String)
     * @since 7.0.0.0 FIX1
     */
    public final static String RELEASE_UNSUPPORTED_WITH_PER_CONTAINER = "RELEASE_UNSUPPORTED_WITH_PER_CONTAINER";

    /**
     * MAPSET_UNSUPPORTED_ON_CONTAINER indicates that an attempt was made to reserve a shard from a map set
     * that is not supported on this container.  Only map sets that were included in the deployment policy
     * at container initialization are supported to run on this container.
     *
     * @see #reserve(String, String, String, String)
     * @since 7.1
     */
    public final static String MAPSET_UNSUPPORTED_ON_CONTAINER = "MAPSET_UNSUPPORTED_ON_CONTAINER";

    /**
     * Tears down and stops the container in a way to allow partitions to be moved to new locations.
     */
    public void teardown();

    /**
     * Terminates a container without coordinating partition movement, partitions will failover.
     */
    public void terminate();

    /**
     * Retrieve the number of active shards hosted in this ObjectGrid container.
     *
     * @return The current number of active shards.
     */
    public int getActiveShardCount();

    /**
     * Retrieve the total number of shards that have been activated for the life of this ObjectGrid container.
     *
     * @return The number of activated shards.
     */
    public int getActivatedShardCount();

    /**
     * Retrieve the total number of shards that have been deactivated for the life of this ObjectGrid container.
     *
     * @return The number of deactivated shards
     */
    public int getDeactivatedShardCount();

    /**
     * Retrieve the name of the catalog server grouping administering this container.
     *
     * @return The domain name.
     */
    public String getDomainName();

    /**
     * Retrieve the name of the zone grouping that this container belongs to.
     *
     * @return The name of the zone.
     */
    public String getZoneName();

    /**
     * Prepare the container for a potential shutdown by moving replica shards, verifying that primaries have required
     * sync replicas and preventing the placement of new shards.
     *
     * @param inQuiesce
     *            Initiate quiesce mode (true) or cancel quiesce mode (false)
     * @return The number of replicas moved off of the ObjectGrid container
     * @see #QUIESCE_COMPLETE
     */
    public int quiesceContainer(Boolean inQuiesce);

    /**
     * Retrieve the status information for the shards in this container.
     *
     * @return The status information for the shards in this container.
     */
    public String getStatus();

    /**
     * Retrieve the status information for the shards in this container, filtered by ObjectGrid and/or mapset. For
     * example, calling retrieveStatus with "og1" and "ms1" as parameters will return the partition status for those
     * partitions in ObjectGrid og1 and mapset ms1. Passing in an empty string ("") objectGridName or mapSetName will return all of
     * the partitions, since the empty string acts as a wildcard. Passing in the empty string for both parameters will return the same status as
     * calling getStatus().
     *
     * @param objectGridName
     *            The name of the ObjectGrid for which the status is requested.
     * @param mapSetName
     *            The name of the mapset within the ObjectGrid for which the status is requested.
     * @return The status information for the shards in this container.
     */
    public String retrieveStatus(String objectGridName, String mapSetName);


    /**
     * Reserve a specific shard on this container. Calling this method will cause the requested shard to move to this
     * container. The shard can be moved to this container only if it is not reserved elsewhere. Calling this method
     * prior to initial placement will pre-reserve the shard so that it will be placed onto this container when initial
     * placement occurs.
     *
     * If non-reserved shard for the same partition is on this container prior to reservation, the non-reserved shard
     * will be moved off the container upon reservation.
     *
     * A reserved shard will not be moved off of this container until it is released or the container is stopped.
     *
     * @param objectGridName
     *            the ObjectGrid containing the shard
     * @param mapSetName
     *            the map set containing the shard
     * @param partitionName
     *            the partition containing the shard
     * @param shardType
     *            the type of shard. Currently, only primary shards can be reserved:
     *            {@link com.ibm.websphere.objectgrid.management.ShardMBean#TYPE_PRIMARY}
     *
     * @return the return code indicating the result of the reserve request
     *
     * @throws IllegalArgumentException
     *             if any of the arguments are null or the empty String. Also thrown if shardType is not
     *             {@link com.ibm.websphere.objectgrid.management.ShardMBean#TYPE_PRIMARY}
     *
     *
     * @see ShardMBean#TYPE_PRIMARY
     * @see #release(String, String, String)
     * @see #RESERVATION_SUCCESSFUL
     * @see #RESERVATION_PRIOR_TO_INITIAL_PLACEMENT
     * @see #SHARD_ALREADY_RESERVED
     * @see #INVALID_PARTITION
     * @see #RESERVE_UNSUPPORTED_WITH_PER_CONTAINER
     *
     * @since 7.0.0.0 FIX1
     */
    public String reserve(String objectGridName, String mapSetName, String partitionName, String shardType);

    /**
     * Release a shard that has been previously reserved by this container. This container can only release shards that
     * it has reserved.
     *
     * Releasing the shard does not guarantee the shard will be moved. The shard may remain on this container. However,
     * it will not be explicitly bound to this container. Releasing a shard allows the shard to move freely to other
     * containers or to be reserved by another container.
     *
     *
     * @param objectGridName
     *            the ObjectGrid containing the shard
     * @param mapSetName
     *            the map set containing the shard
     * @param partitionName
     *            the partition containing the shard
     *
     * @return the return code indicating the result of the release request
     *
     * @throws IllegalArgumentException
     *             if any of the arguments are null or the empty String
     *
     *
     * @see #reserve(String, String, String, String)
     * @see #RELEASE_SUCCESSFUL
     * @see #SHARD_NOT_RESERVED_ON_CONTAINER
     * @see #RELEASE_UNSUPPORTED_WITH_PER_CONTAINER
     * @since 7.0.0.0 FIX1
     */
    public String release(String objectGridName, String mapSetName, String partitionName);

    /**
     * Retrieve the name of the container.  The container name is based on the server name and includes a
     * suffix which uniquely identifies the container within the server.
     *
     * @return the name of the container
     * @since 7.1
     */
    public String getContainerName();
}
