/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ ---------- ------------ --------- ---------------------------------------
 * b20788       7.1 fix1   07/16/2010   vandersn  swap role with primary 
 */

package com.ibm.websphere.objectgrid.management;

/**
 * This MBean interface allows a client process to perform operations on and
 * get status from a shard running in a dynamic environment. The object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=Shard,name=&lt;objectgrid&gt;,objectgrid=&lt;objectgrid&gt;,mapset=&lt;mapset&gt;,partition=&lt;partition id&gt;,container=&lt;container&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since WAS XD 6.1 FIX3, XC10
 */
public interface ShardMBean {
    
    /**
     * Indicates that the role swap was executed successfully.
     * 
     * @see #swapWithPrimary()
     * @since 7.1.0.0 FIX1
     */
    public static final String ROLE_SWAP_SUCCESSFUL = "ROLE_SWAP_SUCCESSFUL";
    
    /**
     * Indicates that this shard is the same type of shard as the requested swap type.  No swap will be executed.
     * 
     * @see #swapWithPrimary()
     * @since 7.1.0.0 FIX1
     */
    public static final String ROLE_SWAP_REQUESTED_WITH_SAME_TYPE = "ROLE_SWAP_REQUESTED_WITH_SAME_TYPE";
    
    /**
     * Indicates that this shard has timed out waiting to inherit its requested role
     * @see #swapWithPrimary()
     * @since 7.1.0.0 FIX1 
     */
    public static final String ROLE_SWAP_TIMEOUT = "ROLE_SWAP_TIMEOUT";

    /**
     * Retrieve the name of the ObjectGrid in which the shard resides.
     * @return The ObjectGrid name.
     */
    public String getObjectGridName();

    /**
     * Retrieve the name of the MapSet in which the shard resides.
     * @return The MapSet name.
     */
    public String getMapSetName();

    /**
     * Retreive the name of the partition in which the shard resides.
     * @return The partition name.
     */
    public String getPartitionName();

    /**
     * Retrieve the type of the shard.
     * @return The shard type.
     */
    public String getType();

    /**
     * Retrieve the name of the catalog server grouping administering this shard.
     * @return The domain name.
     */
    public String getDomainName();

    /**
     * Retrieve the state of the shard.
     * @return The shard state.
     */
    public String getState();

    /**
     * Indicates the shard type is the primary role. This means that this is the shard that handles all updates and
     * coordinates state transitions with the replicas.
     */
    public static final String TYPE_PRIMARY = "Primary";

    /**
     * Indicates the shard type is the synchronous replica role. This means that this shard is receiving state updates
     * from another shard that is acting as primary.
     */
    public static final String TYPE_REPLICA_SYNCHRONOUS = "SynchronousReplica";

    /**
     * Indicates the shard type is the asynchronous replica role. This means that this shard is receiving state updates
     * from another shard that is acting as primary.
     */
    public static final String TYPE_REPLICA_ASYNCHRONOUS = "AsynchronousReplica";

    /**
     * Indicates the shard type is the inactive role. This means that this shard is not actively enrolled in the
     * partition.
     */
    public static final String TYPE_INACTIVE = "Inactive";

    /**
     * Retrieves the number of requests that this shard has processed or forwarded since its inception.
     *
     * @return A count of the total number of requests.
     */
    public long getTotalRequestCount();

    /**
     * Retrieves the number of requests currently being processed by this shard.
     *
     * @return A count of the active requests.
     */
    public long getActiveRequestCount();

    /**
     * Retrieves the number of requests that this shard has forwarded since its inception.
     *
     * @return A count of the total number of forwarded requests.
     */
    public long getForwardedRequestCount();

    /**
     * Retrieves the number of requests that this shard has processed since its inception.
     *
     * @return A count of the total number of processed requests.
     */
    public long getProcessedRequestCount();

    /**
     * Retrieves the name of the container that is hosting this shard.
     *
     * @return The name of the container.
     * @since WAS XD 6.1.0.3
     */
    public String getContainerName();
    
    /**
     * Causes this shard to swap roles with the primary shard for the partition.  This shard becomes the primary
     * while the shard that was previously the primary inherits this shard's former role. <p>
     * 
     * If the role swap is not complete within 10 seconds, this operation will timeout.
     * 
     * @return String the contains the return code of the operation
     * @see #ROLE_SWAP_SUCCESSFUL
     * @see #ROLE_SWAP_REQUESTED_WITH_SAME_TYPE
     * @see #ROLE_SWAP_TIMEOUT
     * @since 7.1.0.0 FIX1
     */
    public String swapWithPrimary();
}
