// IBM Confidential OCO Source Material
// 5733-W70 (C) COPYRIGHT International Business Machines Corp. 2007, 2010
//
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  ObjectGridDataCache.java
//
// Source File Description:
//
//   The OpenJPA plugin class for data cache using ObjectGrid
//
// Change Activity:
//
// Reason          Version   Date     Userid    Change Description
// --------------- --------- -------- --------- -------------------------------
//                 FeP       20070620 dtlee     new file
// b15969          7.0       20090430 cheng1    improve performance and enhance cache statistics
// b18920          7.1       20100409 cheng1    batching put and remove support
// b22644          7.1.x     20110405 cheng1    handle null key in remove method.
// --------------- --------- -------- --------- -------------------------------
package com.ibm.websphere.objectgrid.openjpa;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.openjpa.conf.OpenJPAConfiguration;
import org.apache.openjpa.datacache.AbstractDataCache;
import org.apache.openjpa.datacache.DataCacheManager;
import org.apache.openjpa.datacache.DataCachePCData;
import org.apache.openjpa.event.RemoteCommitListener;
import org.apache.openjpa.lib.log.Log;
import org.apache.openjpa.util.InternalException;
import org.apache.openjpa.util.OpenJPAId;

import com.ibm.websphere.objectgrid.ObjectGridException;
import com.ibm.websphere.objectgrid.TransactionException;
import com.ibm.ws.objectgrid.jpa.cache.common.JPAPMIModule;
import com.ibm.ws.objectgrid.openjpa.ObjectGridCacheMap;
import com.ibm.ws.objectgrid.openjpa.ObjectGridPCDataImpl;
import com.ibm.ws.objectgrid.openjpa.ObjectGridPCKeyImpl;
import com.ibm.ws.objectgrid.openjpa.ObjectGridResourceFactory;
import com.ibm.ws.objectgrid.openjpa.OpenJPAConstants;

/**
 * A {@link org.apache.openjpa.datacache.DataCache} implementation is a
 * DataCache plugin to integrate Open JPA with ObjectGrid for leveraging
 * the cluster server advantages provided by ObjectGrid.
 *
 * <P>
 * ObjectGrid cache is enabled / disable for OpenJPA by setting the openjpa.DataCache and openjpa.QueryCache
 * configuration properties in persistence.xml. The syntax for setting the property is as the following:
 * <pre>
 * &lt;property
 *   name="openjpa.DataCache"
 *   value="com.ibm.websphere.objectgrid.openjpa.ObjectGridDataCache(&lt;property&gt;=&lt;value&gt;,...)" /&gt;
 * &lt;property
 *   name="openjpa.QueryCache"
 *   value="com.ibm.websphere.objectgrid.openjpa.ObjectGridQueryCache(&lt;property&gt;=&lt;value&gt;,...)" /&gt;
 * </pre>
 *
 * <P>
 * If both DataCache and QueryCache are enabled, the QueryCache will use the same configuration as DataCache, so the QueryCache does not need to have any property.
 *
 * <p>
 * The following ObjectGrid related properties can apply to both DataCache and QueryCache:
 *      <ul>
 *      <li> ObjectGridName: the uniquely identified ObjectGrid name of the ObjectGrid to use.
 *      <li> ObjectGridType: (EMBEDDED | EMBEDDED_PARTITION | REMOTE) the type of ObjectGrid.
 *              <ul>
 *              <li> EMBEDDED: This is the default ObjectGrid type. There will be only 1 partition with SYNC replica mode and the maximum number of replicas set to 47 by default.
 *                             With the EMBEDDED type, users can specify the ReplicaMode and MaxNumberOfReplicas to determine the type of replication and
 *                             the maximum number of replicas. The ReplicaReadEnabled will be always true.
 *                             If a system has more than 47 JVMs, users should set the MaxNumberOfReplicas to be equal to the number of JVMs.
 *                             This EMBEDDED mode has two limitations:
 *                             <ul>
 *                             <li>limited cache size: because there is only one partition so the cache size is limited to the heap size of the JVM.
 *                                 If the system need to cache 100GB, it should use EMBEDDED_PARTITION type.
 *                             <li>All write operations will go to one JVM.
 *                             </ul>
 *              <li> EMBEDDED_PARTITION: This type should be used when the system would like to cache large amount of data.
 *                   It is default to have 47 partitions with NONE replica mode. In a small system that has only few JVMs, users should set the NumberOfPartitions to be the number of JVMs.
 *                   Users can specify ReplicaMode, NumberOfPartitions, MaxNumberOfReplicas, and ReplicaReadEnabled to tune the system
 *              <li> REMOTE: With this REMOTE type, the cache simply try to obtain a distributed ObjectGrid from catalog service. Users should place objectGridServer.properties file in classpath.
 *                   The objectGridServer.properties should have the catalogServiceEndPoints property. It is users' responsibility to setup the remote ObjectGrid system.
 *              </ul>
 *      <li> NumberOfPartitions: (>=1) the number of partitions to be used for the cache. The default value is 47 for EMBEDDED-PARTITION. Only apply on EMBEDDED-PARTITION ObjectGrid type.
 *      <li> ReplicaMode: (SYNC/ASYNC/NONE; not case sensitive) how cache is replicated to replica. The default value is NONE. Only apply on EMBEDDED and EMBEDDED-PARTITION ObjectGrid type.
 *      <li> ReplicaReadEnabled: (TRUE/FALSE; not case sensitive) to enable reading data from replica or not. The default value is FALSE. Only apply on EMBEDDED_PARTITION ObjectGrid type.
 *      <li> MaxUsedMemory: (TRUE/FALSE; not case sensitive) to enable memory-based evictors on maps. The default value is TRUE.
 *      <li> MaxNumberOfReplicas: (>=1) the maximum number of replicas to be used for the cache. For EMBEDDED ObjectGrid type, this number should be equal or greater than the number of JVMs in a system.
 *		<li> PlacementScope: (DOMAIN_SCOPE/CONTAINER_SCOPE)
 *				<ul>
 *				<il> DOMAIN_SCOPE:  This is the default placement scope.  It will result in a single primary shard for each partition in the grid
 *									being placed in the replication domain.  When using a single partition topology, all writes will go to the container
 *									hosting this shard.
 *				<il> CONTAINER_SCOPE:  This placement scope will result in a primary shard for every partition being placed in every container.  This should
 *									only be used when a single partition is configured and the ObjectGrid type is EMBEDDED.  This will result in each container
 *									having a local primary shard that it can write to without making a remote call.  The writes performed in different containers
 *									will be reconciled via intra-domain replication.  The number of replicas will be automatically set to zero if this scope is
 *									selected as the redundant primary shards implicitly act as replicas.
 *				</ul>
 *		<li> PlacementScopeTopology: (HUB/RING) determines the connection scheme between containers when CONTAINER_SCOPE placement is used.  HUB corresponds to a hub/spoke model and RING corresponds to a standard ring topology.
 *      </ul>
 *
 * <P>
 * The following is a configuration example of JPA persistence.xml file.
 * <pre>
 * &lt;property
 *   name="openjpa.DataCache"
 *   value="com.ibm.websphere.objectgrid.openjpa.ObjectGridDataCache(objectGridName=MyObjectGridName, objectGridType=EMBEDDED, MaxNumberOfReplicas=47)" /&gt;
 * &lt;property
 *   name="openjpa.ObjectGridQueryCache"
 *   value="com.ibm.websphere.objectgrid.openjpa.ObjectGridQueryCache()" /&gt;
 * &lt;property name="openjpa.RemoteCommitProvider" value="sjvm" /&gt;
 * </pre>
 *
 * @since WAS XD 6.1.0.3
 */
public class ObjectGridDataCache
    extends AbstractDataCache
    implements RemoteCommitListener {

    private static final long serialVersionUID = 5667878558223470911L;

    private final ObjectGridCacheMap _cache = new ObjectGridCacheMap();
    private Properties objectGridConf = new Properties();

    // configuration properties from configuration xml file
    private String _gridName;
    private String _numberOfPartitions;
    private String _replicaMode;
    private String _replicaReadEnabled;

    private String _gridType = OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED;
    private String _nearCache;
    private String _maxUsedMemory;
    private String _maxNumberOfReplicas;
    private boolean isEmbeddedFastRead = true; // default
    private boolean _batchPut = false;
    private boolean _batchRemove = false;
    private String _placementScope=null;
    private String _placementScopeTopology=null;

    private Log _log = null;

    private JPAPMIModule pmiModule = null;

    private static boolean statisticsEnabled = false;
    private Map pmiModuleMap = null;


    /**
     * Returns the underlying ObjectGrid cache that this cache is using.
     */
    private Object getCacheMap() {

        return _cache;
    }

    /**
     * Initialize this data cache
     *
     * @see org.apache.openjpa.datacache.DataCache#initialize(org.apache.openjpa.datacache.DataCacheManager)
     */
    public void initialize(DataCacheManager mgr) {
        super.initialize(mgr);

        _log = conf.getLog(OpenJPAConfiguration.LOG_RUNTIME);
        conf.getRemoteCommitEventManager().addListener(this);

        // the _gridName could be null or ""
        // in that case, the default objectGrid name will be the persistence unit name.
        if (_gridName == null || _gridName.equals("")) {
            _gridName = conf.getId();
        }

        objectGridConf.setProperty(OpenJPAConstants.DATA_GRID_NAME, _gridName);
        objectGridConf.setProperty(OpenJPAConstants.PERSISTENCE_UNIT_NAME, conf.getId());

        if(_gridType != null) {
            if(_gridType.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_TYPE_LOCAL)) {
                // do not support LOCAL officially
                //objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_LOCAL);
            } else if(_gridType.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_TYPE_REMOTE)){
                objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_REMOTE);
                isEmbeddedFastRead = false;
            } else if(_gridType.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED)){
                objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED);
                isEmbeddedFastRead = true;
                _batchPut = true;
                _batchRemove = true;
            } else if(_gridType.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED_PARTITION)){
                objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED_PARTITION);
                isEmbeddedFastRead = false;
            } else {
                if (_log.isTraceEnabled()) {
                    _log.trace("unknown ObjectGridType: " + _gridType);
                    //System.out.println(unknown ObjectGridType: " + _gridType);
                    objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_UNKNOWN);
                }
            }
        }

        if(_maxNumberOfReplicas != null){
            objectGridConf.setProperty(OpenJPAConstants.MAX_NUMBER_OF_REPLICAS, _maxNumberOfReplicas);
        }

        if(_maxUsedMemory != null && _maxUsedMemory.equalsIgnoreCase("false")){
            objectGridConf.setProperty(OpenJPAConstants.MAX_USED_MEMORY, "false");
        }

        if(_nearCache != null && _nearCache.equalsIgnoreCase("true")){
            objectGridConf.setProperty(OpenJPAConstants.NEAR_CACHE, "true");
        }

        if (_numberOfPartitions != null)
            objectGridConf.setProperty(OpenJPAConstants.NUMBER_OF_PARTITIONS,
                                      _numberOfPartitions);

        if (_replicaMode != null)
            objectGridConf.setProperty(OpenJPAConstants.REPLICA_MODE,
                                       _replicaMode.toUpperCase());

        if (_replicaReadEnabled != null && _replicaReadEnabled.equalsIgnoreCase("true"))
            objectGridConf.setProperty(OpenJPAConstants.REPLICA_READ_ENABLED,
                    "true");

        if(_placementScope!=null) {
        	objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_PlacementScope,_placementScope);
        }
        if(_placementScopeTopology!=null) {
        	objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_PlacementScopeTopology,_placementScopeTopology);
        }

        _cache.initializeObjectGrid(conf, objectGridConf);
        this.isEmbeddedFastRead = _cache.isEmbeddedFastReadObjectGrid();
        pmiModule = _cache.getDataCachePmiModule();
        pmiModule.setName("ObjectGridDataCache");

        pmiModuleMap = _cache.getDataCachePmiModuleMap();

        statisticsEnabled = ObjectGridResourceFactory.isCacheStaticsEnabled();
    }

    private JPAPMIModule getJPAPMIModule(Class entityClass) {
    	String entityClassName = entityClass.getName();
    	JPAPMIModule pmiModule = (JPAPMIModule)pmiModuleMap.get(entityClassName);

    	if(pmiModule == null){
    		pmiModule = createJPAPMIModule(entityClass);
    	}
    	return pmiModule;
    }

    private synchronized JPAPMIModule createJPAPMIModule(Class entityClass) {
    	String entityClassName = entityClass.getName();
    	JPAPMIModule pmiModule = (JPAPMIModule)pmiModuleMap.get(entityClassName);
    	if(pmiModule == null){
    		pmiModule = new JPAPMIModule();
    		pmiModule.setName(entityClassName);
    		pmiModule.setDescription("map name = " + _cache.getMapNameByEntityClass(entityClass));
    		pmiModule.setType(JPAPMIModule.TYPE_OpenJPA_Entity);
    		pmiModuleMap.put(entityClassName, pmiModule);
    	}
    	return pmiModule;
    }

    // there is no public pin method provided by ObjectGrid
    /**
     * This is not applicable to ObjectGrid
     * @see org.apache.openjpa.datacache.AbstractDataCache#unpinAll(java.lang.Class, boolean)
     */
    public void unpinAll(Class cls, boolean subs) {}

    /**
     * Acquires write lock
     * <p>
     * writeLock is for synchroning the update on the cache.
     * Before any update to the cache, writeLock should be obtained and
     * held until commit is done.  After each commit, writeLock should be
     * released (by calling writeUnlock()) for others to update the cache.
     * @see org.apache.openjpa.datacache.DataCache#writeLock()
     */
    public void writeLock() {
        _cache.writeLock();
    }

    /**
     * Releases writeLock, paired with writeLock() for update synchronization
     * @see org.apache.openjpa.datacache.DataCache#writeUnlock()
     */
    public void writeUnlock() {
        _cache.writeUnlock();
    }


    /**
     * Returns an object for a given key
     * @see org.apache.openjpa.datacache.AbstractDataCache#getInternal(java.lang.Object)
     */
    protected DataCachePCData getInternal(Object key) {
    	if(statisticsEnabled) {
    		pmiModule.counterGet++;
    		getJPAPMIModule(((OpenJPAId)key).getType()).counterGet++;
    	}

        if (isEmbeddedFastRead) {
			DataCachePCData ogValue = (DataCachePCData) _cache.get(key);
			return ogValue;
		} else {
			ObjectGridPCKeyImpl ogKey = new ObjectGridPCKeyImpl(key);
			ObjectGridPCDataImpl ogValue = (ObjectGridPCDataImpl) _cache
					.get(ogKey);
			return (ogValue == null) ? null : ogValue.convertToPCData(conf
					.getMetaDataRepositoryInstance().getMetaData(
							ogValue.getId(), null, true));

		}
    }

    /**
     * Caches an object by a given key
     * @see org.apache.openjpa.datacache.AbstractDataCache#putInternal(java.lang.Object, org.apache.openjpa.datacache.DataCachePCData)
     */
    protected DataCachePCData putInternal(Object key, DataCachePCData pc) {
    	if(statisticsEnabled) {
    		pmiModule.counterPut++;
    		getJPAPMIModule(((OpenJPAId)key).getType()).counterPut++;
    	}

        if (isEmbeddedFastRead) {
			Object val = _cache.put(key, pc);
			return ((val == null) ? null : pc);
		} else {
			ObjectGridPCKeyImpl ogKey = new ObjectGridPCKeyImpl(key);

			ObjectGridPCDataImpl ogValue = new ObjectGridPCDataImpl(pc, conf
					.getMetaDataRepositoryInstance().getMetaData(pc.getType(),
							null, true));
			Object val = _cache.put(ogKey, ogValue);
			return ((val == null) ? null : pc);

        }
    }

    /**
     * Caches a collection of cache objects
     * @see org.apache.openjpa.datacache.AbstractDataCache#putAllInternal(java.util.Collection)
     */
    protected void putAllInternal(Collection pcs) {
        if (pcs == null || pcs.size() == 0) {
            return;
        }
        // If not batching or less than min batch threshold, process puts individually.
        if (!_batchPut) {
            super.putAllInternal(pcs);
            return;
        }

        // the batch put can only support EMBEDDED topology
        // in the future, if decide to support EMBEDDED_PARTITION and REMOTE
        // need to fix the putAll path, it is broken now.

        boolean committed = putAll(pcs);
        if(!committed) {
        	// when applying batch updates failed, it is most likely due to contention
        	// change to conservative approach to let parent handle it.
        	super.putAllInternal(pcs);
            return;
        }
    }

    /**
     * Builds up a map of values and executes a batch put to the cache
     * @param pcs
     */

    private boolean putAll(Collection pcs) {

        if(statisticsEnabled) {
            pmiModule.counterPutAll++;
        }

        int size = pcs.size();
        Object[] keys = new Object[size];
        Object[] values = new Object[size];
        Map objMap = new HashMap();
        Iterator iter = pcs.iterator();
        //for (DataCachePCData pc : pcs) {
        for (int i=0; i<size; i++) {
        	DataCachePCData pc = (DataCachePCData) iter.next();
        	Object key = null;
        	Object value = null;
        	if(isEmbeddedFastRead) {
        		key = pc.getId();
        		value = pc;
        	} else {
        		key = new ObjectGridPCKeyImpl(pc.getId());
        		value = newPCData(pc);
        	}

            if(statisticsEnabled) {
                pmiModule.counterPut++;
                if(isEmbeddedFastRead)
                    getJPAPMIModule(((OpenJPAId)pc.getId()).getType()).counterPut++;
            }

            objMap.put(key, value);
        }

        return  _cache.putAll(objMap, isEmbeddedFastRead);
    }


    /**
     * Removes a cache object by a given key.
     * @see org.apache.openjpa.datacache.AbstractDataCache#remove(java.lang.Object)
     */
    public DataCachePCData remove(Object key) {
        return removeInternal(key);
    }

    /**
     * Deletes a cached object by a given key.
     * @see org.apache.openjpa.datacache.AbstractDataCache#removeInternal(java.lang.Object)
     */
    protected DataCachePCData removeInternal(Object key) {
    	if(statisticsEnabled) {
    		pmiModule.counterRemove++;
    		getJPAPMIModule(((OpenJPAId)key).getType()).counterRemove++;
    	}

    	if(key == null) {
    		return null;
    	}

    	if (isEmbeddedFastRead) {
    		Object val = _cache.remove(key);
    		return ((val == null) ? null : (DataCachePCData)val);
		} else {
			ObjectGridPCKeyImpl ogKey = new ObjectGridPCKeyImpl(key);
			ObjectGridPCDataImpl ogValue = (ObjectGridPCDataImpl) _cache
					.remove(ogKey);
			return (ogValue == null) ? null : ogValue.convertToPCData(conf
					.getMetaDataRepositoryInstance().getMetaData(
							ogValue.getId(), null, true));
		}
    }


    /**
     * Clear the map corresponding to the entity class
     * @see org.apache.openjpa.datacache.AbstractDataCache#removeAllInternal(java.lang.Class, boolean)
     */
    protected void removeAllInternal(Class cls, boolean subs) {
    	//System.out.println("removeAllInternal: " + cls);
    	if(statisticsEnabled) {
    		pmiModule.counterRemoveAll++;
    		getJPAPMIModule(cls).counterRemoveAll++;
    	}
        // single map for inheritance is used, so all entity shared the same root will be clear.
        _cache.clear(cls);
    }


    /**
     * Deletes a collection of objects from the cache.
     * @see org.apache.openjpa.datacache.AbstractDataCache#removeAllInternal(java.util.Collection)
     */
    protected void removeAllInternal(Collection keys) {

        if(statisticsEnabled) {
            pmiModule.counterRemoveAll++;
        }

        if (keys == null || keys.size() == 0) {
            return;
        }

        // If direct batching is disabled, call the superclass to process
        // keys individually
        if (!_batchRemove) {
            super.removeAllInternal(keys);
            return;
        }

        Collection keyColl = keys;

        List oids = null;
        // If not embedded mode, must convert keys to object grid keys
        if (!isEmbeddedFastRead) {
            oids = new ArrayList(keys.size());
            keyColl = oids;
        }

        // If not embedded mode or stats enabled, spin through the keys, convert
        // to og keys if not embedded, record type specific stats if stats are
        // enabled.
        if (statisticsEnabled || !isEmbeddedFastRead) {
        	int size = keys.size();
        	Iterator iter = keys.iterator();
        	//for (Object oid : keys) {
            for (int i=0; i<size; i++) {
            	Object oid = iter.next();
                Object key = oid;
                OpenJPAId okey = (OpenJPAId)oid;
                if (!isEmbeddedFastRead) {
                    key = new ObjectGridPCKeyImpl(okey);
                    oids.add(key);
                }
                if(statisticsEnabled) {
                    getJPAPMIModule(okey.getType()).counterRemove++;
                }
            }
        }

        boolean committed = _cache.removeAll(keyColl, isEmbeddedFastRead);
        if(!committed) {
        	// when applying batch updates failed, it is most likely due to contention
        	// change to conservative approach to let parent handle it.
        	super.removeAllInternal(keys);
            return;
        }

    }

    /**
     * Clear all maps for a PU.
     * @see org.apache.openjpa.datacache.AbstractDataCache#clearInternal()
     */
    protected void clearInternal() {
    	if(statisticsEnabled) {
    		pmiModule.counterClear++;
    	}

    	// note: if clear all Datacache, then Query cache won't be able to fetch data from Datacache
    	// thus, also clear QueryCache.

    	// clear Query cache first to prevent queries from fetching data in current data cache.
    	_cache.clearQueryMap();

        _cache.clearAllMapsExcludeQueryMap();
    }

    // There is no public method by ObjectGrid to pin objects in the chace
    /**
     * This is not applicable to ObjectGrid
     * @see org.apache.openjpa.datacache.AbstractDataCache#pinInternal(java.lang.Object)
     */
    protected boolean pinInternal(Object key) {
        return true;
    }

    // There is no public method by ObjectGrid to unpin objects in the chace
    /**
     * This is not applicable to ObjectGrid
     * @see org.apache.openjpa.datacache.AbstractDataCache#unpinInternal(java.lang.Object)
     */
    protected boolean unpinInternal(Object key) {
        return true;
    }


    ///** Sets whether near cache is enabled */
    //public void setNearCache(String nearCache) {
    //    _nearCache=nearCache;
    //}

    ///** Returns setting of whether near cache is enabled  */
    //public String getNearCache() {
    //    return _nearCache;
    //}


    /** Sets whether memory based eviction is enabled */
    public void setMaxUsedMemory(String maxUsedMemory) {
        _maxUsedMemory=maxUsedMemory;
    }

    /** Returns setting of whether memory based eviction is enabled */
    public String getMaxUsedMemory() {
        return _maxUsedMemory;
    }

    /** Sets the name of the ObjectGrid that handles the cache */
    public void setObjectGridName(String name) {
        _gridName=name;
    }

    /** Returns the name of the ObjectGrid that handel the cache */
    public String getObjectGridName() {
        return _gridName;
    }

    /** Sets the ObjectGrid type. Valid values includ EMBEDDED, EMBEDDED_PARTITION, and REMOTE */
    public void setObjectGridType(String objectGridType) {
        _gridType=objectGridType;
    }

    /** Returns the ObjectGrid type. */
    public String getObjectGridType() {
        return _gridType;
    }


    /** Sets the number of partitions to be used in the cache */
    public void setNumberOfPartitions(String number) {
        _numberOfPartitions=number;
    }

    /** Returns the number of partitions used in the cache */
    public String getNumberOfPartitions() {
        return _numberOfPartitions;
    }

    /** Sets the replica mode */
    public void setReplicaMode(String mode) {
        _replicaMode=mode;
    }

    /** Returns the replica mode */
    public String getReplicaMode() {
        return _replicaMode;
    }

    /** Sets whether reading from replica is enabled */
    public void setReplicaReadEnabled(String read) {
        _replicaReadEnabled=read;
    }

    /** Returns setting of whether reading from the replica is enabled */
    public String getReplicaReadEnabled() {
        return _replicaReadEnabled;
    }

    /** Returns the maximum number of replicas for the EMBEDDED or EMBEDDED_PARTITION ObjectGrid type */
    public String getMaxNumberOfReplicas() {
        return _maxNumberOfReplicas;
    }

    /** Sets the maximum number of replicas for the EMBEDDED or EMBEDDED_PARTITION ObjectGrid type */
    public void setMaxNumberOfReplicas(String numberOfReplicas) {
        _maxNumberOfReplicas = numberOfReplicas;
    }

    /**
     * Retrieve the scope used for partition placement.  Either DOMAIN_SCOPE or CONTAINER_SCOPE.
     * DOMAIN_SCOPE corresponds to default partition placement behavior in which one primary shard
     * will exist for each partition, within the scope of the entire domain.
     * CONTAINER_SCOPE corresponds to intra-domain replication behavior in which each one primary shard
     * will exist for each partition within the scope of each container.
     * @return the placement scope for the grid
     * @since 7.1.1
     */
    public String getPlacementScope() {
    	return _placementScope;
    }

    /**
     * Set the scope used for partition placement.  Either DOMAIN_SCOPE or CONTAINER_SCOPE.
     * Retrieve the scope used for partition placement.  Either DOMAIN_SCOPE or CONTAINER_SCOPE.
     * DOMAIN_SCOPE corresponds to default partition placement behavior in which one primary shard
     * will exist for each partition, within the scope of the entire domain.
     * CONTAINER_SCOPE corresponds to intradomain replication behavior in which each one primary shard
     * will exist for each partition within the scope of each container.
     * @param scope the placement scope for the grid
     * @since 7.1.1
     */
    public void setPlacementScope(String scope) {
    	_placementScope=scope;
    }

    /**
     * Retrieve the link topology used to perform intradomain replication.  Supported values are RING and HUB.
     * @return the link topology
     * @since 7.1.1
     */
    public String getPlacementScopeTopology() {
    	return _placementScopeTopology;
    }

    /**
     * Set the link topology used to perform intradomain replication.  Supported values are RING and HUB.
     * @param scopeTopology the link topology
     * @since 7.1.1
     */
    public void setPlacementScopeTopology(String scopeTopology) {
    	_placementScopeTopology=scopeTopology;
    }

    /**
     * Commits a transaction
     * @see org.apache.openjpa.datacache.AbstractDataCache#commit(java.util.Collection, java.util.Collection, java.util.Collection, java.util.Collection)
     */
    public void commit(Collection additions, Collection newUpdates,
            Collection existingUpdates, Collection deletes) {
        super.commit(additions, newUpdates, existingUpdates, deletes);

        if(statisticsEnabled) {
        	pmiModule.counterCommit++;
        }
        //System.out.println("commit");

        try {
            _cache.commit();
        } catch (TransactionException e) {
            System.err.println("Exception::" + e);
            throw (new InternalException(e));
        } catch (ObjectGridException e) {
            System.err.println("Exception::" + e);
            throw (new InternalException(e));
        }
    }

    /**
     * Returns a list of objects associated with the given keys
     *
     * @param keys the keys of the entries to retrieve.
     *
     * @return a list of PCData
     * @see org.apache.openjpa.datacache.AbstractDataCache#getAll(java.util.List)
     */
    public Map getAll(List keys) {
    	if(statisticsEnabled) {
    		pmiModule.counterGetAll++;
    	}

        //System.out.println("getAll");

    	int len = keys.size();
        if(len == 0) {
    		return Collections.EMPTY_MAP;
    	}

        if (isEmbeddedFastRead) {
	        Map ogom = _cache.getAll(keys);
	        return ogom;
		} else {
			Map resultMap = new HashMap (len);
			if(len==1) {
				Object key = keys.get(0);
				ObjectGridPCDataImpl ogValue = (ObjectGridPCDataImpl) _cache.get(new ObjectGridPCKeyImpl(key));
				resultMap.put(key, (ogValue == null)? null :
	                ogValue.convertToPCData(
	                        conf.getMetaDataRepositoryInstance()
	                        .getMetaData(ogValue.getId(), null, true)));
				return resultMap;
			}

	        List ogkeys = new ArrayList (len);
	        Iterator keyItr = null;
	        for (keyItr = keys.iterator(); keyItr.hasNext();) {
	            ogkeys.add(new ObjectGridPCKeyImpl(keyItr.next()));
	        }

	        Map ogom = _cache.getAll(ogkeys);

	        keyItr = keys.iterator();
	        // use while loop ... to do
	        // use size for don't use hasNext.

	        for (int i=0; i<len; i++) {
	            Object key = keyItr.next();
	            // note: ogom.get() could return EntryErrorValueImpl
	            // -> java.lang.ClassCastException: com.ibm.ws.objectgrid.datagrid.EntryErrorValueImpl incompatible with com.ibm.ws.objectgrid.openjpa.ObjectGridPCDataImp
	            ObjectGridPCDataImpl ogValue =
	                (ObjectGridPCDataImpl) ogom.get(new ObjectGridPCKeyImpl(key));
	            /*
	            ObjectGridPCDataImpl ogValue =
	                (ObjectGridPCDataImpl) getObject((byte[])ogom.get(new ObjectGridPCKeyImpl(key)));
	            */

	            resultMap.put(key, (ogValue == null)? null :
	                ogValue.convertToPCData(
	                        conf.getMetaDataRepositoryInstance()
	                        .getMetaData(ogValue.getId(), null, true)));
	        }

	        return resultMap;
		}
    }

    /**
     * Close this data cache
     * @see org.apache.openjpa.datacache.AbstractDataCache#close()
     */
    public void close() {
        _cache.close();
        super.close();
    }

    /**
     * Return <code>true</code> because ObjectGrid is a external cache
     * and updates to data already in the cache (either in {@link #commit}
     * or the {@link #update}) should be put back into the cache.
     * @see org.apache.openjpa.datacache.AbstractDataCache#recacheUpdates()
     */
    protected boolean recacheUpdates() {
        return true;
    }

    private String dumpStatus() {
    	//return pmiModule.toString();
    	return _cache.getObjectGridResource().dumpCacheStatistics();
    }

    private Object newPCData(DataCachePCData pc) {
    	if (isEmbeddedFastRead) {
    		return pc;
    	} else {
            return new ObjectGridPCDataImpl(pc, conf
                .getMetaDataRepositoryInstance().getMetaData(pc.getType(),
                    null, true));
    	}
    }

    private DataCachePCData getPCData(Object ogKey, boolean remove) {
        Object value = (remove ? _cache.remove(ogKey) : _cache.get(ogKey));
        if (value == null) {
            return null;
        }
        if (value instanceof ObjectGridPCDataImpl) {
            ObjectGridPCDataImpl ogValue = (ObjectGridPCDataImpl) value;
            return (ogValue == null) ? null : ogValue.convertToPCData(conf
                .getMetaDataRepositoryInstance().getMetaData(
                ogValue.getId(), null, true));
        }
        return (DataCachePCData)value;
    }

	// back support
    /*
	private static void displayStatus() {
		ObjectGridResourceFactory.dumpActiveObjectGridResourceCacheStatistics();
        //System.out.println(dumpStatus());
    }*/


	private static String dumpCacheStatistics() {
		return ObjectGridResourceFactory.dumpActiveObjectGridResourceCacheStatistics();
	}

	private static void setCacheStaticsEnabled(boolean enabled) {
		ObjectGridResourceFactory.setCacheStaticsEnabled(enabled);
	}

	private static void setDumpQueryKeys(boolean enabled) {
		ObjectGridResourceFactory.setDumpQueryKeys(enabled);
	}
}
