// IBM Confidential OCO Source Material
// 5733-W70 (C) COPYRIGHT International Business Machines Corp. 2007, 2009
//
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  ObjectGridQueryCache.java
//
// Source File Description:
//
//   The OpenJPA plugin class for query cache using ObjectGrid
//
// Change Activity:
//
// Reason          Version   Date     Userid    Change Description
// --------------- --------- -------- --------- -------------------------------
//                 FeP       20070620 dtlee     new file
// 9603            XD61      20091001 dtlee     fix keySet()
// b15969          7.0       20090430 cheng1    improve performance and enhance cache statistics
// b22644          7.1.x     20110405 cheng1    parse DataCache ObjectGrid configuration and apply on QueryCache
// --------------- --------- -------- --------- -------------------------------
package com.ibm.websphere.objectgrid.openjpa;

import java.util.Collection;
import java.util.Map;
import java.util.Properties;

import org.apache.openjpa.conf.OpenJPAConfiguration;
import org.apache.openjpa.datacache.AbstractQueryCache;
import org.apache.openjpa.datacache.DataCacheManager;
import org.apache.openjpa.datacache.QueryKey;
import org.apache.openjpa.datacache.QueryResult;
import org.apache.openjpa.event.RemoteCommitListener;
import org.apache.openjpa.lib.log.Log;

import com.ibm.ws.objectgrid.jpa.cache.common.JPAPMIModule;
import com.ibm.ws.objectgrid.openjpa.ObjectGridCacheMap;
import com.ibm.ws.objectgrid.openjpa.ObjectGridResourceFactory;
import com.ibm.ws.objectgrid.openjpa.OpenJPAConstants;

/**
 * A {@link org.apache.openjpa.datacache.QueryCache} implementation is a
 * QueryCache plugin to integrate OpenJPA with ObjectGrid for leveraging the
 * cluster server advantages provided by ObjectGrid.
 *
 * See {@link ObjectGridDataCache} for detail configuration information.
 *
 * @since WAS XD 6.1.0.3
 */
public class ObjectGridQueryCache
    extends AbstractQueryCache
    implements RemoteCommitListener {

	private static final long serialVersionUID = -1865413335370636697L;

    private final ObjectGridCacheMap _cache = new ObjectGridCacheMap();
    private Properties objectGridConf = new Properties();

    // configuration properties from configuration xml file
    private String _gridName;
    private String _numberOfPartitions;
    private String _replicaMode;
    private String _replicaReadEnabled;

    private String _gridType = OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED;

    private String _nearCache;
    private String _maxUsedMemory;
    private String _maxNumberOfReplicas;
    private String _placementScope=null;
    private String _placementScopeTopology=null;

    private Log _log = null;
    private JPAPMIModule pmiModule = null;

    private static boolean statisticsEnabled = true;
    private Map pmiModuleMap = null;


    /**
     * Returns the underlying ObjectGrid cache that this cache is using.
     */
    private Object getCacheMap() {
        return _cache;
    }

    /**
     * Sets the configuration for initializing Query Cache
     *
     * @see org.apache.openjpa.datacache.QueryCache#initialize(org.apache.openjpa.datacache.DataCacheManager)
     */
    public void initialize(DataCacheManager mgr) {
        super.initialize(mgr);

        _log = conf.getLog(OpenJPAConfiguration.LOG_RUNTIME);
        conf.getRemoteCommitEventManager().addListener(this);

        // the _gridName could be null or ""
        // in that case, the default objectGrid name will be the persistence unit name.
        if (_gridName == null || _gridName.equals("")) {
            _gridName = conf.getId();
        }

        objectGridConf.setProperty(OpenJPAConstants.QUERY_GRID_NAME, _gridName);
        objectGridConf.setProperty(OpenJPAConstants.PERSISTENCE_UNIT_NAME, conf.getId());

        if(_gridType != null) {
            if(_gridType.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_TYPE_LOCAL)) {
                // do not support LOCAL officially
                // objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_LOCAL);
            } else if(_gridType.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_TYPE_REMOTE)){
                objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_REMOTE);
            } else if(_gridType.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED)){
                objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED);
            } else if(_gridType.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED_PARTITION)){
                objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_TYPE, OpenJPAConstants.OBJECTGRID_TYPE_EMBEDDED_PARTITION);
            } else {
                if (_log.isTraceEnabled()) {
                    _log.trace("unknown ObjectGridType: " + _gridType);
                    //System.out.println(unknown ObjectGridType: " + _gridType);
                }
            }
        }

        if(_maxNumberOfReplicas != null){
            objectGridConf.setProperty(OpenJPAConstants.MAX_NUMBER_OF_REPLICAS, _maxNumberOfReplicas);
        }

        if(_maxUsedMemory != null && _maxUsedMemory.equalsIgnoreCase("false")){
            objectGridConf.setProperty(OpenJPAConstants.MAX_USED_MEMORY, "false");
        }

        if(_nearCache != null && _nearCache.equalsIgnoreCase("true")){
            objectGridConf.setProperty(OpenJPAConstants.NEAR_CACHE, "true");
        }

        if (_numberOfPartitions != null)
            objectGridConf.setProperty(OpenJPAConstants.NUMBER_OF_PARTITIONS,
                    _numberOfPartitions);
        if (_replicaMode != null)
            objectGridConf.setProperty(OpenJPAConstants.REPLICA_MODE,
                    _replicaMode.toUpperCase());
        if (_replicaReadEnabled != null && _replicaReadEnabled.equalsIgnoreCase("true"))
            objectGridConf.setProperty(OpenJPAConstants.REPLICA_READ_ENABLED,
                    "true");

        // QueryCache may be initialized before DataCache, and QueryCache depends on DataCache.
        // So, always copy DataCache ObjetcGrid configuration to QueryCache to ensure same ObjectGridResource is created.
        processDataCacheConfiguration(conf.getDataCache());
        String configuredObjectGridName = objectGridConf.getProperty(OpenJPAConstants.OBJECTGRID_NAME);
        if(configuredObjectGridName != null && configuredObjectGridName.trim().length() > 0) {
        	_gridName = configuredObjectGridName;
        	objectGridConf.setProperty(OpenJPAConstants.QUERY_GRID_NAME, configuredObjectGridName);
        }

        if(_placementScope!=null) {
        	objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_PlacementScope,_placementScope);
        }
        if(_placementScopeTopology!=null) {
        	objectGridConf.setProperty(OpenJPAConstants.OBJECTGRID_PlacementScopeTopology,_placementScopeTopology);
        }

        _cache.initializeObjectGrid(conf, objectGridConf);
        pmiModule = _cache.getQueryCachePmiModule();
        pmiModule.setName("ObjectGridQueryCache");

        statisticsEnabled = ObjectGridResourceFactory.isCacheStaticsEnabled();
    }

    private void processDataCacheConfiguration(String dataCacheString) {
    	// e.g. dataCacheString = com.ibm.websphere.objectgrid.openjpa.ObjectGridDataCache(objectGridName=BasicTestObjectGrid, ObjectGridType=REMOTE)
    	String objectgridConfiguration = null;
    	if(dataCacheString != null && dataCacheString.trim().length() > 0) {
        	int completeLength = dataCacheString.length();
        	int startIndex = dataCacheString.indexOf('(') + 1;
        	int endIndex = dataCacheString.indexOf(')');
         	objectgridConfiguration = dataCacheString.substring(startIndex, completeLength-1);
        }

        if (objectgridConfiguration != null && objectgridConfiguration.trim().length() > 0) {
            String[] ogConfigProperties = objectgridConfiguration.split(",");
            for (int i = 0; i < ogConfigProperties.length; i++) {
                String propertyToken = ogConfigProperties[i].trim();
                // System.out.println(" propertyToken = " + propertyToken );
                String[] propertyNameValue = propertyToken.split("=");
                if (propertyNameValue.length == 2) {
                    if (propertyNameValue[0] == null || propertyNameValue[0].equals("")) {
                        // System.out.println("\t NO key part of propertyNameValue : " + propertyNameValue[0]);
                    } else if (propertyNameValue[1] == null || propertyNameValue[1].equals("")) {
                        // System.out.println("\t NO value part of propertyNameValue : " + propertyNameValue[1]);
                    } else {
                        // OK
                        //System.out.println("\t propertyNameValue = [" + propertyNameValue[0] + ", " + propertyNameValue[1] + "]");
                        // check if the property is recongnized

                        setObjectGridConfigurationProperty(propertyNameValue[0].trim(), propertyNameValue[1].trim());
                    }
                } else {
                    // System.out.println("\t wrong propertyToken: " + propertyToken);
                }
            }
        }
    }

    private boolean setObjectGridConfigurationProperty(String name, String value) {
        String mappedName = getMappedObjectGridConfPropertyName(name);
        if (mappedName != null) {
            objectGridConf.setProperty(mappedName, value);
        }

        return false;
    }

    private String getMappedObjectGridConfPropertyName(String name) {
        if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_CONFIGURATION_ObjectGridName)) {
            return OpenJPAConstants.OBJECTGRID_NAME;
        } else if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_ObjectGridType)) {
            return OpenJPAConstants.OBJECTGRID_TYPE;
        } else if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_NumberOfPartitions)) {
            return OpenJPAConstants.NUMBER_OF_PARTITIONS;
        } else if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_ReplicaMode)) {
            return OpenJPAConstants.REPLICA_MODE;
        } else if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_ReplicaReadEnabled)) {
            return OpenJPAConstants.REPLICA_READ_ENABLED;
        } else if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_MaxUsedMemory)) {
            return OpenJPAConstants.MAX_USED_MEMORY;
        } else if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_MaxNumberOfReplicas)) {
            return OpenJPAConstants.MAX_NUMBER_OF_REPLICAS;
        } else if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_PlacementScope)) {
        	return OpenJPAConstants.OBJECTGRID_PlacementScope;
        } else if (name.equalsIgnoreCase(OpenJPAConstants.OBJECTGRID_PlacementScopeTopology)) {
        	return OpenJPAConstants.OBJECTGRID_PlacementScopeTopology;
        }  else {
            return null;
        }
    }


    // there is no public pin method provided by ObjectGrid
//    /**
//     * this is not applicable to ObjectGrid
//     */
//    public void unpinAll(Class cls, boolean subs) {
//    }

    /**
     * writeLock is for synchroning the update on the cache.  Before any
     * update to the cache, writeLock should be obtained and held
     * until commit is done.  After each commit, writeLock should be
     * released (by calling writeUnlock()) for others to update the cache.
     * @see org.apache.openjpa.datacache.QueryCache#writeLock()
     */
    public void writeLock() {
//        _cache.writeLock();
    }

    /**
     * Releases writeLock, paired with writeLock() for update synchronization
     * @see org.apache.openjpa.datacache.QueryCache#writeUnlock()
     */
    public void writeUnlock() {
//        _cache.writeUnlock();
    }

    /**
     * Returns a query result for a given query key
     * @see org.apache.openjpa.datacache.AbstractQueryCache#getInternal(org.apache.openjpa.datacache.QueryKey)
     */
    protected QueryResult getInternal(QueryKey qk) {
    	if(statisticsEnabled) {
    		pmiModule.counterGet++;
    	}
        return (QueryResult) _cache.get(qk);
    }

    /**
     * Caches a query result set with a given query key
     * @see org.apache.openjpa.datacache.AbstractQueryCache#putInternal(org.apache.openjpa.datacache.QueryKey, org.apache.openjpa.datacache.QueryResult)
     */
    protected QueryResult putInternal(QueryKey qk, QueryResult qr) {
    	if(statisticsEnabled) {
    		pmiModule.counterPut++;
    	}
        Object val = _cache.put(qk, qr);
        return ((val == null)? null : qr);
    }

    /**
     * Removes the result set associated with the given query key
     * @see org.apache.openjpa.datacache.AbstractQueryCache#removeInternal(org.apache.openjpa.datacache.QueryKey)
     */
    protected QueryResult removeInternal(QueryKey qk) {
    	if(statisticsEnabled) {
    		pmiModule.counterRemove++;
    	}
        Object val = _cache.remove(qk);
        return (val == null)? null : (QueryResult) val;
    }


//    /**
//     * Clear the map corresponding to the entity class
//     */
//    protected void removeAllInternal(Class cls, boolean subs) {
//        // single map for inheritance is used, so all entity shared the same root will be clear.
//        _cache.clear(cls);
//    }

    /**
     * Clear the query cache
     * @see org.apache.openjpa.datacache.AbstractQueryCache#clearInternal()
     */
    protected void clearInternal() {
    	if(statisticsEnabled) {
    		pmiModule.counterClear++;
    	}
        _cache.clearQueryMap();
    }

    // There is no public method by ObjectGrid to pin objects in the chace
    /**
     * This is not applicable to ObjectGrid
     * @see org.apache.openjpa.datacache.AbstractQueryCache#pinInternal(org.apache.openjpa.datacache.QueryKey)
     */
    protected boolean pinInternal(QueryKey qk) {
        return true;
    }

    // There is no public method by ObjectGrid to unpin objects in the chace
    /**
     * This is not applicable to ObjectGrid
     * @see org.apache.openjpa.datacache.AbstractQueryCache#unpinInternal(org.apache.openjpa.datacache.QueryKey)
     */
    protected boolean unpinInternal(QueryKey qk) {
        return true;
    }

    ///** Sets whether near cache is enabled */
    //public void setNearCache(String nearCache) {
    //    _nearCache=nearCache;
    //}

    ///** Returns setting of whether near cache is enabled  */
    //public String getNearCache() {
    //    return _nearCache;
    //}


    /** Sets whether memory based eviction is enabled */
    public void setMaxUsedMemory(String maxUsedMemory) {
        _maxUsedMemory=maxUsedMemory;
    }

    /** Returns setting of whether memory based eviction is enabled */
    public String getMaxUsedMemory() {
        return _maxUsedMemory;
    }

    /** Sets the name of the ObjectGrid that handles the cache */
    public void setObjectGridName(String name) {
        _gridName=name;
    }

    /** Returns the name of the ObjectGrid that handel the cache */
    public String getObjectGridName() {
        return _gridName;
    }


    /** Sets the ObjectGrid type. Valid values includ LOCAL, REMOTE, and EMBEDDED */
    public void setObjectGridType(String objectGridType) {
        _gridType=objectGridType;
    }

    /** Returns the ObjectGrid type. */
    public String getObjectGridType() {
        return _gridType;
    }

    /** Sets the number of partitions to be used in the cache */
    public void setNumberOfPartitions(String number) {
        _numberOfPartitions=number;
    }

    /** Returns the number of partitions used in the cache */
    public String getNumberOfPartitions() {
        return _numberOfPartitions;
    }

    /** Sets the replica mode */
    public void setReplicaMode(String mode) {
        _replicaMode=mode;
    }

    /** Returns the replica mode */
    public String getReplicaMode() {
        return _replicaMode;
    }

    /** Sets whether reading from replica is enabled */
    public void setReplicaReadEnabled(String read) {
        _replicaReadEnabled=read;
    }

    /** Returns whether reading from the replica is enabled */
    public String getReplicaReadEnabled() {
        return _replicaReadEnabled;
    }

    /** Returns the maximum number of replicas for the EMBEDDED or EMBEDDED_PARTITION ObjectGrid type */
    public String getMaxNumberOfReplicas() {
        return _maxNumberOfReplicas;
    }

    /** Sets the maximum number of replicas for the EMBEDDED or EMBEDDED_PARTITION ObjectGrid type */
    public void setMaxNumberOfReplicas(String numberOfReplicas) {
        _maxNumberOfReplicas = numberOfReplicas;
    }

    /**
     * Retrieve the scope used for partition placement.  Either DOMAIN_SCOPE or CONTAINER_SCOPE.
     * DOMAIN_SCOPE corresponds to default partition placement behavior in which one primary shard
     * will exist for each partition, within the scope of the entire domain.
     * CONTAINER_SCOPE corresponds to intra-domain replication behavior in which each one primary shard
     * will exist for each partition within the scope of each container.
     * @return the placement scope for the grid
     * @since 7.1.1
     */
    public String getPlacementScope() {
    	return _placementScope;
    }

    /**
     * Set the scope used for partition placement.  Either DOMAIN_SCOPE or CONTAINER_SCOPE.
     * DOMAIN_SCOPE corresponds to default partition placement behavior in which one primary shard
     * will exist for each partition, within the scope of the entire domain.
     * CONTAINER_SCOPE corresponds to intra-domain replication behavior in which each one primary shard
     * will exist for each partition within the scope of each container.
     * @param scope the placement scope for the grid
     * @since 7.1.1
     */
    public void setPlacementScope(String scope) {
    	_placementScope=scope;
    }

    /**
     * Retrieve the link topology used to perform intradomain replication.  Supported values are RING and HUB.
     * @return the link topology
     * @since 7.1.1
     */
    public String getPlacementScopeTopology() {
    	return _placementScopeTopology;
    }

    /**
     * Set the link topology used to perform intradomain replication.  Supported values are RING and HUB.
     * @param scopeTopology the link topology
     * @since 7.1.1
     */
    public void setPlacementScopeTopology(String scopeTopology) {
    	_placementScopeTopology=scopeTopology;
    }

    /**
     * @see org.apache.openjpa.datacache.AbstractQueryCache#keySet()
     */
    protected Collection keySet() {
        return _cache.keySet ();
    }

    /**
     * Close this query cache
     * @see org.apache.openjpa.datacache.AbstractQueryCache#close()
     */
    public void close() {
        _cache.close();
        super.close();
    }

    private String dumpStatus() {
    	return pmiModule.toString();
    }

}