// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.9 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/Loader.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:22:51
//
// Module  :  Loader.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M3_B.10 WASX.XD   04/07/2005   sutter    Re-factor datagrid/datagrid.impl
// 267383       WASX.XD   04/20/2005   jhanders  touch support
// 271911       WASX      May 16, 2005 sutter    Partitioning support updates
// 277357       WASX.XD   May 21, 2005 kjlaw     preloadMap signature change.
// XD60_M4_B.17 WASX.XD   05/25/2005   sutter    ObjectGrid plan reset (remove some public APIs)
// 284443       WASX.XD   06/23/2005   sutter    modified preloadMap signature
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.io.InvalidObjectException;
import java.io.ObjectStreamException;
import java.io.Serializable;
import java.util.List;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.CopyMode;
import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.OutputFormat;
import com.ibm.websphere.objectgrid.PartitionManager;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.TxID;
import com.ibm.websphere.objectgrid.plugins.io.annotations.PluginOutputFormat;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedEntry;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedValue;

/**
 * <p>Data in a map sometimes are backed up by a "database". This "database"
 * can be any kind of storage. It could be database, a text file, or even an
 * in-memory object. This plugin allows users to plug in their own "database"
 * access protocol to load and store data.</p>
 *
 * <p>This Loader instance will be used by a BackingMap implementation at
 * runtime.</p>
 * <p>
 * A <code>Loader</code> implementation that also implements the {@link BackingMapLifecycleListener}
 * interface will be automatically added as an {@link EventListener} on the {@link BackingMap} when the
 * loader set on the backing map.
 * <p>
 * A <code>Loader</code> may also implement the {@link BackingMapPlugin} interface in order
 * to receive enhanced BackingMap plug-in lifecycle method calls. The plug-in is then
 * also required to correctly implement each of the bean methods related to
 * introspection of its state (for example <code>isInitialized()</code>,
 * <code>isDestroyed()</code>, etc).
 *
 *
 * @since WAS XD 6.0
 *
 * @see BackingMap#addMapEventListener(EventListener)
 * @see BackingMap#setLoader(Loader)
 * @see TransactionCallback
 */
public interface Loader
{
    /**
     * Special value class used for representing the key not being found in the
     * <code>Loader</code>.
     *
     * @since WAS XD 6.0
     */
    public static final class SpecialValue implements Serializable {

        private static final long serialVersionUID = -146836213363194509L;

        private byte ivValueType;

        SpecialValue(byte valueType) {
            ivValueType = valueType;
        }

        /**
         * @see Object#equals(Object)
         */
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if (o == null) {
                return false;
            }
            if (o.getClass().equals(SpecialValue.class) == false) {
                return false;
            }
            if (((SpecialValue)o).ivValueType == ivValueType) {
                return true;
            }
            return false;
        }

        /**
         * @see java.lang.Object#toString()
         */
        public String toString() {
            return ivValueType == 0 ? super.toString() + ":KEY_NOT_FOUND" : super.toString();
        }

        private Object readResolve() throws ObjectStreamException {
            if (ivValueType == 0) {
                return KEY_NOT_FOUND;
            }
            throw new InvalidObjectException("Unknown Loader.SpecialValue");
        }
    }

    /**
     * A special value type indicating that the key cannot be found
     */
    public static final SpecialValue KEY_NOT_FOUND = new SpecialValue((byte)0);

    /**
     * Signals the <code>Loader</code> to preload the data into the map.
     * <p>
     * The <code>Session</code> parameter should be used for session
     * demarcation.  The session parameter has no security enabled and allows
     * the various maps to be accessed without credentials. The
     * <code>Loader</code> is considered system code. The loader should use the
     * <code>Session.beginNoWriteThrough()</code> method for all session
     * transactions.  It should also use several smaller session transactions
     * when the amount of data is very large to avoid running out of log space.
     * <p>
     * This method also provides <code>Loader</code> initialization semantics
     * since it is called when an ObjectGrid is initialized. The
     * <code>Session</code> and <code>BackingMap</code> parameters should
     * provide all of the necessary access to the various ObjectGrid objects,
     * such as the <code>PartitionManager</code>.
     * <p>
     * The preload can be asynchronous when the async preload attribute is set
     * to <code>true</code> on the <code>BackingMap</code>.
     * <p>
     * Objects implementing this interface should also consider implementing
     * the <code>ReplicaPreloadController</code> interface as this interface
     * allows a recoverable preload when the map is being replicated. This
     * interface allows a replica to resume preload if the primary fails before
     * preload completes.
     * <p>
     * @param session a <code>Session</code> reference to use for demarcating transactions.
     * @param backingMap the <code>BackingMap</code> this object is associated with
     *
     * @throws LoaderException if an error occurs during processing of this method
     *
     * @see BackingMap
     * @see BackingMap#setPreloadMode(boolean)
     * @see PartitionManager
     * @see ReplicaPreloadController
     * @see Session
     * @see Session#beginNoWriteThrough()
     */
    void preloadMap(Session session, BackingMap backingMap) throws LoaderException;

    /**
     * Returns the list of values corresponding to the set of keys passed in.
     * <p>
     * Each key provided will have a corresponding entry in the returned list.
     *
     * If no entry exists for a specified key, the special value
     * <code>Loader.KEY_NOT_FOUND</code> should be returned as the value for
     * that key.  This value exist to distinguish between a <code>null</code>
     * value and a value not being found in the Loader.
     *
     * For a loader in an {@link ObjectMap} that is configured to use a {@link OutputFormat#RAW},
     * the key objects passed will be a {@link SerializedKey} objects. If required, you
     * can use the {@link SerializedKey#getObject()} method to retrieve (possibly inflating
     * the serialized key) the original key object.
     *
     * @param txid the transaction ID object
     *
     * @param keyList List of keys or {@link SerializedKey} objects as described
     *
     * @param forUpdate <code>true</code> when a getForUpdate method is called.
     *                  This parameter can be used to do a "SELECT for UPDATE"
     *                  invocation for proper database locking.
     *
     * @return List of Object values
     *
     * @throws LoaderException if an error occurs during processing of this method
     *
     * @see #KEY_NOT_FOUND
     * @see TxID
     */
    List/*<Object value>*/ get(TxID txid, List keyList, boolean forUpdate)
        throws LoaderException;

    /**
     * Called to tell the Loader to write the provided changes
     * to the backend. The <code>LogSequence</code> parameter is the set of
     * changes that have been applied to the <code>BackingMap</code> associated
     * with this loader.
     *
     * If the <code>BackingMap</code> uses a value interface,
     * the loader can use partial update support for improved performance. If
     * the loader knows the values that were modified, it should update the
     * object returned from <code>LogElement.getCurrentValue()</code> with the
     * value actually in the database. This rule applies when using automatic
     * sequence columns in a database or when using database triggers to modify
     * the value indirectly. Under no circumstances should the committed value
     * be modified.
     *
     * For a loader in an {@link ObjectMap} that is configured to use {@link OutputFormat#RAW}
     * for the keys or value, the keys and values objects in the LogSequence
     * will be {@link SerializedKey} or {@link SerializedValue} objects respectively. If required, you
     * can use the {@link SerializedEntry#getObject()} method to retrieve (possibly inflating
     * the serialized object) the original key or value object.
     * 
     * To override the map's output format configuration, use the {@link PluginOutputFormat} 
     * annotation in the implementation class.
     *
     * @param txid the transaction id object
     * @param sequence the set of changes being applied to this object's map
     *
     * @throws LoaderException if an error occurs during processing of this method
     * @throws OptimisticCollisionException if a version mismatch is detected
     *
     * @see BackingMap#setCopyMode(CopyMode, Class)
     * @see LogElement#getCurrentValue()
     * @see LogSequence
     * @see TxID
     * @see ValueProxyInfo
     */
    void batchUpdate(TxID txid, LogSequence sequence)
        throws LoaderException, OptimisticCollisionException;
}
