package com.ibm.websphere.objectgrid.plugins;

import com.ibm.websphere.objectgrid.ObjectGrid;

/**
 * Listener for ObjectGrid life cycle events.
 * <p>
 * An ObjectGridLifecycleListener implementation may also
 * implement the {@link ObjectGridPlugin} interface in order
 * to receive enhanced ObjectGrid plug-in lifecycle method calls. The plug-in is
 * also required to correctly implement each of the bean methods related to
 * introspection of its state (for example <code>isInitialized()</code>,
 * <code>isDestroyed()</code>, etc).
 *
 * @see EventListener
 * @since 7.1.1
 */
public interface ObjectGridLifecycleListener extends EventListener {

    /**
     * The state of a ObjectGrid's life cycle.
     */
    public enum State {
        /**
         * The ObjectGrid instance has been created.
         */
        NEW,

        /**
         * The ObjectGrid initialization phase is starting.
         * The ObjectGrid and ObjectGrid plug-ins are about to be initialized.
         *
         * A LifecycleFailedException sent during this state transition
         * will result in an abort of the state change and destruction
         * of the ObjectGrid.
         */
        INITIALIZING,

        /**
         * The ObjectGrid initialization phase is complete.
         * All ObjectGrid plug-ins are initialized. The INITIALIZED
         * state may re-occur when shard placement activities (promotion, demotion) occur.
         *
         * All BackingMap plug-ins in the BackingMap instances owned by this
         * ObjectGrid instance have been initialized.
         *
         * A LifecycleFailedException sent during this state transition
         * will result in an abort of the state change and destruction
         * of the ObjectGrid.
         */
        INITIALIZED,

        /**
         * The ObjectGrid instance is being activated for use as a local
         * instance, client instance or as an instance in a primary or replica
         * shard on the server.
         *
         * A LifecycleFailedException sent during this state transition
         * will result in an abort of the state change and destruction
         * of the ObjectGrid.
         */
        STARTING,

        /**
         * The ObjectGrid instance is set to the PRELOAD state by the
         * StateManager API or other configuration.
         *
         * A LifecycleFailedException sent during this state transition
         * will result in an abort of the state change and destruction
         * of the ObjectGrid.
         */
        PRELOAD,

        /**
         * The ObjectGrid instance is ready for work as a
         * local instance, client instance or as an instance in a
         * primary or replica shard on the server.
         *
         * This is the typical steady state of the ObjectGrid.
         */
        ONLINE,

        /**
         * Work is stopping on the ObjectGrid as a result of the
         * StateManager API or other event.
         * No new work will be allowed.
         * End any existing work as soon as possible.
         */
        QUIESCE,

        /**
         * All work is stopped on the ObjectGrid as a result of the
         * StateManager API or other event. No new work will be allowed.
         */
        OFFLINE,

        /**
         * The ObjectGrid instance is starting the destroy phase.
         * ObjectGrid plug-ins for the instance are about to be destroyed.
         *
         * During the destroy phase, all BackingMap instances owned by
         * this ObjectGrid instance are also destroyed.
         */
        DESTROYING,

        /**
         * The ObjectGrid instance, its BackingMap instances and all
         * ObjectGrid plug-ins have been destroyed.
         */
        DESTROYED;

    }

    /**
     * The <code>LifecycleEvent</code> describes the situation that
     * caused a change in the lifecycle of the ObjectGrid.
     */
    static interface LifecycleEvent {

        /**
         * Retrieve the ObjectGrid instance that the event applies to.
         * @return the ObjectGrid instance that the event applies to.
         */
        public ObjectGrid getObjectGrid();

        /**
         * Retrieve the lifecycle state occurring for this event.
         *
         * @return State - the new life cycle state
         */
        public State getState();

        /**
         * Answers true when the ObjectGrid instance is currently readable.  Data can be
         * read from the grid.  A grid is readable for all local and client grid instances
         * that have been initialized. 
         * Server instances are locally readable when they are currently on-line. Replica
         * instances on the server are not necessarily readable depending on the current
         * configuration of the grid.
         */
        public boolean isReadable();

        /**
         * Answers true when the ObjectGrid instance is currently writable.  Data can be
         * modified in the grid.  A grid is writable for all local and client grid instances that
         * have been initialized.
         * Primary shard, server instances are locally writable when they are currently on-line.
         */
        public boolean isWritable();
    };

    /**
     * Invoked when there is a change to the ObjectGrid's life cycle.
     * When LifecycleFailedExceptions are thrown for the following lifecycle
     * states, the ObjectGrid instance will abort the state change and
     * be destroyed: INITIALIZING, INITIALIZED, STARTING, and PRELOAD.
     * Exceptions thrown during other phases will be logged as error conditions.
     * @param event contextual data for the event.
     *
     * @throws LifecycleFailedException thrown when there is a fatal exception. For
     * some states, the ObjectGrid instance will abort the state change and be destroyed.
     *
     */
    public void objectGridStateChanged(LifecycleEvent event) throws LifecycleFailedException;
}
