// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.2 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/ReplicaPreloadController.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:24 
//
// Module  :  ReplicaPreloadController.java
//
// Source File Description: 
//
// See javadoc for this class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M3_B.2 WASX.XD   Oct 15, 2005 bnewport  new code drop 
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.Session;

/**
 * A Map Loader should also implement this interface in the following 
 * circumstances.
 * <p>
 * This interface is useful in a replicated scenario. If a map is being 
 * preloaded and the primary fails before preload is complete, normally the new
 * replica removes all old replicated entries and restarts preload. This process
 * can waste a considerable amount of resources and delay the map coming online.
 * Map Loaders can implement this interface and the replication manager will 
 * call the <code>checkPreloadStatus</code> method to determine what to do when 
 * a replica has become the primary. If the controller determines that the map 
 * is already fully loaded, it should return preloaded already. If it determines 
 * it cannot recover, it should return full preload needed. ObjectGrid clears 
 * the Map and invokes a 'cold' preload. Finally, if it returns partial preload 
 * needed, the controller doesn't clear the map and invokes the normal preload 
 * method. The Loader can then continue with preload.
 * <p>
 * How could an application know whether a preload completely partially or not? 
 * Use an extra Map called "PreloadStatus". The Map has a string key (the map 
 * name) and a value indicating the last block preloaded in to the Map. When 
 * a Loader's preload method is invoked, as part of each transaction, a value
 * should be updated in the status map. This value should hold enough state to 
 * allow a second JVM to continue with preload from this point. It might just 
 * be the last key loaded as an example. When preload is complete, it should 
 * remove the entry.
 * <p>
 * The checkPreloadStatus method when invoked would just need to check the map
 * entry in the status map for this map. If an entry is present, return partial 
 * preload needed. If no entry is present then return preloaded already. If it 
 * can't figure out where to start from (usually means an application bug), full
 * preload needed should be returned.
 * <p>
 * If the map is empty when the replica is promoted to primary, this method 
 * isn't called at all. ObjectGrid will always try to do a full preload. If the 
 * map is not empty, the check method is invoked and its return code indicates 
 * how the Map will be treated.
 * <p>
 * If the application is using a multiple related Maps but is using a single 
 * Loader to preload all the maps together, all Map Loaders should have a 
 * ReplicaPreloadController and they should all return the same value. The
 * 'main' Map can then continue preloading while the other Maps preload methods
 * simply implement a no operation.
 *  
 * @since WAS XD 6.0.1
 * 
 * @see Loader#preloadMap(Session, BackingMap)
 */
public interface ReplicaPreloadController 
{
    /**
     * This class is used as an enumerator for the Map preload status.
     * @since WAS XD 6.0.1
     */
    public static final class Status
    {
        final int value;
        final static private int K_PRELOADED_ALREADY = 0;
        final static private int K_FULL_PRELOAD_NEEDED = 1;
        final static private int K_PARTIAL_PRELOAD_NEEDED = 2;

        private Status(int v)
        {
            value = v;
        }

        /**
         * This constant should be returned when the controller determines that 
         * preloading is not necessary. This status value normally indicates the 
         * side status Map had no entry for this map.
         */
        static public final Status PRELOADED_ALREADY = new Status(K_PRELOADED_ALREADY);

        /**
         * This constant means an application bug has occurred and the controller 
         * cannot determine whether preload has completed or can be recovered. This 
         * status value forces a complete preload.
         */
        static public final Status FULL_PRELOAD_NEEDED = new Status(K_FULL_PRELOAD_NEEDED);

        /**
         * This constant means the controller has determined a preload was in 
         * progress on the primary and that preload can commence from where the
         * primary stopped.
         */
        static public final Status PARTIAL_PRELOAD_NEEDED = new Status(K_PARTIAL_PRELOAD_NEEDED);

        /**
         * @see Object#equals(Object)
         */
        public boolean equals(Object o)
        {
            Status s = (Status)o;
            return value == s.value;
        }
    }

    /**
     * Called when a replica is promoted to primary. The method should return a 
     * status indicating how preload should be handled.
     * 
     * @param session A session to examine the side status Map or any other maps.
     * @param bmap The Map being recovered
     * 
     * @return The status indicating the action to take
     * 
     * @see Status#FULL_PRELOAD_NEEDED
     * @see Status#PARTIAL_PRELOAD_NEEDED
     * @see Status#PRELOADED_ALREADY
     */
    Status checkPreloadStatus(Session session, BackingMap bmap);
}
