// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  BasicMapSerializerPlugin.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Properties;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.objectgrid.config.ConfigPropertyType;
import com.ibm.websphere.objectgrid.config.ObjectGridConfigurationException;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.Association;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.Attribute;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.DataDescriptorException;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.DataDescriptorFactory;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.MapDataDescriptor;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.config.PropertySetter;
import com.ibm.ws.objectgrid.map.MapListenerHandlerHelper;
import com.ibm.ws.xs.util.dopriv.DoPrivUtil;

/**
 * A basic implementation of a MapSerializerPlugin plug-in that can reference an
 * arbitrary KeyDataSerializer and ValueDataSerializer.<p>
 *
 * Use the {@link #setKeySerializerPluginClass(String)} and {@link #setValueSerializerPluginClass(String)}
 * properties to set the key and value data serializer type.<p>
 *
 * Use the {@link #setKeyProperties(String)} and {@link #setValueProperties(String)}
 * properties to set one or more string attributes on the target key or value
 * DataSerializer.<p>
 *
 * Use the {@link #setAssociations(String)} property to describe any associations
 * between the map in which this MapSerializer references and other maps.
 * 
 * Use the {@link #setAddressableKeyName(String)} property to identify the addressable
 * attribute name to use for accessing KeyDataSerializer data when using partial
 * key lookups.  
 * 
 * @since 7.1.1
 */
public class BasicMapSerializerPlugin implements MapSerializerPlugin {
    final static String svClassName = BasicMapSerializerPlugin.class.getName();
    final static TraceComponent tc = Tr.register(svClassName, Constants.TR_GROUP_NAME, Constants.TR_RESOURCE_BUNDLE_NAME);

    private BackingMap map;
    private KeySerializerPlugin serKey;
    private ValueSerializerPlugin serValue;
    private final MapDataDescriptor ddMap;
    private String mapAssociations;
    private String keySerializerClassName = null;
    private String valueSerializerClassName = null;
    private Map<String, String> keyProperties = new LinkedHashMap<String, String>();
    private Map<String, String> valueProperties = new LinkedHashMap<String, String>();

    private enum State {
        NEW, INITIALIZING, INITIALIZED, DESTROYING, DESTROYED
    }
    private State state = State.NEW;

    /**
     * Default constructor.
     */
    public BasicMapSerializerPlugin() {
        ddMap = DataDescriptorFactory.instance().createMapDataDescriptor();
    }

    /**
     * Constructor that sets a KeySerializerPlugin and ValueSerializerPlugin.
     * @param serKey the KeySerializerPlugin or null if not defined.
     * @param serValue the ValueSerializerPlugin or null if not defined.
     */
    public BasicMapSerializerPlugin(KeySerializerPlugin serKey, ValueSerializerPlugin serValue) {
        this.serKey = serKey;
        this.serValue = serValue;
        ddMap = DataDescriptorFactory.instance().createMapDataDescriptor();
    }


    /**
     * Set the String property on the KeyDataSerializer.   The property is in the format:
     * <pre>propertyName=value[,propertyName=value]</pre>.
     *
     * Embedded commas can be specified
     * in the property values by escaping the comma with a back-slash.  Additionally, the "\," can be escaped
     * using one additional "\" characters, allowing a terminal "\" on the value.
     *
     * For example:
     *   <pre>embeddedCommas=val1\,val2\,val3,trailingBackSlash=val4\\,name3=val5</pre>.
     * The properties are not flushed to the ValueDataSerializeer until it is initialized.  After
     * that time, the properties cannot be set.
     *
     * @param property a non-null, comma delimited set of name/value pairs to set on the
     * KeyDataSerializer
     */
    public void setKeyProperties(String property) {
        internalSetProperties(property, true);
    }

    /**
     * Retrieve the String property from the KeyDataSerializer.
     *
     * @return the properties set with the setKeyProperties method.
     */
    public String getKeyProperties() {
        return internalGetProperties(true);
    }

    /**
     * Set the String properties on the ValueDataSerializer.  The property is in the format:
     * <pre>propertyName=value[,propertyName=value]</pre>.
     *
     * Embedded commas can be specified
     * in the property values by escaping the comma with a back-slash.  Additionally, the "\," can be escaped
     * using one additional "\" characters, allowing a terminal "\" on the value.
     *
     * For example:
     *   <pre>embeddedCommas=val1\,val2\,val3,trailingBackSlash=val4\\,name3=val5</pre>.
     *
     * The properties are not flushed to the ValueDataSerializeer until it is initialized.  After
     * that time, the properties cannot be set.
     *
     * @param properties a non-null, comma delimited set of name/value pairs to set on the
     * ValueDataSerializer
     */
    public void setValueProperties(String properties) {
       internalSetProperties(properties, false);
    }

    /**
     * Retrieve the String property from the ValueDataSerializer.
     * @return the properties set with the setValueProperties method.
     */
    public String getValueProperties() {
        return internalGetProperties(false);
    }

    /**
     * The property is in the form: <name>=<value>,<name>=<value>,<name>=<val\,val>
     */
    private void internalSetProperties(String propertyIn, boolean key) {
        checkPreInitializedState();
        if (propertyIn == null)
            throw new NullPointerException("Property must not be null");
        Map<String, String> propsMap = key?keyProperties:valueProperties;

        // Replace two backslash characters followed by a separator with
        // one backslash character, a space, and the separator. This (since we do a trim
        // later on the values) will allow the value to have a terminal \ without escaping
        // the comma as a literal comma in the value. backslash gets weird because
        // its escaped once for the string literal and once for the regexp.
        propertyIn = propertyIn.replaceAll("\\\\\\\\,", "\\\\ ,");

        String[] props = propertyIn.split(",");
        StringBuilder buffer = new StringBuilder();
        for (int p=0; p<props.length; ++p) {
            if (props[p].length()==0) {
                // Ignore this two commas next to each other.
                continue;
            }
            buffer.append(props[p]);
            // Ending with backslash, but not two backslash
            if (props[p].endsWith("\\")) {
                // A literal comma was inserted, and we split on it, remove the slash, add
                // the comma back and go to the next token
                buffer.setLength(buffer.length()-1);
                buffer.append(",");
                continue;
            }

            String property = buffer.toString();
            buffer.setLength(0);
            // Don't use split. Values can have = in them.
            int idx = property.indexOf('=');
            if (idx==-1) {
                throw new DataDescriptorException("Invalid property format.  Missing '=' symbol for property: " + property);
            }
            String name=property.substring(0, idx).trim();
            if (name.contains(",") || name.contains("\\")) {
                throw new DataDescriptorException("Invalid property name. Properties cannot have a , or \\ in the name: " + name);
            }
            String value=property.substring(idx+1).trim();
            value = value.replaceAll("\\\\,", ",");
            propsMap.put(name, value);
        }
    }

    private String internalGetProperties(boolean key) {
        Map<String, String> propsMap = key?keyProperties:valueProperties;
        StringBuilder sb = new StringBuilder();

        Iterator<Map.Entry<String, String>> entries = propsMap.entrySet().iterator();
        while (entries.hasNext()) {
            Map.Entry<String, String>  entry = entries.next();
            String val = entry.getValue();
            if (val.contains(",")) {
                val = val.replaceAll(",", "\\,");
            }

            sb.append(entry.getKey());
            sb.append("=");
            sb.append(val);
            if (entries.hasNext()) {
                if (val.endsWith("\\")) {
                    // Add another slash to the end so that we don't accidentally
                    // escape the property delimiter comma.
                    sb.append("\\");
                }
                sb.append(",");
            }
        }
        return sb.toString();
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.io.MapSerializerPlugin#getMapDataDescriptor()
     */
    public MapDataDescriptor getMapDataDescriptor() {
        return ddMap;
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.io.MapSerializerPlugin#getKeySerializerPlugin()
     */
    public KeySerializerPlugin getKeySerializerPlugin() {
        return serKey;
    }

    /**
     * Used to programmatically set the KeySerializerPlugin for local ObjectGrid instances.
     * @param keySerializerPlugin the KeySerializerPlugin instance.
     */
    public void setKeySerializerPlugin(KeySerializerPlugin keySerializerPlugin) {
        checkPreInitializedState();
        this.serKey = keySerializerPlugin;
    }

    /**
     * Sets the name of the KeySerializerPlugin class.
     * @param serClass the name of the KeySerializerPlugin class.
     */
    public void setKeySerializerPluginClass(String serClass) {
        checkPreInitializedState();
        keySerializerClassName = serClass;
    }

    /**
     * Retrieves the name of the {@linkplain KeySerializerPlugin} class that was set with the
     * {@linkplain #setKeySerializerPluginClass(String)} method.
     * @return the name of the {@linkplain KeySerializerPlugin} class
     */
    public String getKeySerializerPluginClass() {
        return keySerializerClassName;
    }

    /**
     * Sets the name of the {@linkplain ValueSerializerPlugin} class.
     * @param serClass the name of the {@linkplain ValueSerializerPlugin} class.
     */
    public void setValueSerializerPluginClass(String serClass) {
        checkPreInitializedState();
        valueSerializerClassName = serClass;
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.io.MapSerializerPlugin#getValueSerializerPlugin()
     */
    public ValueSerializerPlugin getValueSerializerPlugin() {
        return serValue;
    }

    /**
     * Retrieves the name of the ValueDataSerializer class that was set with the
     * {@linkplain #setValueSerializerPluginClass(String)}.
     * @param valueSerializerPlugin the ValueSerializerPlugin instance.
     */
    public void setValueSerializerPlugin(ValueSerializerPlugin valueSerializerPlugin) {
        checkPreInitializedState();
        this.serValue = valueSerializerPlugin;
    }

    // TODO DS: How to escape the paths if the path separater is a comma or semicolon?  Use quotes or double them?
    /**
     * Associations are a set of comma-delimited properties that identify relationships from
     * object attributes in map entries in the BackingMap associated with this MapSerializer (the source map)
     * to map entries in another map (the target map).
     *<p>
     * I.e. The object attributes hold key values for entries in another map.
     *<p>
     * Each property is defined as follows:
     *
     * <table>
     * <tr><th align="left">Property Name</th><th align="left">Description</th></tr>
     * <tr><td>attribute</td><td>The name or path to the source attribute that holds the target map entry's key
     * (the foreign key). If the attribute is an attribute on a key object, it must be prefixed by the
     * addressable key name and the path separator (by default "key.").
     * If the attribute is stored inside other embedded attributes, then attribute name must include the
     * entire path to the key, using the DataSerializer's path separator.</td></tr>
     * <tr><td>targetMap</td><td>The name of the associated map which contains the target map entry</td></tr>
     * </table><p>
     *
     * For example, consider the MapSerializerPlugin associated with the ParentMap, which has
     * a one-to-many bi-directional relationship to the ChildMap and a uni-directional
     * one-to-one relationship with the AddressMap map.
     * <p>
     * The relationships might be defined as follows: <p>
     * ParentMap:
     * <code>
     * "targetMap=ChildMap,attribute=childrenKeys,inverseAttribute=parentKey"
     * </code><p>
     *
     * ChildMap:
     * <code>
     * "targetMap=ParentMap,attribute=parentKey,inverseAttribute=childrenKeys"
     * </code>
     *
     * The foreign key to the ChildMap may be stored in an embedded attribute:<P>
     * ParentMap:
     * <code>
     * "targetMap=ChildMap,attribute=embeddedType/childrenKeys"
     * </code><p>
     *
     * @param mapAssociations the list of map associations.
     */
    public void setAssociations(String mapAssociations) {
        checkPreInitializedState();
        this.mapAssociations = mapAssociations;
    }

    /**
     * Retrieve the associations for this MapSerializer.
     *
     * @see #setAssociations(String)
     * @return the list of map associations.
     */
    public String getAssociations() {
        return mapAssociations;
    }

    /**
     * Set the addressableKeyName property for the MapDataDescriptor used by this MapSerializer.  The addressableKeyName property 
     * allows you to change the default name that is used when navigating to key fields in a query or index.  The default 
     * addressibleKeyName is: "key".  <br>
     * <br>
     * For example, suppose that a key in your map contains two fields named x and y.  If you only want to query field x, then
     * the query might look like : <code>"select c.key.x from Customer c"</code> <br>
     * If you change the addressableKeyName to "myKey", then the above query must be re-written as <code>"select c.myKey.x from Customer c"</code>
     * 
     * @param addressableKeyName the addressable attribute name to use for accessing KeyDataSerializer data when navigating to key attributes in queries
     * and indexes. 
     * @since 8.5
     */
    public void setAddressableKeyName(String addressableKeyName ) {
        checkPreInitializedState();
        this.ddMap.setAddressableKeyName(addressableKeyName);
    }
    
    /**
     * Retrieve the addressable attribute name for the KeyDataSerializer for this MapSerializer
     * @return the addressable attribute name to use for accessing KeyDataSerializer data when navigating to key attributes in queries
     * and indexes. 
     * @see #setAddressableKeyName(String) 
     */
    public String getAddressableKeyName()
    {
        return this.ddMap.getAddressableKeyName();
    }
   
   
    /**
     * @see com.ibm.websphere.objectgrid.plugins.BackingMapPlugin#destroy()
     */
    public void destroy() {
        state = State.DESTROYING;
        if(serKey != null && serKey.isDestroyed() == false)
            serKey.destroy();
        if(serValue != null && serValue.isDestroyed() == false)
            serValue.destroy();
        state = State.DESTROYED;
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.Destroyable#isDestroyed()
     */
    public boolean isDestroyed() {
        return state == State.DESTROYED;
    }

    /**
     * Perform initialization of the map serializer, including but not limited to
     * the following steps.
     * <ul>
     * <li>Creating the key serializer and value serializer instances if
     * the {@link #setKeySerializerPluginClass(String) } or {@link #setValueSerializerPluginClass(String) }
     * properties were set.
     * <li>Initializing the serializer instances using the appropriate properties as
     * specified by {@link #setKeyProperties(String)} or {@link #setValueProperties(String)}
     * <li>Initialize any associations as set by {@link #setAssociations(String)}
     * </ul>
     * @see com.ibm.websphere.objectgrid.plugins.Initializable#initialize()
     */
    public void initialize() throws ObjectGridRuntimeException {
        if(tc.isEntryEnabled()) {
            Tr.entry(tc, "BasicMapSerializerPlugin.initialize", map);
        }
        state = State.INITIALIZING;
        try {
            if(keySerializerClassName != null) {
                // If running in a container in an OSGi environment or in WebSphere need to check the context classloader first.
                // In OSGi the thread context classloader has the bundle classloader, and in WAS it is the application classloader.
                try {
                    serKey = (KeySerializerPlugin) DoPrivUtil.contextClassLoaderForName(keySerializerClassName, false).newInstance();
                } catch (ClassNotFoundException cnfe) {
                    serKey = (KeySerializerPlugin) DoPrivUtil.forName(keySerializerClassName, false).newInstance();
                }
            }
            if(serKey != null) {
                internalSetProperties(serKey, keyProperties);
                MapListenerHandlerHelper.fireSingleInitialize(map, serKey);
            }
        } catch (Exception e) {
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, "initialize BasicMapSerializerPlugin key serializer encountered an unexpected exception", e);
            }
            if (e instanceof ObjectGridRuntimeException) {
                throw (ObjectGridRuntimeException)e;
            }
            throw new ObjectGridRuntimeException("Failed to initialize KeySerializer for map: " + (map==null?"null":map.getName()), e);
        }
        try {
            if(valueSerializerClassName != null) {
                // If running in a container in an OSGi environment or in WebSphere need to check the context classloader first.
                // In OSGi the thread context classloader has the bundle classloader, and in WAS it is the application classloader.
                try {
                    serValue = (ValueSerializerPlugin) DoPrivUtil.contextClassLoaderForName(valueSerializerClassName, false).newInstance();
                } catch (ClassNotFoundException cnfe) {
                    serValue = (ValueSerializerPlugin) DoPrivUtil.forName(valueSerializerClassName, false).newInstance();
                }
            }
            if(serValue != null) {
                internalSetProperties(serValue, valueProperties);
                // Optionally set backing map and safely initialize
                MapListenerHandlerHelper.fireSingleInitialize(map, serValue);
            }
        } catch (Exception e) {
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, "initialize BasicMapSerializerPlugin value serializer encountered an unexpected exception ", e);
            }
            if (e instanceof ObjectGridRuntimeException) {
                throw (ObjectGridRuntimeException)e;
            }
            throw new ObjectGridRuntimeException("Failed to initialize ValueSerializer for map: " + (map==null?"null":map.getName()), e);
        }
        HashMap<String, Association> associations;
        try {
            associations = parseAssociations(mapAssociations);
        } catch (Exception e) {
            FFDCFilter.processException(e, "initialize", "437");
            String msg = "Failed to initialize BasicMapSerializerPlugin associations for map: "
                + map.getName() + ", mapAssociations=\""
                + mapAssociations + "\"";
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, msg, e);
            }
            if (e instanceof ObjectGridRuntimeException) {
                throw (ObjectGridRuntimeException)e;
            }
            throw new ObjectGridRuntimeException(msg, e);
        } finally {
            if(tc.isEntryEnabled()) {
                Tr.exit(tc, "BasicMapSerializerPlugin.initialize", map);
            }
        }
        ddMap.setAssociations(associations);
        state = State.INITIALIZED;
    }

    private void internalSetProperties(DataSerializer ds, Map<String, String> properties) {
        checkPreInitializedState();
        for(Map.Entry<String, String> entry : properties.entrySet()) {
            try {
                PropertySetter.setProperty(ds, entry.getKey(), ConfigPropertyType.STRING_JAVA_LANG, entry.getValue(), true);
            } catch (ObjectGridConfigurationException e) {
                throw new ObjectGridRuntimeException("Unexpected exception setting a DataSerializer property: " + e.getMessage(), e );
            }
        }
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.Initializable#isInitialized()
     */
    public boolean isInitialized() {
        return state == State.INITIALIZED;
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.BackingMapPlugin#setBackingMap(com.ibm.websphere.objectgrid.BackingMap)
     */
    public void setBackingMap(BackingMap map) {
        checkPreInitializedState();
        this.map = map;
    }

    /**
     * @see com.ibm.websphere.objectgrid.plugins.BackingMapPlugin#getBackingMap()
     */
    public BackingMap getBackingMap() {
        return map;
    }

    /**
     * This method is invoked during initialization, converting the association string
     * to a set of associations used to initialize the MapDataDescriptor.
     * @param associationString the string form of the associations.
     * @return the Set of Association instances used to set on the MapDataDescriptor
     */
    private HashMap<String, Association> parseAssociations(String associationString) throws DataDescriptorException {
        try {
            if (tc.isEntryEnabled()) {
                Tr.entry(tc, "BasicMapSerializerPlugin.parseAssociations entry");
            }
            HashMap<String, Association> associations = new HashMap<String, Association>();
            if(associationString == null) {
                return associations;
            }
            Map<String, Attribute> valueAttributes = Collections.emptyMap();
            if(serValue != null) {
                valueAttributes = serValue.getValueDataDescriptor().getAttributes();
            }
            Map<String, Attribute> keyAttributes = Collections.emptyMap();
            if(serKey!=null) {
                keyAttributes = serKey.getKeyDataDescriptor().getAttributes();
            }

            // These guys are not user data, they're attribute names, map names, etc, they can't
            // have special characters at least they don't have embedded ; and , no need to escape them.
            String assocDefs[] = associationString.split(";");

            for (String assocDef : assocDefs) {
                assocDef = assocDef.trim();
                if (assocDef.length() == 0) {
                    // Allow new lines and empty values.
                    continue;
                }
                String properties[] = assocDef.split(",");

                Properties  props = new Properties();
                for (String property : properties) {
                    property = property.trim();
                    if (property.length() == 0) {
                        // Allow newlines and empty values.
                        continue;
                    }
                    // Don't use split. Values can have = in them.
                    int idx = property.indexOf('=');
                    if (idx==-1) {
                        String msg = "Invalid property format for BasicMapSerializerPlugin.  Missing '=' symbol for property: " + property +
                        " in the association specified as: \"" + assocDef + "\"";
                        if (tc.isDebugEnabled()) {
                            Tr.debug(tc, msg);
                        }
                        throw new DataDescriptorException(msg);
                    }
                    String name=property.substring(0, idx).trim();
                    name = name == null ? null : name.trim();
                    if (name == null || name.length() == 0) {
                        String msg = "Invalid property name for BasicMapSerializerPlugin. Property names cannot be empty in the association specified as: \"" +
                        assocDef + "\"";
                        if (tc.isDebugEnabled()) {
                            Tr.debug(tc, msg);
                        }
                        throw new DataDescriptorException(msg);
                    }
                    String value=property.substring(idx+1).trim();
                    value = value == null ? null : value.trim();
                    if (value == null || value.length() == 0) {
                        String msg = "Invalid property value for BasicMapSerializerPlugin. Property values cannot be empty in the association specified as: \"" +
                        assocDef + "\"";
                        if (tc.isDebugEnabled()) {
                            Tr.debug(tc, msg);
                        }
                        throw new DataDescriptorException(msg);
                    }
                    props.put(name, value);
                }

                String attrName = props.getProperty("attribute");
                if (attrName == null) {
                    String msg = "Invalid association string for BasicMapSerializerPlugin. Missing attribute parameter in the association specified as: \"" +
                    assocDef + "\"";
                    if (tc.isDebugEnabled()) {
                        Tr.debug(tc, msg);
                    }
                    throw new DataDescriptorException(msg);
                }
                // Find the attributes in our serializer metadata
                String keyPrefix = serKey == null?null:getMapDataDescriptor().getAddressableKeyName().concat(serKey.getKeyDataDescriptor().getPathSeparator());
                Attribute attr = valueAttributes.get(attrName);
                if(attr == null) {
                    // It's not the value.  See if it has the key prefix and it the attribute exists
                    if (keyPrefix != null && attrName.startsWith(keyPrefix)) {
                        String keyAttrName = attrName.substring(keyPrefix.length());
                        attr = keyAttributes.get(keyAttrName);
                    }
                    if(attr == null) {
                        String msg = "Invalid association string for BasicMapSerializerPlugin. Source attribute: " + attrName + " is not defined in the KeyDataDescriptor or ValueDataDescriptor.";
                        if (tc.isDebugEnabled()) {
                            Tr.debug(tc, msg);
                        }
                        throw new DataDescriptorException(msg);
                    }
                }

                String mapName = props.getProperty("targetMap");
                if (mapName == null) {
                    String msg = "Invalid association string for BasicMapSerializerPlugin. Missing targetMap parameter in the association specified as: \"" + assocDef + "\"";
                    if (tc.isDebugEnabled()) {
                        Tr.debug(tc, msg);
                    }
                    throw new DataDescriptorException(msg);
                }

                associations.put(attr.getAttributeName(), DataDescriptorFactory.instance().createAssociation(attr.getAttributeName(), mapName));
            }
            return associations;
        }
        finally {
            if (tc.isEntryEnabled()) {
                Tr.exit(tc, "BasicMapSerializerPlugin.parseAssociations done");
            }
        }
    }

    private void checkPreInitializedState() {
        if(state != State.NEW && state != State.INITIALIZING) {
            throw new IllegalStateException("BasicMapSerialier already initialized.  Current state: " + state);
        }
    }
}
