// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  DataSerializer.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io;

import java.io.IOException;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.io.XsDataInputStream;
import com.ibm.websphere.objectgrid.io.XsDataOutputStream;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.DataObjectContext;

/**
 * Includes methods to serialize and inflate a data objects.<p>
 *
 * Implement the child interfaces to add additional functionality to the DataSerializer.<p>
 * The serialized form typically needs the data type and/or attribute encoded
 * into it so that single attributes can be extracted from the serialized form.<p>
 *
 * Use the {@link MapSerializerPlugin}, {@link KeySerializerPlugin} and
 * {@link ValueSerializerPlugin} interfaces to create custom DataSerializers that can be used
 * to interact with data objects stored in {@link BackingMap}s.
 * @since 7.1.1
 */
public interface DataSerializer  {

    /**
     * Serialize the specified data object.
     * <p>
     * Do not store or use the data stream beyond the confines of this
     * method call. The data stream is owned by the WebSphere eXtreme Scale
     * framework and my be closed, pooled or reused as required.
     *
     * @param ctx the Context of the method call.
     * @param dataObject the object to serialize.  The object may be null.
     * @param dataObjectOutput the data output stream to receive the serialized form of the object.
     * @throws IOException thrown if there is a problem serializing the data.
     */
    public void serializeDataObject(DataObjectContext ctx,
                                    Object dataObject,
                                    XsDataOutputStream dataObjectOutput) throws IOException;

    /**
     * Inflate a data object from the specified data input stream.
     *
     * <p>
     * Do not store or use the data stream beyond the confines of this
     * method call. The data stream is owned by the WebSphere eXtreme Scale
     * framework and my be closed, pooled or reused as required.
     *
     * @param ctx the Context of the method call.
     * @param dataObjectInput the data input stream to read the object, produced from
     * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}.
     * Must not be null.
     * @return the inflated object.
     * @throws IOException thrown if there is a problem reading the data.
     */
    public Object inflateDataObject(DataObjectContext ctx,
                                    XsDataInputStream dataObjectInput) throws IOException;
    /**
     * DataSerializers implement this interface to allow inflating attributes from the serialized
     * form of a data object.  Attribute path strings are passed to the {@link #getAttributeContexts(String...)} method,
     * returning a user defined context object.  The context object is cached and passed to the
     * {@link #inflateDataObjectAttributes(DataObjectContext, XsDataInputStream, Object)} method to
     * inflate the specified attributes.<p>
     *
     * For each call to {@link #getAttributeContexts(String...)} for a set of attribute paths,
     * there will typically be more than one call to {@link #inflateDataObjectAttributes(DataObjectContext, XsDataInputStream, Object)}.
     * The {@link #getAttributeContexts(String...)} can, therefore, be used to optimize
     * calculation, allocation or other processing that might normally be done during the call
     * to {#link {@link #inflateDataObjectAttributes(DataObjectContext, XsDataInputStream, Object)}.
     *
     * This mix-in interface is required when using ObjectQuery or the HashIndex plug-in.<p>
     *
     * @since 7.1.1
     */
    public interface DataAttributeInflatable extends DataSerializer {

        /**
         * Retrieve a context from the specified attribute paths.  This method is invoked
         * prior to the call to {@link DataAttributeInflatable#inflateDataObjectAttributes(DataObjectContext, XsDataInputStream, Object)}
         * and is used to identify the paths that will be inflated.  The resulting attribute context object is passed
         * to the inflateDataObjectAttributes method as-is.<p>
         *
         * This is useful for returning optimized data structures for the implementation.  For example, returning
         * int[] instead of String[] to identify each attribute to extract.
         *
         * @param attributePaths the non-null strings that identify attributes that this DataSerializer can
         * inflate.
         * @return an attribute context.  This object will be passed to the
         * {@link DataAttributeInflatable#inflateDataObjectAttributes(DataObjectContext, XsDataInputStream, Object)}
         * method of the same DataSerializer instance.  The resulting object must
         * not be null.
         */
        Object getAttributeContexts(String... attributePaths);

        /**
         * From the specified data input stream, inflate the specified attributes.<p>
         *
         * If the attribute doesn't exist, {@link SpecialValue#NOT_FOUND} must be returned.<p>
         *
         * Attributes with isCollection() set to true, must return Collections.  Collections of primitive
         * objects must return primitive type wrapper collections.
         * For example:  Collection<Byte>
         * <p>
         * Embedded attributes are returned as the native data type, such that it can be used
         * directly with the {@link #serializeDataObject(DataObjectContext, Object, XsDataOutputStream)} method..<p>
         * For example:
         * <pre>
         * Person {
         *   Name: String,
         *   Address : {
         *     Street: String,
         *     City: String,
         *     State: String
         *   }
         * }
         * </pre>
         * Object attrCtx = getAttributeContext("Address");
         * inflateDataObjectAttributes(..., attrCtx) would return:
         * <pre>
         * {
         *   Street: String,
         *   City: String,
         *   State: String
         * }
         * </pre>
         * Object attrCtx = getAttributeContext("Name");
         * inflateDataObjectAttributes(..., attrCtx) would return:
         * "John Doe"
         *
         * <p>
         * Do not store or use the data stream beyond the confines of this
         * method call. The data stream is owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param dataObjectContext the context of the method call.
         * @param dataObjectInput the data input stream to read the object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}.
         * Must not be null.
         * @param attributeContexts the context of the attributes to retrieve from the input stream.   Must not be null.
         * @return an array of attribute values, one value for each attribute
         * included in the specified context, or {@link SpecialValue#NOT_FOUND} for any values not found.
         * @throws IOException thrown if there is a problem reading the data.
         */
        public Object[] inflateDataObjectAttributes(DataObjectContext dataObjectContext,
                                                    XsDataInputStream dataObjectInput,
                                                    Object attributeContexts) throws IOException;

    }

    /**
     * DataSerializers implement this optional interface to allow a customized human readable
     * representation of a data object.
     * @since 7.1.1
     */
    public interface UserReadable extends DataSerializer {

        /**
         * Return a user-readable form of the data object.  Implementations can choose to mask out
         * sensitive data, such as passwords.
         *
         * <p>
         * Do not store or use the data stream beyond the confines of this
         * method call. The data stream is owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param ctx the Context of the method call.
         * @param dataObjectInput the data input stream to read the object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}.
         * Must not be null.
         * @return the String representation of the data object or null if the default representation should be used.
         * @throws IOException thrown if there is a problem reading the data.
         */
        public String toStringDataObject(DataObjectContext ctx,
                                         XsDataInputStream dataObjectInput) throws IOException;
    }

    /**
     * DataSerializers implement this interface to allow an alternative identity.  This
     * interface is most useful on a KeyDataSerializer, when the serialized form of
     * the data includes formatting or other data inconsistent with the key's identity.<p>
     *
     * If omitted from the DataSerializer plug-in or if the {@link #hasBinaryEquality()}
     * answers true, then the serialized form of the data is used to
     * calculate a hash code and test for equality.
     * @since 7.1.1
     */
    public interface Identifiable extends DataSerializer {
        /**
         * Answers <code>true</code> when the serialized form of the
         * data object can be used directly to calculate equality and the hashCode.
         * When true, the caller can skip calling the hashCode and equals method on this DataSerializer
         * and use the entire serialized form as the identity.<p>
         *
         * If the serialized form of the identity includes unstructured data or meta-information
         * that isn't part of the identity, this method must answer false.
         *
         * A DataSerializer must return consistent answers for hasBinaryEquality()
         * for the same type of object.
         * When this method returns true, the caller calculates the hashCode based
         * on its own algorithm. When this method returns false,
         * the {@link Identifiable#hashCode(DataObjectContext, XsDataInputStream)} method is used to calculate the hashCode using
         * a different algorithm, thereby breaking the contract for {@link Object#hashCode()}.
         *
         * @return false if the hashCode and equals method should always be called on this serializer, or
         * true, if the hashCode and equals can optionally be invoked.
         */
        public boolean hasBinaryEquality();
        /**
         * For the serialized data object, retrieve the hash code for that object.<p>
         *
         * Any two objects which answer <code>true</code> when passed to <code>.equals</code>
         * must answer the same value for this method.
         *
         * <p>
         * Do not store or use the data stream beyond the confines of this
         * method call. The data stream is owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param ctx the Context of the method call.
         * @param dataObjectInput the data input stream to read the object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}.
         * Must not be null.
         * @return the hashCode of the serialized data object.
         * @throws IOException thrown if there is a problem reading the data.
         */
        public int hashCode(DataObjectContext ctx,
                            XsDataInputStream dataObjectInput) throws IOException;

        /**
         * Answers true if the two serialized data objects are equivalent.
         * <p>
         * Do not store or use the data streams beyond the confines of this
         * method call. The data streams are owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param ctx the context of the method call.
         * @param dataObjectInput1 the data input stream to read the object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}.
         * Must not be null.
         * @param dataObjectInput2 the data input stream to read the object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}.
         * Must not be null.
         * @return true if the two serialized data objects are equivalent
         * @throws IOException thrown if there is a problem reading the data.
         */
        public boolean equals(DataObjectContext ctx,
                              XsDataInputStream dataObjectInput1,
                              XsDataInputStream dataObjectInput2) throws IOException;
    }

    /**
     * Specialty values used by {@link DataAttributeInflatable#inflateDataObjectAttributes(DataObjectContext, XsDataInputStream, Object)}.
     */
    public enum SpecialValue {
        /**
         * A value for identifying an attribute that cannot be found.
         */
        NOT_FOUND
    }
}