// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ValueDataSerializer.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      7/10/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io;

import java.io.IOException;

import com.ibm.websphere.objectgrid.LockStrategy;
import com.ibm.websphere.objectgrid.io.XsDataInputStream;
import com.ibm.websphere.objectgrid.io.XsDataOutputStream;
import com.ibm.websphere.objectgrid.plugins.Loader;
import com.ibm.websphere.objectgrid.plugins.LogElement;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer.DataAttributeInflatable;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.ValueDataDescriptor;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.DataObjectContext;

/**
 * DataSerializers implement this interface to serialize and inflate value data
 * objects and attributes.<p>
 *
 * Several mix-in interfaces are provided to allow additional capabilities for values.
 * @since 7.1.1
 */
public interface ValueDataSerializer extends DataSerializer, DataAttributeInflatable {

    /**
     * Retrieve the metadata for the objects that this DataSerializer understands.
     * @return the ValueDataDescriptor metadata, or null if there is no data descriptor.
     */
    public ValueDataDescriptor getValueDataDescriptor();

    /**
     * ValueDataSerializers implement the optional Mergeable interface when it is
     * possible that the clients or replicas in the grid may have multiple versions
     * of the serializer.  The Mergeable interface allows the VersionDataSerializer
     * to merge the two serialized forms when the data is updated, to prevent
     * data loss.
     * @since 7.1.1
     */
    public interface Mergeable extends ValueDataSerializer {

        /**
         * Used by the {@link Mergeable#mergeDataObjects(DataObjectContext, XsDataInputStream, XsDataInputStream, XsDataOutputStream) }
         * method to indicate the form of the data object's merged result.
         *
         * @since 7.1.1
         */
        public enum MergeType {
            /**
             * The data object should be merged. The serialized form of the
             * resulting merged object is used from the data output stream
             * is stored in the cache.
             */
            MERGE,
            /**
             * The data object should not be merged. The new object should
             * be used as is.<p>
             *
             * This is useful for data types that do not require merging when there
             * is no chance that the objects contain disparate sets of attributes,
             * all of which must be preserved.
             */
            USE_NEWVALUE
        }


        /**
         * This method is invoked when updating a cache value.  If the existing object contains
         * data that the current DataSerializer doesn't recognize, the two data streams can be combined
         * into the resulting merged data output stream.
         * <p>
         * Do not store or use the data streams beyond the confines of this
         * method call. The data streams are owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param ctx the context of the method call.
         * @param existingDataObjectInputStream the data input stream to read the old object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)} before the update.
         * Must not be null.
         * @param updatedDataObjectInputStream the data input stream to read the old object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)} before the update.
         * Must not be null.
         * @param mergedDataObjectOutputStream the data output stream to store the merged object, compatible with
         * that produced from {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}
         * for the merged object. Ignored if the return value is not {@link MergeType#MERGE}.
         * Must not be null.
         * @return One of the MergeType attributes representing the form
         * of the object merge. Data in the mergedObject stream is ignored
         * if the value is other than {@link MergeType#MERGE}
         * @throws IOException thrown if there is a problem reading or writing the data.
         */
        public MergeType mergeDataObjects(DataObjectContext ctx,
                                          XsDataInputStream existingDataObjectInputStream,
                                          XsDataInputStream updatedDataObjectInputStream,
                                          XsDataOutputStream mergedDataObjectOutputStream) throws IOException;
    }

    /**
     * ValueDataSerializers can implement the optional Versionable interface to allow high
     * performance versioning when using in conjunction with the optimistic locking
     * strategy.<p>
     *
     * Optimistic versioning is used to identify the version of a data object when using
     * the {@link LockStrategy#OPTIMISTIC} locking strategy.  The Versioning interface
     * allows the eXtreme Scale runtime to update the version in the data object
     * and read and compare the version whenever updates occur in one of the
     * cache tiers:  the transactional, client or server cache tiers.<p>
     * 
     * The Versionable interface can also allow the customer to enable optimistic locking
     * without versioning by returning the {@linkplain VersionType#NO_VERSION} value from the 
     * {@linkplain #getVersion(DataObjectContext, XsDataInputStream, XsDataOutputStream)} method.<p>
     *
     * @since 7.1.1
     */
    public interface Versionable extends ValueDataSerializer {

        /**
         * The VersionType enumeration indicates the type of version information
         * that the Versionable data serializer exports from the {@link Versionable#getVersion(DataObjectContext, XsDataInputStream, XsDataOutputStream)}
         * method.
         *
         * @since 7.1.1
         */
        public enum VersionType {
            /**
             * The data object should be versioned using the entire serialized form
             * of the data object.<p>
             *
             * This is useful when the data object does not contain a version field, but
             * optimistic version checking is required.
             */
            DEFAULT_VERSION,
            /**
             * The data object should not be versioned.  The eXtreme Scale runtime
             * will not perform any version checking on the object.<p>
             *
             * This is useful for data types that do not require versioning when there
             * is no chance for a collision.
             */
            NO_VERSION,
            /**
             * The serialized form of the version attribute is written to the data output stream.
             * The eXtreme Scale runtime will use the serialized form of the version data
             * for version checking or access via the {@link Versionable#versionEquals(DataObjectContext, XsDataInputStream, XsDataInputStream)}
             * and {@link Versionable#inflateDataObjectVersion(DataObjectContext, XsDataInputStream)} methods.
             */
            USER_VERSION,
        }

        /**
         * Retrieve the serialized form of the attribute that identifies
         * the version.<p>
         *
         * This method is called whenever a data object is associated with a transaction
         * so that at commit time, the ObjectGrid runtime can do qualified updates
         * using this value to detect if the value changed during the optimistic
         * transaction. The <code>Loader</code> also gets this value from the
         * <code>LogElement</code>.<p>
         *
         * This method is called prior to the other {@linkplain Versionable} methods.  If this method
         * returns null or {@link VersionType#NO_VERSION}, then the cache entry will abstain from versioning.
         * <p>
         * Do not store or use the data streams beyond the confines of this
         * method call. The data streams are owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param ctx the context of the method call.
         * @param existingDataObjectInput the data input stream to read the object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}.
         * Must not be null.
         * @param versionDataOutput the data output stream to receive the serialized
         * form of the attributes that represent the version. Ignored if the return
         * value is not {@link VersionType#USER_VERSION}
         * Must not be null.
         * @return One of the VersionType attributes representing the form
         * of the object version. Data in the versionOutput stream is ignored
         * if the value is other than {@link VersionType#USER_VERSION}
         * @throws IOException thrown if there is a problem reading or writing the data.
         * @see Loader
         * @see LogElement#getVersionedValue()
         */
        public VersionType getVersion(DataObjectContext ctx,
                                      XsDataInputStream existingDataObjectInput,
                                      XsDataOutputStream versionDataOutput) throws IOException;

        /**
         * Invoked when the version of the cache entry needs to be updated to
         * the next version. If the default version is being used (the entire object),
         * the updateVersion() method should return {@link VersionType#DEFAULT_VERSION}
         * or {@link VersionType#NO_VERSION}.
         * If the updateVersion() method writes the new object data (with the updated
         * version attribute) to the output stream, the method should return {@link VersionType#USER_VERSION};
         * <p>
         * Do not store or use the data streams beyond the confines of this
         * method call. The data streams are owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param ctx the context of the method call.
         * @param existingDataObjectInput the data input stream to read the existing object, produced from
         * {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}.
         * Must not be null.
         * @param updatedDataObjectOutput the data output stream to store the updated object, compatible with
         * that produced from {@link DataSerializer#serializeDataObject(DataObjectContext, Object, XsDataOutputStream)}
         * for an object of the subsequent version from the input object. Ignored if
         * the return value is other than {@link VersionType#USER_VERSION}.
         * Must not be null.
         * @return type - {@link VersionType#USER_VERSION} if the object data was updated
         * @throws IOException thrown if there is a problem reading or writing the data.
         */
        public VersionType updateVersion(DataObjectContext ctx,
                                         XsDataInputStream existingDataObjectInput,
                                         XsDataOutputStream updatedDataObjectOutput) throws IOException;

        /**
         * Answers true if the two serialized data objects equate to the same
         * version.
         * <p>
         * Do not store or use the data streams beyond the confines of this
         * method call. The data streams are owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param ctx the context of the method call.
         * @param versionDataInput1 the data input stream to read the serialized form
         * of a data object produced from {@link #getVersion(DataObjectContext, XsDataInputStream, XsDataOutputStream)}
         * Must not be null.
         * @param versionDataInput2 the data input stream to read the serialized form
         * of a data object produced from {@link #getVersion(DataObjectContext, XsDataInputStream, XsDataOutputStream)}.
         * May be the serialized form of the entire object if {@link VersionType#USER_VERSION} was
         * not used.
         * Must not be null.
         * @return true if the version is equivalent.
         * @throws IOException thrown if there is a problem reading the data.
         */
        public boolean versionEquals(DataObjectContext ctx,
                                     XsDataInputStream versionDataInput1,
                                     XsDataInputStream versionDataInput2) throws IOException;

        /**
         * From the specified Version object, inflate the version attribute
         * <p>
         * Do not store or use the data stream beyond the confines of this
         * method call. The data stream is owned by the WebSphere eXtreme Scale
         * framework and my be closed, pooled or reused as required.
         *
         * @param ctx - the context of the method call
         * @param versionDataInput the data input stream to read the serialized form
         * of a data object produced from {@link #getVersion(DataObjectContext, XsDataInputStream, XsDataOutputStream)}
         * Must not be null.
         * @return The POJO version object, suitable for use with qualified updates by
         * the Loader or ObjectGrid framework.
         * @throws IOException thrown if there is a problem reading the data.
         */
        public Object inflateDataObjectVersion(DataObjectContext ctx,
                                               XsDataInputStream versionDataInput) throws IOException;
    }

}
