package com.ibm.websphere.objectgrid.plugins.io.annotations;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Specifies one or more attributes that identify the portion of the key to calculate the
 * partition hash code.<p>
 * 
 * (Note: this annotation is currently only used when eXtreme scale data format (XDF) is enabled)<br>
 *
 * If this annotation is applied to a class, the order in which the partition hashCode is calculated is determined
 * by the order the PartitionKey annotations are defined.  The order can be overridden using the
 * order attribute in the PartitionKey annotation.
 *
  * The &#64;PartitionKey annotation can be specified for the class using a path
 * syntax to identify all of the fields that are part of the partitioning key or
 * by individually identifying the fields within the class definition.<br>
 *
 * <p><b>The PartitionKey is not inheritable.</b></p>
 * The following examples illustrate how to identify a top-level, primitive field:<br>
 *
 * <strong>Class annotation example with implicit ordering:</strong>
 * <pre>
 * &#64;PartitionKeys({&#64;PartitionKey("deptId"), &#64;PartitionKey("addr/country")})
 * class Employee {
 *     int deptId;
 *     Address addr;
 * }
 *
 * class Address {
 *     String city;
 *     String country;
 * }
 * </pre>
 *
 * <strong>Class annotation example with explicit ordering:</strong>
 * <pre>
 * &#64;PartitionKeys(
 *  {&#64;PartitionKey(value="deptId", order=0),
 *   &#64;PartitionKey(value="addr/country", order=1)})
 * class Employee {
 *     int deptId;
 *     Address addr;
 * }
 *
 * class Address {
 *     String city;
 *     String country;
 * }
 * </pre>
 *
 * <strong>Field annotation example with explicit ordering:</strong>
 * <pre>
 *
 * class Employee {
 *     &#64;PartitionKey(order = 0)
 *     int deptId;
 *
 *     &#64;PartitionKey(value="country", order = 0)
 *     Address addr;
 * }
 *
 * class Address {
 *     String city;
 *     String country;
 * }
 * </pre>
 *
 * <strong>Field annotation example with explicit ordering:</strong>
 * <pre>
 *
 * class Employee {
 *     &#64;PartitionKey(order = 0)
 *     int deptId;
 *
 *     &#64;PartitionKeys({
 *         &#64;PartitionKey(value = "state", order = 1),
 *         &#64;PartitionKey(value = "country", order = 2) })
 *     Address addr;
 * }
 *
 * class Address {
 *     String city;
 *     String state;
 *     String country;
 * }
 * </pre>
 *
 * @since 8.6
 */
@Target({TYPE,FIELD})
@Retention(RUNTIME)
public @interface PartitionKeys {

    /**
     * One or more partition key attributes.
     * @return the path to the partition key.
     */
    PartitionKey[] value() default {};
}
