// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  DataDescriptorFactory.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io.datadescriptor;


import java.util.Map;

import com.ibm.ws.objectgrid.plugins.io.datadescriptor.AssociationImpl;
import com.ibm.ws.objectgrid.plugins.io.datadescriptor.AttributeImpl;
import com.ibm.ws.objectgrid.plugins.io.datadescriptor.EmbeddedTypeImpl;
import com.ibm.ws.objectgrid.plugins.io.datadescriptor.KeyDataDescriptorImpl;
import com.ibm.ws.objectgrid.plugins.io.datadescriptor.MapDataDescriptorImpl;
import com.ibm.ws.objectgrid.plugins.io.datadescriptor.ValueDataDescriptorImpl;

/**
 * A factory for creating DataDescriptor instances when using custom DataSerializers.  Each descriptor
 * object includes factory methods for a mutable version that can be altered until it is initialized, and
 * an immutable version which cannot be altered.
 *
 * @since 7.1.1
 */
public final class DataDescriptorFactory {

    private static DataDescriptorFactory instance = new DataDescriptorFactory();

    /**
     * Retrieve an instance of the DataDescriptorFactory
     * @return a DataDescriptorFactory instance
     */
    public static DataDescriptorFactory instance() {
        return instance;
    }

    /**
     * Create a mutable MapDataDescriptor instance.
     * @return a mutable MapDataDescriptor instance
     */
    public MapDataDescriptor createMapDataDescriptor() {
        return new MapDataDescriptorImpl();
    }

    /**
     * Create an immutable MapDataDescriptor instance.
     * @param associations the associations between this map and another map.  Each association
     * identifies a relationship between two maps using a foreign key.  If null, there are no associations.
     * @param addressableKeyName the addressable attribute name to use for accessing KeyDataSerializer data when
     * using partial key lookups such as a query.  If null, the value: {@link MapDataDescriptor#DEFAULT_ADDRESSABLEKEYNAME} is used.
     * @return an immutable MapDataDescriptor instance
     */
    public MapDataDescriptor createMapDataDescriptor(Map<String, Association> associations, String addressableKeyName) {
        MapDataDescriptorImpl mdd = new MapDataDescriptorImpl();
        if(addressableKeyName != null)
            mdd.setAddressableKeyName(addressableKeyName);
        mdd.setAssociations(associations);
        mdd.initialize();
        return mdd;
    }

    /**
     * Create a mutable KeyDataDescriptor instance.
     * @return a mutable KeyDataDescriptor instance
     */
    public KeyDataDescriptor createKeyDataDescriptor() {
        return new KeyDataDescriptorImpl();
    }

    /**
     * Create an immutable KeyDataDescriptor instance.
     * @param attributes the attributes that define the attributes exposed by the KeyDataDescriptor.
     * @param pathSeparator the string to use to use for attribute navigation for embedded attributes.  The string must not include
     * a valid attribute name that the DataDescriptor exposes, nor the underscore "_" character.
     * If null, the default {@link DataDescriptor#DEFAULT_PATHSEPARATOR} is used.
     * @return an immutable KeyDataDescriptor instance
     */
    public KeyDataDescriptor createKeyDataDescriptor(Map<String, Attribute> attributes, String pathSeparator) {
        KeyDataDescriptorImpl kdd = new KeyDataDescriptorImpl();
        kdd.setAttributes(attributes);
        if(pathSeparator != null)
            kdd.setPathSeparator(pathSeparator);
        kdd.initialize();
        return kdd;
    }

    /**
     * Create a mutable ValueDataDescriptor instance.
     * @return a mutable ValueDataDescriptor instance
     */
    public ValueDataDescriptor createValueDataDescriptor() {
        return new ValueDataDescriptorImpl();
    }

    /**
     * Create an immutable ValueDataDescriptor instance.
     * @param attributes the attributes that define the attributes exposed by the ValueDataDescriptor.
     * @param pathSeparator the string to use to use for attribute navigation for embedded attributes.  The string must not include
     * a valid attribute name that the DataDescriptor exposes, nor the underscore "_" character.
     * If null, the default {@link DataDescriptor#DEFAULT_PATHSEPARATOR} is used.
     * @return an immutable ValueDataDescriptor instance
     */
    public ValueDataDescriptor createValueDataDescriptor(Map<String, Attribute> attributes, String pathSeparator) {
        ValueDataDescriptorImpl vdd = new ValueDataDescriptorImpl();
        vdd.setAttributes(attributes);
        if(pathSeparator != null)
            vdd.setPathSeparator(pathSeparator);
        vdd.initialize();
        return vdd;
    }

    /**
     * Create a mutable Attribute instance.
     * @return a mutable Attribute instance.
     */
    public Attribute createAttribute() {
        return new AttributeImpl();
    }

    /**
     * Create an immutable Attribute instance.
     * @param attributeName the name of the attribute. This value is required and must not be null.
     * @param attributeType the type of the attribute. This value is required and must not be null.
     * @param embeddedType the embedded type of the attribute, or null if the type is not {@link AttributeType#EMBEDDED}.
     * @param isCollection true, if the attribute has more than one element.
     * @return an immutable Attribute instance.
     */
    public Attribute createAttribute(String attributeName, AttributeType attributeType, EmbeddedType embeddedType, boolean isCollection) {
        AttributeImpl attr = new AttributeImpl();
        attr.setAttributeName(attributeName);
        attr.setAttributeType(attributeType);
        attr.setEmbeddedType(embeddedType);
        attr.setCollection(isCollection);
        attr.initialize();
        return attr;
    }
    /**
     * Create a mutable new Association instance.
     * @return a mutable Association instance.
     */
    public Association createAssociation() {
        return new AssociationImpl();
    }

    /**
     * Create an immutable Association instance with the minimum required
     * values set.
     * @param attributeName the name of the attribute that holds the association key.  This value is required and must not be null.
     * @param targetMap the name of the map that the association references.  This value is required and must not be null.
     * @return an immutable Association instance.
     */
    public Association createAssociation(String attributeName, String targetMap) {
        AssociationImpl assoc =  new AssociationImpl();
        assoc.setAttributeName(attributeName);
        assoc.setTargetMapName(targetMap);
        assoc.initialize();
        return assoc;
    }

    /**
     * Create a mutable EmbeddedType instance.
     * @return a mutable EmbeddedType instance instance.
     */
    public EmbeddedType createEmbeddedType() {
        return new EmbeddedTypeImpl();
    }

    /**
     * Create an immutable EmbeddedType instance.
     * @param attributes a map of attribute definitions, where the key is the attribute name.
     * @return an immutable EmbeddedType instance instance.
     */
    public EmbeddedType createEmbeddedType(Map<String, Attribute> attributes) {
        EmbeddedTypeImpl et = new EmbeddedTypeImpl();
        et.setAttributes(attributes);
        et.initialize();
        return et;
    }
}
