// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006-2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ObjectQuery.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b834         6.1       04/26/2006   cdjohnson new part
// b1576        6.1       08/11/2006   cdjohnson Feature updates for M5
// b2399        XD 6.1    09/28/2006   cdjohnson Feature updates for M6
// b2402        XD 6.1    11/27/2006   cdjohnson Feature updates for M7
// b10896       XD 6101   12/17/2007   cdjohnson Add setForUpdate
// b12664       XD 6103   06/24/2008   cdjohnson Add getPlan
// b23144       7.1.1     07/14/2011   cheng1    Add option to allow queries to return raw or native data
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.query;

import java.util.Iterator;

import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.PartitionManager;
import com.ibm.websphere.objectgrid.plugins.io.KeySerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.ValueSerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedEntry;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedValue;


/**
 * An ObjectQuery provides a mechanism for introspecting one or more ObjectMaps using
 * standard ObjectGrid query language syntax.  ObjectQuery instances are retrieved using
 * the {@link com.ibm.websphere.objectgrid.Session#createObjectQuery(String)} method and
 * are valid for the session and inherits the session's current state.  For example,
 * if the session has a transaction active, the <code>ObjectQuery.getResultMap()</code>
 * method will participate in that transaction.
 *
 * @since WAS XD 6.1
 *
 * @see com.ibm.websphere.objectgrid.Session#createObjectQuery(String)
 */
public interface ObjectQuery
{
    /**
     * Hint to the query engine to use the specified index.<p>
     *
     * String Format:
     * <pre>
     * &lt;IndexHint&gt;     ::= &lt;Entity Name&gt; "." &lt;AttributeName&gt;
     * &lt;Entity Name&gt;   ::= ObjectGrid map name
     * &lt;AttributeName&gt; ::= An indexed attribute.
     * </pre>
     *
     * Example:
     * <pre>
     * ObjectQuery q = session.createObjectQuery("SELECT p FROM Person p WHERE p.name=?1 AND p.city=?2 AND p.state=?3");
     * q.setHint(ObjectQuery.HINT_USEINDEX, "Person.city");
     * q.setParameter(1, "Smith");
     * q.setParameter(2, "Rochester");
     * q.setParameter(3, "MN");
     * Iterator it = q.getResultIterator();
     * </pre>
     * @see #setHint(String, Object)
     * @since WAS XD 6.1.0.4
     */
    public String HINT_USEINDEX = "HINT_USEINDEX";

    /**
     * The result type indicates when the query engine will return the cached
     * objects results in their native Object form or in the RAW form.
     * The RAW form is only valid when using a DataSerializer, which represents cache object keys and values
     * as {@link SerializedEntry} objects.
     *
     * For a query using {@link ObjectMap}s that are configured with a {@link KeySerializerPlugin}
     * or {@link ValueSerializerPlugin}, the <code>ResultType.RAW</code> value indicates that the type
     * of cache objects returned from those maps will be {@link SerializedEntry}
     * <p>
     * The default ResultType is <strong>Native</strong>.
     * <p>
     * Attribute objects returned from the query will be in their native Java format.
     *
     * @since WXS 7.1.1
     */
    public enum ResultType {
        /**
         * For the <code>NATIVE</code> ResultType, a query returns the deserialized native Java objects
         * for keys and objects from
         * {@link ObjectMap}s configured with a {@link KeySerializerPlugin} or {@link ValueSerializerPlugin}.
         */
        NATIVE,
        /**
         * For the <code>RAW</code> result type, a query using an {@link ObjectMap} that is configured with a {@link KeySerializerPlugin}
         * or {@link ValueSerializerPlugin}, the keys and values objects returned from the query
         * will be {@link SerializedKey} or {@link SerializedValue} objects respectively.
         * <p>
         * If required, you
         * can use the {@link SerializedEntry#getObject()} method to retrieve (possibly inflating
         * the serialized object) the original key or value object.
         * <p>
         * Attribute objects returned from the query will be in their native format even if
         * the RAW result type is set.
         */
        RAW
    }

    /**
     * Set the {@link ResultType} for a query using an {@link ObjectMap} that is configured with a {@link KeySerializerPlugin}
     * or {@link ValueSerializerPlugin}. The keys and values objects returned from the query
     * will be {@link SerializedKey} or {@link SerializedValue} objects respectively. If required, you
     * can use the {@link SerializedEntry#getObject()} method to retrieve (possibly inflating
     * the serialized object) the original key or value object.
     * <p>
     * Attribute objects returned from the query will be in their native format even if
     * the RAW result type is set.
     *
     * Example:
     * <pre>
     * ObjectQuery q = session.createObjectQuery("SELECT p FROM Person p WHERE p.name=?1 AND p.city=?2 AND p.state=?3");
     * q.setResultType(ObjectQuery.ResultType.RAW);
     * q.setParameter(1, "Smith");
     * q.setParameter(2, "Rochester");
     * q.setParameter(3, "MN");
     * Iterator it = q.getResultIterator();
     * while (it.hasNext()) {
     *      Person p = ((SerializedValue)it.next()).getObject();
     * }
     * </pre>
     * @param type the type of object to return when returning cache objects.
     */
    public void setResultType(ResultType type);

    /**
     * Return the current ResultType for the ObjectQuery.
     *
     * @return the current ResultType for the ObjectQuery.
     */
    public ResultType getResultType();


    /**
     * Execute a SELECT query and return the query results in an ObjectMap with the results in
     * query-specified order.  The ObjectMap is only
     * valid only for the current transaction.<p>
     *
     * The map key is the result number (of type long) starting at 0.<p>
     *
     * The map value is of type <code>Object[]</code>
     * where each element in the object array is based on it's ordinal
     * position within the query's select clause.<p>
     *
     * Example:<br>
     * The following query returns 2 rows:<br>
     * <pre>
     * String ql = "SELECT e.name, e.id, d from Employee e join e.dept d WHERE d.number=5";
     * ObjectQuery q = createObjectQuery(ql);
     * ObjectMap resultMap = q.getResultMap();
     * Object oResult = (Object[]) resultMap.get(new Long(0));
     * while(oResult != null) {
     *     // The first attribute is name and has an ordinal position of 0.
     *     String name = (String) oResult[0];
     *     Integer id = (Integer) oResult[1];
     *     Dept d = (Dept) oResult[2];
     *     ...
     * }
     * </pre>
     *
     * @return an ObjectMap that contains the results of the query.
     * @throws ObjectQueryException thrown if query validation
     *             fails or if there is no active transaction.  The resulting
     *             transaction will be marked to rollback.
     */
    public ObjectMap getResultMap();

    /**
     * Execute a SELECT query and return the query results in an Iterator where
     * each result is either an Object (for a single valued query) or Object[] (for
     * a multiple valued query).  The values in the Object[] result are stored in
     * query order. The Iterator is only valid only for the current transaction.<p>
     *
     * Example:<br>
     * The following query returns 2 rows:<br>
     * <pre>
     * String ql = "SELECT e.name, e.id, e from Employee e WHERE e.dept.number=5";
     * ObjectQuery q = session.createObjectQuery(ql);
     * Iterator results = q.getResultIterator();
     * while(results.hasNext()) {
     *     Object[] curEmp = (Object[]) results.next();
     *     String name = (String)curEmp[0];
     *     Integer id = (Integer)curEmp[1]
     *     Dept d = (Dept)curEmp[2];
     *     ...
     * }
     * </pre>
     *
     * @return an Iterator with the query results in the form of <code>Object</code> or <code>Object[]</code>
     * @throws ObjectQueryException thrown if query validation
     *             fails or if there is no active transaction.  The resulting
     *             transaction will be marked to rollback.
     */
    public Iterator getResultIterator();

    /**
     * Execute a SELECT query that returns a single result.<p>
     *
     * If the SELECT clause has more than one field defined, then the result
     * will be of type <code>Object[]</code>
     * where each element in the object array is based on it's ordinal
     * position within the query's select clause.
     *
     * <pre>
     * String ql = "SELECT e from Employee e WHERE e.id=100";
     * Employee e = (Employee) session.createObjectQuery(ql).getSingleResult();
     *
     * String ql = "SELECT e.name, e.dept from Employee e WHERE e.id=100";
     * Object[] empData = (Object[]) session.createObjectQuery(ql).getSingleResult();
     * String empName= (String) empData[0];
     * Department empDept = (Department) empData[1];
     * </pre>
     *
     * @return the result
     * @throws NoResultException
     *             if there is no result
     * @throws NonUniqueResultException
     *             if more than one result
     * @throws ObjectQueryException thrown if query validation fails.   The resulting
     *             transaction will be marked to rollback.
     */
    public Object getSingleResult();

    /**
     * Set the maximum number of results to retrieve.
     *
     * @param maxResults the maximum number of results to retrieve.
     * @return the same ObjectQuery instance
     * @throws IllegalArgumentException
     *             if the argument is negative
     */
    public ObjectQuery setMaxResults(int maxResults);

    /**
     * Set the position of the first result to retrieve.
     *
     * @param startPosition position of the first result, numbered from 0
     * @return the same ObjectQuery instance
     * @throws IllegalArgumentException
     *             if the argument is negative
     */
    public ObjectQuery setFirstResult(int startPosition);

    /**
     * The map to insert the results into.  If not specified or is null, the
     * map name is undefined.<p>
     *
     * If the result map name is specified and the map already exists,
     * the query will fail.
     *
     * @param resultMapName the name of the ObjectMap to insert the query results into.
     * @return the same ObjectQuery instance
     */
    public ObjectQuery setResultMapName(String resultMapName);

    /**
     * Set a query hint. If the hint name is not recognized, it is silently ignored.
     *
     * @param hintName the name of the hint.  See the constant values defined in this interface.
     * @param value the value of the hint.
     * @return the same query instance
     * @throws IllegalArgumentException
     *             thrown if the second argument is not valid for the hint.
     *             This exception may be deferred until the query is run.
     * @since WAS XD 6.1.0.4
     */
    public ObjectQuery setHint(String hintName, Object value);

    /**
     * Bind an argument to a named parameter.
     *
     * @param name the parameter name
     * @param value the value of the parameter.
     * @return the same ObjectQuery instance
     * @throws IllegalArgumentException
     *             if parameter name does not correspond to parameter in query string or argument is of incorrect type.
     *             This exception may be deferred until the query is run.
     */
    public ObjectQuery setParameter(String name, Object value);

    /**
     * Bind an argument to a positional parameter.
     *
     * @param position the position of the parameter.  The first parameter is 1.
     * @param value the value of the parameter.
     * @return the same ObjectQuery instance
     * @throws IllegalArgumentException
     *             if position does not correspond to positional parameter of query or argument is of incorrect type.
     *             This exception may be deferred until the query is run.
     */
    public ObjectQuery setParameter(int position, Object value);

    /**
     * Set the partition to route the query to.<p>
     *
     * The partition id is required if the maps in the query are partitioned and the Session commit strategy is set to LOCAL.<p>
     *
     * Use the {@link PartitionManager} to determine the number of partitions
     * for a given backing map and to calculate the partition number from an
     * object.
     *
     * @param partitionId the partition to route the query to.
     * @return the same ObjectQuery instance
     */
    public ObjectQuery setPartition(int partitionId);

    /**
     * Set the update intent for the query.  If set to true, subsequent
     * query executions will lock records appropriately for updating.<p>
     *
     * If not set, the default is false.
     *
     * @param forUpdate if true, lock records for update.
     * @return the same ObjectQuery instance
     * @since WAS XD 6.1.0.1
     */
    public ObjectQuery setForUpdate(boolean forUpdate);

    /**
     * Return a string representation of the query plan.<p>
     *
     * This method can be used to explain the plan of a query without running the query.  The plan describes
     * when a map is to be scanned, an index used and the order in which the maps are accessed.
     *
     * @return a string representation of the query plan.
     * @throws ObjectQueryException thrown if query validation fails.  The transaction will not be marked to rollback.
     * @since WAS XD 6.1.0.3
     */
    public String getPlan();
}
