/**
 *
 */
package com.ibm.websphere.objectgrid.revision;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.plugins.EventListener;
import com.ibm.websphere.objectgrid.plugins.ObjectGridLifecycleListener;
import com.ibm.websphere.objectgrid.plugins.ObjectGridPlugin;

/**
 * Decides what value to store when there is a collision on multiple primaries. The output of the arbitrateCollision method must be consistent across
 * all primaries. Inconsistencies in this implementation will result in inconsistencies in data.
 * <p>
 * A <code>CollisionArbiter</code> implementation that also implements the {@link ObjectGridLifecycleListener}
 * interface will be automatically added as an {@link EventListener} on the {@link ObjectGrid} when the
 * arbiter is set on the object grid.
 * <p>
 * A CollisionArbiter may implement the {@link ObjectGridPlugin} interface in order
 * to receive enhanced ObjectGrid plug-in lifecycle method calls. The plug-in is
 * also required to correctly implement each of the bean methods related to
 * introspection of its state (for example <code>isInitialized()</code>,
 * <code>isDestroyed()</code>, etc).
 *
 *
 * @see ObjectGrid#addEventListener(EventListener)
 * @see ObjectGrid#setCollisionArbiter(CollisionArbiter)
 *
 * @author dcberg
 * @since 7.1
 */
public interface CollisionArbiter {
    /**
     * Marks how the collision should be resolved.
     *
     * @since 7.1
     */
    public static final class Resolution {
        private Resolution() {
        }

        /**
         * Indicates that the collision should keep the entry that currently exists in the cache and ignore the colliding entry.
         */
        public static final Resolution KEEP = new Resolution();

        /**
         * Indicates that the collision should override the existing entry with the colliding entry.
         */
        public static final Resolution OVERRIDE = new Resolution();

        /**
         * Indicates that the collision has been resolved programmatically in the revision transaction.
         */
        public static final Resolution REVISE = new Resolution();
    }

    /**
     * Called when the ObjectGrid which this arbiter is attached to is initialized. The arbiter can get any information about the grid necessary for
     * making decisions.
     *
     * @param grid
     *            The ObjectGrid that this arbiter is responsible for arbitrating collisions for.
     */
    public void initialize(ObjectGrid grid);

    /**
     * Indicates that a revision event has discovered a write operation has occurred on two primaries simultaneously. The revisioning logic can not
     * resolve this event without data loss. <br>
     * This method has four valid outcomes and an error condition: <br>
     * <table border="1">
     * <tr>
     * <th>Return value</th>
     * <th>Outcome</th>
     * </tr>
     * <tr>
     * <td><code>null</code></td>
     * <td>Abstain, allow the default arbiter to make the decision.</td>
     * </tr>
     * <tr>
     * <td>{@link Resolution#KEEP}</td>
     * <td>Keep the existing entry, throw away the collision entry.</td>
     * </tr>
     * <tr>
     * <td>{@link Resolution#OVERRIDE}</td>
     * <td>Override the existing entry with the collision entry.</td>
     * </tr>
     * <tr>
     * <td>{@link Resolution#REVISE}</td>
     * <td>Modify the entry either by merging the entries or removing the entry.</td>
     * </tr>
     * <tr>
     * <td>{@link Exception}</td>
     * <td>The revision transaction will be rolled back. No revision will be applied and a message will be logged.</td>
     * </tr>
     * </table>
     * <br>
     * The session that is being used to apply the revision changes is supplied for making modifications to collision elements. Care should be taken
     * in using this session as it is an active internal session. This session must not be used outside the scope of this method call.
     *
     *@param collisionData
     *            The data object that provides information about the collision.
     * @return The resolution marker that determines how to deal with the collision.
     */
    public Resolution arbitrateCollision(CollisionData collisionData);
}
