// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  SimpleLDAPLoginModule.java
//

//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
//              WXS7.10x  07/10/2012   mcgarvey   new
// ------------ --------- ------------ --------- -----------------------------------------

// This login module just does an LDAP bind.  The userid that is passed is the full DN to be used in the bind.
// The subject will have an X500Principal with this DN


package com.ibm.websphere.objectgrid.security.plugins.builtins;

import java.util.Hashtable;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import javax.security.auth.x500.X500Principal;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;

/**
 * SimpleLDAPLoginModule authenticates a user name and password to the 
 * configured LDAP server.
 * <p>
 * A login configuration should provide at least the following two options:
 * <ul>
 * <li>providerURL: The LDAP server provider URL.  This may be an SSL URL, in which case the trust store for the
 * server will be used to validate the LDAP server certificate </li>

 * <li>factoryClass: The ldap context factory implementation class, which is typically "com.sun.jndi.ldap.LdapCtxFactory".</li>
 * </ul>
 * <p>
 * Here is a Login module configuration example which may be used by
 * LDAPAuthenticator: 
 * <pre><code>
 * LDAPLogin {
 *     com.ibm.websphere.objectgrid.security.plugins.builtins.SimpleLDAPLoginModule required 
 *     providerURL="ldap://bluepages.ibm.com:389/" 
 *     factoryClass="com.sun.jndi.ldap.LdapCtxFactory"
 *     debug=true;
 * }; 
 * </code></pre>
 * 
 * The example providerURL points to an LDAP server with the port number 
 * 389. The initial context factory is com.sun.jndi.ldap.LdapCtxFactory.
 * 
 * This login module returns a principal of class javax.security.auth.x500.X500Principal.
 * 
 * @since WebSphere eXtreme Scale 8.6  
 * 
 * @see LoginModule
 * 
 */
public class SimpleLDAPLoginModule implements LoginModule {
    private static final String CLASS_NAME = SimpleLDAPLoginModule.class.getName();

    private static final TraceComponent TC = Tr.register(
            CLASS_NAME,
            Constants.TR_SECURITY_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);

    /** 
     * LDAP server provider URL property name 
     */
    private static final String PROVIDER_URL = "providerURL";

    /** 
     * LDAP context factory implementation class property name 
     */
    private static final String FACTORY_CLASS = "factoryClass";

    private Subject subject;
    private X500Principal userPrincipal;

    private CallbackHandler callbackHandler;

    private Map<String, ? > options;

    // configurable option
    private boolean debug = false;

    // the authentication status
    private boolean succeeded = false;

    private boolean commitSucceeded = false;

    // username and password
    private String username;

    private char[] password;


	/* (non-Javadoc)
	 * @see javax.security.auth.spi.LoginModule#abort()
	 */
	public boolean abort() throws LoginException {
        if (succeeded == false) {
            return false;
        }
        else if (succeeded == true && commitSucceeded == false) {
            // login succeeded but overall authentication failed
            succeeded = false;
            username = null;
            if (password != null) {
                for (int i = 0; i < password.length; i++)
                    password[i] = ' ';
                password = null;
            }
            userPrincipal = null;
        }
        else {
            // overall authentication succeeded and commit succeeded,
            // but someone else's commit failed
            logout();
        }
        return true;
	}

	/* (non-Javadoc)
	 * @see javax.security.auth.spi.LoginModule#commit()
	 */
	public boolean commit() throws LoginException {
        if (succeeded == false) {
            return false;
        }
        // add a Principal (authenticated identity)
        // to the Subject
        // assume the user we authenticated is the X500Principal
        if (!subject.getPrincipals().contains(userPrincipal)) {
            subject.getPrincipals().add(userPrincipal);
        }                
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isDebugEnabled() || debug) {
            Tr.debug(TC, "[SimpleLDAPLoginModule] " + "added Principal to Subject");
        }
        // in any case, clean out state
        username = null;
        for (int i = 0; i < password.length; i++)
            password[i] = ' ';
        password = null;
        commitSucceeded = true;
        return true;
	}

	/* (non-Javadoc)
	 * @see javax.security.auth.spi.LoginModule#initialize(javax.security.auth.Subject, javax.security.auth.callback.CallbackHandler, java.util.Map, java.util.Map)
	 */
	public void initialize(Subject _subject, CallbackHandler _callbackHandler,
			Map<String, ?> _sharedState, Map<String, ?> _options) {
        this.subject = _subject;
        this.callbackHandler = _callbackHandler;
        this.options = _options;
        if (subject == null)
            throw new IllegalArgumentException("Subject is not specified");
        if (callbackHandler == null)
            throw new IllegalArgumentException("CallbackHander is not specified");
        // initialize any configured options
        debug = "true".equalsIgnoreCase((String) options.get("debug"));
	}

	private void authenticate(String user, String pwd) throws NamingException {
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
            Tr.entry(TC, "authenticate", new Object[] { user, "xxxxxx", this });
        Hashtable<String,String> env = new Hashtable<String,String>();
        env.put(Context.INITIAL_CONTEXT_FACTORY,(String) options.get(FACTORY_CLASS));
        env.put(Context.PROVIDER_URL, (String) options.get(PROVIDER_URL));
        env.put(Context.SECURITY_PRINCIPAL, user);
        env.put(Context.SECURITY_CREDENTIALS, pwd);
        env.put(Context.SECURITY_AUTHENTICATION, "simple");

        InitialContext initialContext = new InitialContext(env);
        initialContext.close();  // need a performance improvement here to do connection pooling -- if possible
        						 // I am not sure there is a way to do an LDAP rebind via JNDI
    }
	/* (non-Javadoc)
	 * @see javax.security.auth.spi.LoginModule#login()
	 */
	public boolean login() throws LoginException {
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled() || debug)
            Tr.entry(TC, "login", this);

        if (subject == null || callbackHandler == null) {
            throw new LoginException("Module initialization failed");
        }

        NameCallback nameCallback = new NameCallback("Username:");
        PasswordCallback pwdCallback = new PasswordCallback("Password:", false);

        try {
            callbackHandler.handle(new Callback[] { nameCallback, pwdCallback });
        }
        catch (Exception e) {
            FFDCFilter.processException(e, CLASS_NAME + ".login", "223", this);

            if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
                Tr.exit(TC, "login", "SecurityException");
            throw new LoginException("Callback failed: " + e);
        }

        username = nameCallback.getName();
        char[] tempPwd = pwdCallback.getPassword();

        if (tempPwd == null) {
            // treat a NULL password as an empty password
            tempPwd = new char[0];
        }
        password = new char[tempPwd.length];
        System.arraycopy(tempPwd, 0, password, 0, tempPwd.length);

        pwdCallback.clearPassword();

        if (ObjectGridManagerImpl.isTraceEnabled && TC.isDebugEnabled() || debug)
            Tr.debug(TC, "[SimpleLDAPLoginModule] login: " + "user entered user name: " + username);

        // LDAP authentication

        try {
            authenticate(username, new String(password));
            // authentication succeeded!!!
             if (ObjectGridManagerImpl.isTraceEnabled && TC.isDebugEnabled() || debug)
                Tr.debug(TC, "[SimpleLDAPLoginModule] " + "authentication succeeded");
             userPrincipal = new X500Principal(username);
             succeeded = true;
        } catch (NamingException ne) {
            if (ObjectGridManagerImpl.isTraceEnabled && TC.isDebugEnabled() || debug)
                Tr.debug(TC, "[SimpleLDAPLoginModule] " + "authentication failed");
            succeeded = false;
            username = null;
            for (int i = 0; i < password.length; i++)
                password[i] = ' ';
            password = null;
            if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled() || debug)
                Tr.exit(TC, "login", "FailedLoginException");
            throw new FailedLoginException("Login failed");
        }
        return succeeded;
	}

	/* (non-Javadoc)
	 * @see javax.security.auth.spi.LoginModule#logout()
	 */
	public boolean logout() throws LoginException {
        subject.getPrincipals().remove(userPrincipal);
        succeeded = false;
        succeeded = commitSucceeded;
        username = null;
        if (password != null) {
            for (int i = 0; i < password.length; i++)
                password[i] = ' ';
            password = null;
        }
        userPrincipal = null;
        return true;
    }
}
