// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005-2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  WSTokenAuthenticator.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d329154      WASX.XD   12/05/2005   jitang    Provider WAS authentication
// d335512      WASX.XD   01/04/2006   jitang    push both AT and OAT in the WS credential generator
// PK17543      WASX.XD   01/11/2006   jitang    Update JavaDoc
// b10193       XD 6.1    10/16/2007   jitang    Add doPriv block
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.plugins.builtins;

import java.security.AccessController;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;

import javax.security.auth.Subject;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.security.plugins.Authenticator;
import com.ibm.websphere.objectgrid.security.plugins.Credential;
import com.ibm.websphere.objectgrid.security.plugins.ExpiredCredentialException;
import com.ibm.websphere.objectgrid.security.plugins.InvalidCredentialException;
import com.ibm.websphere.security.WSSecurityException;
import com.ibm.websphere.security.auth.TokenExpiredException;
import com.ibm.websphere.security.auth.WSSecurityContext;
import com.ibm.websphere.security.auth.WSSecurityContextResult;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.management.util.SecurityHelper;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.objectgrid.util.ObjectGridUtil;
import com.ibm.ws.security.core.ContextManagerFactory;
import com.ibm.wsspi.security.token.WSOpaqueTokenHelper;

/**
 * This class represents a <code>Authenticator</code> when both ObjectGrid 
 * clients and ObjectGrid servers are launched in the WebSphere application 
 * server JVMs, and these application servers are in the same security domain. 
 * <p>
 * When both ObjectGrid clients and servers run in WebSphere application servers, 
 * the client can use the <code>WSTokenCredentialGenerator</code> class to
 * generate a credential and ask the server to use this Authenticator implementation 
 * class to validate the WSTokenCredential. If the token is validated successfully, 
 * a <code>Subject</code> is returned.
 * <p>
 * This scenario takes advantage of the fact that an ObjectGrid client has already 
 * been authenticated. Since the application servers housing the ObjectGrid servers are in 
 * the same security domain as the application servers housing the ObjectGrid client, 
 * the security tokens can be propagated from the ObjectGrid client to the ObjectGrid server 
 * so there is no need to re-authenticate to the same user registry.
 * 
 * This authenticator returns a Subject with a Principal of class com.ibm.websphere.security.auth.WSPrincipal.
 *     
 * @since WAS XD 6.0.1
 * 
 * @see Authenticator
 * @see WSTokenCredential
 * @see WSTokenCredentialGenerator
 */
public class WSTokenAuthenticator implements Authenticator {

    private static final TraceComponent TC = Tr.register(
            WSTokenAuthenticator.class,
            Constants.TR_SECURITY_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);

    /**
     * Authenticates the WSTokenCredential and returns a 
     * <code>Subject</code> object. 
     * <p>
     * The credential is expected to be a <code>WSTokenCredential</code> type.
     *   
     * @throws InvalidCredentialException if credential isn't a 
     *         WSTokenCredential or some error occurs during processing
     *         of the supplied WSTokenCredential
     * 
     * @throws ExpiredCredentialException if credential is expired.  This exception
     *         is not used by this implementation
     *  
     * @see Authenticator#authenticate(Credential)
     * @see WSTokenCredential
     */
    public Subject authenticate(Credential credential) throws InvalidCredentialException, ExpiredCredentialException {
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isDebugEnabled()) {
            Tr.debug(TC, "authenticate", credential);
        }
        Subject subject = null;
        try {

            if (credential == null) {
                throw new InvalidCredentialException("Supplied credential is null");
            }

            if (!(credential instanceof WSTokenCredential)) {
                throw new InvalidCredentialException(
                "The credential is not a WSTokenCredential type.");
            }

            // Get the OAT
            byte[] OAT = ((WSTokenCredential) credential).getOAT();

            // get the ArrayList of tokens from the OAT
            java.util.List tokenHolderList = WSOpaqueTokenHelper
                .getInstance()
                .createTokenHolderListFromOpaqueToken(OAT);

            // set the ArrayList on the thread of execution, prior to authenticating the AT above.
            ContextManagerFactory.getInstance().put(
                    WSOpaqueTokenHelper.getInstance().getOpaqueTokenLookup(),
                    tokenHolderList);

            //get AT from your message...
            final byte[] bytes = ((WSTokenCredential) credential).getAT();
            if (bytes == null) {
                throw new InvalidCredentialException("The credential is null.");
            }

            try {
                subject = (Subject) AccessController.doPrivileged(new PrivilegedExceptionAction() {
                   public Object run() throws WSSecurityException {
                       WSSecurityContext ctx = SecurityHelper.getHelper().getWSSecurityContext();

                       // this correctly decodes the bytes, prepares a callback handler, and calls a JAAS login module
                       WSSecurityContextResult result = ctx.acceptSecContext(bytes);
                       // the resulting authenticated Subject.
                       Subject subject = result.getSubject();
                       // set this as the caller subject
                       ContextManagerFactory.getInstance().initializeCallerContext(subject);
                        return subject;
                    } 
                });
            } catch (PrivilegedActionException e) {
                Throwable t= e.getCause();
                if (t instanceof WSSecurityException) {
                    throw (WSSecurityException) t;
                }
                throw new InvalidCredentialException (e);
            }
        }
        catch (WSSecurityException ex) {

            FFDCFilter.processException(
                    ex,
                    "com.ibm.websphere.objectgrid.security.plugins.builtins.WSTokenAuthenticator.authenticate",
                    "82",
                    this);

            if (ObjectGridManagerImpl.isTraceEnabled && TC.isDebugEnabled())
                Tr.debug(TC, "WSSecurityException: " + ex);
            if (ObjectGridUtil.isException(ex, TokenExpiredException.class)) {
                throw new ExpiredCredentialException(ex);
            }
            throw new InvalidCredentialException(ex);
        }
        
        if (subject ==  null) {
            throw new InvalidCredentialException("The subject from the WS token is null");
        }
        return subject;
    }
}