/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * b24587       7.1.1     08/31/2011   koehler   add placementDeferralInterval property
 */

package com.ibm.websphere.objectgrid.server;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.transport.XsTransportType;

/**
 * Defines a set of properties to be used at initialization by the ObjectGrid server runtime. This property object is
 * used to determine if this process will act as a catalog service and if so the properties of the catalog service.
 *
 * @since WAS XD 6.1 FIX3
 */
public interface CatalogServerProperties {
    /**
     * The default name for a catalog service domain.
     * @see #setDomainName(String)
     */
    public static final String DEFAULT_DOMAIN = "DefaultDomain";

    /**
     * Defines the domain name used to uniquely identify this catalog service grid to clients
     * when routing to multiple domains.
     * @see #setDomainName(String)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_DOMAIN_NAME = "domainName";

    /**
     * Determines whether or not quorum is enabled.
     * @see #setQuorum(boolean)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_ENABLE_QUORUM = "enableQuorum";

    /**
     * Determines whether or not the initialization of the MBeanServer notifications concentrator
     * @see #setManagementConcentrator(boolean)
     * @since 8.6
     */
    public static final String PROP_ENABLE_MANAGEMENT_CONCENTRATOR = "enableManagementConcentrator";

    /**
     * A bidirectional connection will be established to each of the foreign domains for
     * the purpose of exchanging data in a multi-primary scenario.<p>
     *
     * See the {@link CatalogServerProperties#PROP_SUFFIX_DOMAIN_ENDPOINTS} property for an
     * example on how to set the end points for each foreign domain.
     *
     * @since 7.1
     */
    public static final String PROP_FOREIGN_DOMAINS = "foreignDomains";

    /**
     * Use this string as a suffix when reading and writing domain end points properties.  Domain end
     * points are specified in the following format:
     * <pre>
     * &lt;domainName&gt;.endPoints=&lt;host&gt;:&lt;port&gt;,&lt;host&gt;:&lt;port&gt;
     * </pre><p>
     *
     * Example:
     * <pre>
     * domainName=NewYork
     * foreignDomains=London,Tokyo
     * London.endPoints=datagridcat1.london.acme.com:2809,datagridcat2.london.acme.com:2809
     * Tokyo.endPoints=datagridcat1.tokyo.acme.com:2809,datagridcat2.tokyo.acme.com:2809
     * </pre>
     * @since 7.1
     */
    public static final String PROP_SUFFIX_DOMAIN_ENDPOINTS = ".endPoints";


    /**
     * Sets the port that the MBean Server should listen on.
     * @since WAS XD 6.1.0.3
     * @deprecated Deprecated in version 7.1.  Use {@link ServerProperties#PROP_JMX_SERVICE_PORT} instead
     * @see ServerProperties#PROP_JMX_SERVICE_PORT
     */
    public static final String PROP_JMX_SERVICE_PORT = ServerProperties.PROP_JMX_SERVICE_PORT;  // fix-vandersn: shouldn't this property be on the ServerProperties?

    /**
     * The format of the string should be "serverName:hostName:clientPort:peerPort<,serverName:hostName:clientPort:peerPort>".
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_CATALOG_CLUSTER_END_POINTS = "catalogClusterEndPoints";

    /**
     * Sets the heartbeat frequency level.
     *
     * @see #setHeartBeatFrequencyLevel(int)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_HEARTBEAT_FREQUENCY_LEVEL = "heartBeatFrequencyLevel";

    /**
     * A boolean to indicate whether the server should start a catalog service.
     *
     * @see #setCatalogServer(boolean)
     * @since 7.1.1
     */
    public static final String PROP_CATALOG_SERVER = "isCatalog";

    /**
     * Constant representing a heartbeat level at a typical rate.<p>
     *
     * A typical heartbeat frequency allows reasonable failover detection and resource
     * utilization.  This value is the default.
     */
    public static final int HEARTBEAT_FREQUENCY_LEVEL_TYPICAL= 0;

    /**
     * Constant representing a heartbeat level at an aggressive rate.<p>
     *
     * An increased heartbeat frequency allows failures to be detected more quickly, but can also
     * uses additional CPU and network resources.  This level is more sensitive to missing heartbeats
     * when the server is stressed.
     */
    public static final int HEARTBEAT_FREQUENCY_LEVEL_AGGRESSIVE= -1;

    /**
     * Constant representing a heartbeat level at relaxed rate.<p>
     *
     * A decreased heartbeat frequency increases the time to detect failures, but also
     * decreases CPU and network utilization.
     */
    public static final int HEARTBEAT_FREQUENCY_LEVEL_RELAXED= 1;

    /**
     * Constant representing the default interval for deferral of balancing and placement of work items to the containers <p>
     *
     * An increase in the deferral interval will lower CPU utilization, but the placement of work items will be completed over time.
     * A decrease in the deferral interval will increase short term CPU utilization, but the placement of work items will be more
     * immediate and expedited. The value is in milliseconds.
     * @see #setPlacementDeferralInterval(long)
     * @see #PROP_PLACEMENT_DEFERRAL_INTERVAL
     * @since 7.1.1
     */
    public static final long PLACEMENT_DEFERRAL_INTERVAL_DEFAULT = 15000;

    /**
     * Determines the placement deferral interval.
     * @see #setPlacementDeferralInterval(long)
     * @since 7.1.1
     */
    public static final String PROP_PLACEMENT_DEFERRAL_INTERVAL = "placementDeferralInterval";
    
    /***
     * Determines the transport on the catalog service domain for server to server communication
     * @see #XIO_TRANSPORT
     * @see #ORB_TRANSPORT
     * @since 8.6.0
     */
    public static final String PROP_TRANSPORT = "transport";
    
    /**
     * The Object Request Broker or ORB transport
     * @see #PROP_TRANSPORT
     * @see #setTransport(String)
     * @see #getTransport()
     */
    public static final String ORB_TRANSPORT = String.valueOf(XsTransportType.ORB);

    /**
     * The eXtremeIO or XIO transport
     * @see #PROP_TRANSPORT
     * @see #setTransport(String)
     * @see #getTransport()
     */
    public static final String XIO_TRANSPORT = String.valueOf(XsTransportType.XIO);

    /**
     * Defines this process as a catalog service if <code>isCatalog</code> is true. The default value is
     * <code>false</code>.
     *
     * @param isCatalog
     *            Defines this process as a catalog service if true.
     */
    public void setCatalogServer(boolean isCatalog);

    /**
     * Returns whether this process is defined as a catalog service or not.
     *
     * @return Whether or no this process is defined as a catalog service.
     */
    public boolean isCatalogServer();

    /**
     * Assigns the name of the ObjectGrid domain that this catalog service manages. The domain is defined by the set of
     * containers that this catalog service manages. Note that this name only needs to be unique when an ObjectGrid
     * client may access maps in multiple domains.
     *
     * @param domainName
     *            A unique name for this catalog service.
     * @see #DEFAULT_DOMAIN
     */
    public void setDomainName(String domainName);

    /**
     * Retrieves the name of the domain that this catalog service defines.
     *
     * @return A unique name for this catalog service.
     */
    public String getDomainName();

    /**
     * Sets the endpoint for each of the catalog service grid members. Each catalog service <b>must</b> be set with the same
     * list of endpoints, this is because the catalog service members will be set into a core group for availability
     * monitoring and must be aware of each other. The format of the string should be
     * "serverName:hostName:clientPort:peerPort<,serverName:hostName:clientPort:peerPort>".
     *
     * @param clusterEndpoints
     *            The endpoints string containing all of the catalog services.
     */
    public void setCatalogClusterEndpoints(String clusterEndpoints);

    /**
     * Retrieves the list of catalog service endpoints.
     *
     * @return A string concatenation of endpoints for all of the catalog service members.
     */
    public String getCatalogClusterEndpoints();

    /**
     * Sets the heartbeat frequency level.  The heartbeat frequency level tells
     * the catalog service how aggressively to check for container failures.  The default is 0, which
     * is suitable for most environments.<p>
     *
     * Valid values include:
     * <ul>
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_TYPICAL} (default)
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_RELAXED}
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_AGGRESSIVE}
     * </ul>
     *
     * @param frequencyLevel one of three values: -1, 0 or 1 as defined by the
     * constants that begin with name <tt>HEARTBEAT_FREQUENCY_LEVEL</tt>.
     * @throws IllegalArgumentException if the frequencyLevel is out of range.
     * @since WAS XD 6.1.0.3
     */
    public void setHeartBeatFrequencyLevel(int frequencyLevel);

    /**
     * Retrieves the heartbeat frequency level.<p>
     *
     * Valid values include:
     * <ul>
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_TYPICAL}
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_RELAXED}
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_AGGRESSIVE}
     * </ul>
     *
     * @return the heartbeat frequency level:  -1, 0 or 1 as defined by the
     * constants that begin with name <tt>HEARTBEAT_FREQUENCY_LEVEL</tt>.
     * @since WAS XD 6.1.0.3
     */
    public int getHeartBeatFrequencyLevel();

    /**
     * Sets the location of the security file specific to the catalog service.
     *
     * @param clusterSecurity
     *            The cluster security file location.
     */
    public void setClusterSecurityURL(URL clusterSecurity);

    /**
     * Retrieves the cluster security file location.
     *
     * @return The cluster security file location.
     */
    public URL getClusterSecurityURL();

    /**
     * Sets the server security file name.
     *
     * @param serverSecurity
     *            Retrieves the server security file name.
     */
    public void setServerSecurityFileName(String serverSecurity);

    /**
     * Retrieves the server security file name.
     *
     * @return The server security file name.
     */
    public String getServerSecurityFileName();

    /**
     * Defines the behavior of this catalog service to enable quorum. Quorum is used
     * to ensure that a majority of the catalog service grid is available before allowing
     * modification to the placement of partitions on available container servers. This is done
     * to protect against inconsistencies that may arise from an "islanded" catalog service.
     * <p>
     * By default, the quorum is disabled.
     * @param quorum
     *            Whether this catalog service should enforce quorum or not.
     */
    public void setQuorum(boolean quorum);

    /**
     * Returns whether or not this catalog service is enforcing quorum.
     *
     * @return Answers true if quorum is enabled on the catalog server.
     */
    public boolean getQuorum();

    /**
     * Defines the behavior of this catalog service to enable the MBean notifications concentrator.
     * The Management Concentrator is used to consolidate all MBean notifications emitted in
     * the catalog or container servers. This is done to ensure single point of subscription for
     * JMX clients and access to canonical formatted notification events from all emitters.
     * <p>
     * By default, the Management Concentrator is enabled.
     * @param enabled
     *            Whether this catalog service initialises the notification concentrator or not.
     * @since 8.6
     */
    public void setManagementConcentrator(boolean enabled);

    /**
     * Returns whether or not this catalog service is initialising the MBean notifications concentrator.
     *
     * @return Answers true if the notifications concentrator is enabled on the catalog server.
     * @since 8.6
     */
    public boolean isManagementConcentrator();


    /**
     * Set the foreign domains that this domain will communicate with to establish a
     * multi-primary topology.
     * <p>
     * Provide domain names in a comma separated list.
     *
     * @param foreignDomains A comma separated list of foreign domain names.
     * @since 7.1
     */
    public void setForeignDomains(String foreignDomains);

    /**
     * Get the current foreign domains.
     * @return A String of comma separated domain names
     * @since 7.1
     */
    public String getForeignDomains();

    /**
     * Set end points for catalog servers in a foreign domain.  The end points correspond to the
     * listener host and listener port of the catalog servers.
     * <p>
     * Domain end points are specified in the following format:
     * <pre>
     * &lt;domainName&gt;.endPoints=&lt;host&gt;:&lt;port&gt;,&lt;host&gt;:&lt;port&gt;
     * </pre><p>
     *
     * @param domain The domain name to set the end points for.
     * @param endPoints A comma separated list of host:port values.
     * @since 7.1
     */
    public void setDomainEndPoints(String domain, String endPoints);

    /**
     * Retrieves the end points for the specified domain
     *
     * @param domain The domain name in which to retrieve the end points.
     * @since 7.1
     */
    public String getDomainEndPoints(String domain);

    /**
     * Loads the properties from <code>input</code> into this property object.
     *
     * @param input
     *            A stream to load properties from.
     * @throws IOException
     *             If the properties input has an invalid format.
     */
    public void load(InputStream input) throws IOException;

    /**
     * Writes the properties from this object into <code>output</code>.
     *
     * @param output
     *            A stream to write out the properties to.
     * @throws IOException
     *             If the properties input has an invalid format.
     */
    public void store(OutputStream output) throws IOException;

    /**
     * Sets the placement deferral interval.  The placement deferral interval tells
     * the catalog service how aggressively to do placement of work items.  The default is 15000 milliseconds, which
     * is suitable for most environments.
     *
     * @param placementDeferralInterval is the interval in milliseconds for the deferral of balancing
     * and placement of work items to the containers.  If the interval is -1 the default
     * interval will be used.
     *
     * @since 7.1.1
     */
    public void setPlacementDeferralInterval(long placementDeferralInterval);

    /**
     * Gets the placement deferral interval.  The placement deferral interval tells
     * the catalog service how aggressively to do placement of work items.  The default is 15000, which
     * is suitable for most environments.
     *
     * @return the placement deferral interval in milliseconds
     * @since 7.1.1
     */
    public long getPlacementDeferralInterval();
    
    /**
     * Sets the transport for the catalog service domain
     * @see #XIO_TRANSPORT
     * @see #ORB_TRANSPORT
     * @since 8.6.0
     */
    public void setTransport(String transport);

    /**
     * Returns the transport for the catalog service domain
     * @see #XIO_TRANSPORT
     * @see #ORB_TRANSPORT
     * @since 8.6.0
     */
    public String getTransport();


}
