/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007-2012
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * b23160       7.1.0.3   05/04/2011   jhanders  Add ability to configure JMX SSL port
 */

package com.ibm.websphere.objectgrid.server;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;
import java.util.Properties;

import com.ibm.websphere.objectgrid.ObjectGridManager;
import com.ibm.websphere.objectgrid.management.DynamicServerMBean;
import com.ibm.websphere.objectgrid.stats.StatsSpec;


/**
 * The set of properties used to define the behavior of the ObjectGrid server runtime.
 *
 * @since WAS XD 6.1 FIX3
 */
public interface ServerProperties {
    /**
     * The name of the zone that all containers that do not specify a zone are in.
     *
     * @see #setZoneName(String)
     */
    public static final String DEFAULT_ZONE = "DefaultZone";

    /**
     * Defines the name the server should use to identify itself.
     *
     * @see #setServerName(String)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_SERVER_NAME = "serverName";

    /**
     * Defines the name of the zone this server belongs to.
     *
     * @see #setZoneName(String)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_ZONE_NAME = "zoneName";

    /**
     * The property that defines which directory the ObjectGrid server should use for all default settings.
     *
     * @see #setWorkingDirectory(String)
     */
    public static final String PROP_WORKING_DIRECTORY = "workingDirectory";

    /**
     * The log record filtering regular expression that the ObjectGrid server should initially use.
     * The default is null, i.e. no filtering is applied based on log record, but only the default
     * log level severity (SEVERE & WARNING).
     *
     * @see DynamicServerMBean#setLogNotificationFilter(String)
     * @since 8.6
     */
    public static final String PROP_LOG_NOTIFICATION_FILTER = "logNotificationFilter";

    /**
     * The trace specification that the ObjectGrid server should use initially. The default is *=all=disabled.
     *
     * @see ObjectGridManager#setTraceSpecification(String)
     */
    public static final String PROP_TRACE_SPEC = "traceSpec";

    /**
     * The name of the file that trace output should be sent to.
     *
     * @see ObjectGridManager#setTraceFileName(String)
     */
    public static final String PROP_TRACE_FILE = "traceFile";

    /**
     * The statistics specification that the ObjectGrid should use for metric gathering.
     *
     * @see #setStatsSpecification(String)
     */
    public static final String PROP_STATS_SPEC = "statsSpec";

    /**
     * Determines whether or not the ObjectGrid will register MBeans in this process.
     *
     * @see #setMBeansEnabled(boolean)
     */
    public static final String PROP_ENABLE_MBEANS = "enableMBeans";

    /**
     * Defines whether SystemOut and SystemErr should be sent to file or not.
     * <p>
     * Valid values include: <tt>true</tt> and <tt>false</tt>.
     */
    public static final String PROP_SYSTEM_STREAM_TO_FILE_ENABLED = "systemStreamToFileEnabled";

    /**
     * Defines the port number the High Availability Manager will use. If this doesn't present, the catalog server will
     * generate an available port automatically.
     *
     * @see #setHAManagerPort(int)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_HAMANAGERPORT = "HAManagerPort";

    /**
     * Sets the end points to connect to the catalog service grid. This should be of the form
     * <code>host:port<,host:port></code> where the host value is the listenerHost and the port value is the
     * listenerPort of the catalog service process.
     *
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_CATALOG_SERVICE_ENDPOINTS = "catalogServiceEndPoints";

    /**
     * Sets the host name that the ORB should bind to.
     *
     * @see #setListenerHost(String)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_LISTENER_HOST = "listenerHost";

    /**
     * Sets the port that the ORB should bind to.
     *
     * @see #setListenerPort(int)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_LISTENER_PORT = "listenerPort";

    /**
     * Sets the memory threshold (percentage of max heap) for memory based eviciton
     *
     * @see #setMemoryThresholdPercentage(int)
     * @since WAS XD 6.1.0.3
     */
    public static final String PROP_MEMORY_USAGE_THRESHOLD = "memoryThresholdPercentage";

    /**
     * Sets the minimum thread pool size used by the server and client runtimes in this process
     *
     * @see #setMinimumThreadPoolSize(int)
     * @since 7.0.0.0 FIX2
     */
    public static final String PROP_MINIMUM_THREAD_POOL_SIZE = "minThreads";

    /**
     * Sets the minimum thread pool size used by the server and client runtimes in this process
     *
     * @see #setMaximumThreadPoolSize(int)
     * @since 7.0.0.0 FIX2
     */
    public static final String PROP_MAXIMUM_THREAD_POOL_SIZE = "maxThreads";

    /**
     * The property that defines which port the JMX service will bind to
     *
     * @see #setJMXServicePort(int)
     * @since 7.1
     */
    public static final String PROP_JMX_SERVICE_PORT = "JMXServicePort";

    /**
     * The property that defines which SSL port the JMX service will bind to
     *
     * @see #setJMXConnectorPort(int)
     * @since 7.1.0.3
     */
    public static final String PROP_JMX_CONNECTOR_PORT = "JMXConnectorPort";

    /**
     * Sets the TransportMode on the IBM ORB properties to ChannelFramework
     * @since 7.1
     */
    public static final String PROP_ENABLE_CHANNEL_FRAMEWORK = "enableChannelFramework";

    /**
     * Enables the eXtremeMemory memory storage mode for the server.  When
     * enabled, all data is stored outside of the Java heap.
     * @since 7.1.1
     */
    public static final String PROP_ENABLE_EXTREMEMEMORY = "enableXM";

    /**
     * Sets the maximum amount of eXtremeMemory (in megabytes) to allocate when eXtremeMemory is enabled.
     * If not set or set to -1, the maximum is not set.
     * @since 7.1.1
     */
    public static final String PROP_MAXIMUM_EXTREMEMEMORY_SIZE = "maxXMSize";

    /**
     * The property prefix to use for all eXtremeIO transport channel properties.
     * The properties are in the form:<p>
     * <code> xioChannel.&lt;channelname&gt;&lt;propertyname&gt;=&lt;propertyvalue&gt; </code>
     * @since 7.1.1
     */
    public static final String PROP_XIOCHANNEL_PREFIX = "xioChannel";

    /**
     * The eXtremeIO transport channel name for TCP/IP communication.
     * @since 8.6
     */
    public static final String PROP_XIOCHANNELNAME_TCP = "XIOInboundTCP";

    /**
     * The eXtremeIO transport container channel property name for non-secure TCP.
     * @since 7.1.1
     * @deprecated 8.6
     */
    public static final String PROP_XIOCHANNELNAME_CONTAINER_TCP_NON_SECURE = "xioContainerTCPNonSecure";

    /**
     * The eXtremeIO transport container channel property name for non-secure TCP.
     * @since 7.1.1
     * @deprecated 8.6
     */
    public static final String PROP_XIOCHANNELNAME_CONTAINER_TCP_SECURE = "xioContainerTCPSecure";

    /**
     * The eXtremeIO channel property to identify the port.
     * @since 7.1.1
     */
    public static final String PROP_XIOCHANNELPROP_PORT = "Port";

    /**
     * Sets the minimum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @since 7.1.1
     */
    public static final String PROP_MINIMUM_XIO_WORKER_THREAD_POOL_SIZE = "minXIOWorkerThreads";

    /**
     * Sets the maximum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @since 7.1.1
     */
    public static final String PROP_MAXIMUM_XIO_WORKER_THREAD_POOL_SIZE = "maxXIOWorkerThreads";

    /**
     * Sets the minimum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @since 7.1.1
     */
    public static final String PROP_MINIMUM_XIO_NETWORK_THREAD_POOL_SIZE = "minXIONetworkThreads";

    /**
     * Property string for maximum number of waiting messages for XIO transport to process.
     * The messages are not received after the limit has been reached.
     *
     * @since 7.1.1
     * @deprecated 8.6.0
     */
    public static final String PROP_XIO_WAITING_MESSAGE_QUEUE_LIMIT = "xioWaitingMessageQueueLimit";

    /**
     * Sets the maximum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @since 7.1.1
     */
    public static final String PROP_MAXIMUM_XIO_NETWORK_THREAD_POOL_SIZE = "maxXIONetworkThreads";



    /**
     * Sets the replication idle level or the maximum amount of time a
     * replica will sleep between initiated replication with the primary when
     * replication changes are not received.
     */
    // public static final String PROP_REPLICATION_IDLE_LEVEL = "replicationIdleLevel";

    /**
     * Constant representing a replication idle level at a short sleep.<p>
     *
     * A replication idle level set to short may slightly delay replication when a primary changes data again
     * after a period of no change, but also decreases CPU and network utilization during times of low
     * data change in comparison to when the system is actively replicating data.
     */
    public static final int REPLICATION_IDLE_LEVEL_SHORT = 0;

    /**
     * Constant representing a replication idle level that is turned off.<p>
     *
     * A replication idle level set to none does not change the replication polling if there is a period
     * of data not changing.
     */
    public static final int REPLICATION_IDLE_LEVEL_NONE = -1;

    /**
     * Constant representing a replication idle level at a relaxed rate.<p>
     *
     * A replication idle level set to relaxed may delay replication when a primary changes data again
     * after a period of no change, but also decreases CPU and network utilization during times of low
     * data change.
     */
    public static final int REPLICATION_IDLE_LEVEL_RELAXED = 1;

    // HPEL variables ///////////////////////////////////////////////////////
	/**
     * Indicates if the HPEL feature is to be enable or not: true for enabled
     * false for disabled. HPEL is disabled by default.
     * @since 8.6
     */
    public final String PROP_HPEL_ENABLED = "hpelEnabled";

    /**
     * Indicates if the HPEL feature is to purge log files by size. This is enabled by default.
     * @since 8.6
     */
    public final String PROP_HPEL_PURGE_BY_SIZE_ENABLED = "hpelEnablePurgeBySize";

    /**
     * Indicates if the HPEL feature is to purge log files by time. This is enabled by default.
     * @since 8.6
     */
    public final String PROP_HPEL_PURGE_BY_TIME_ENABLED = "hpelEnablePurgeByTime";

    /**
     * Indicates if the HPEL feature is to enable file switch. This is disabled by default.
     * @since 8.6
     */
    public final String PROP_HPEL_FILE_SWITCH_ENABLED = "hpelEnableFileSwitch";

    /**
     * Indicates if the HPEL feature is to enable buffering. This is disabled by default.
     * @since 8.6
     */
    public final String PROP_HPEL_BUFFERING_ENABLED = "hpelEnableBuffering";

    /**
     * Indicates if the HPEL text files are to include tracing. This is disabled by default.
     * @since 8.6
     */
    public final String PROP_HPEL_INCLUDE_TRACE = "hpelIncludeTrace";

    /**
     * Indicates the action to be performed when the disk space has been exceeded (String value)
     * @since 8.6
     */
    public final String PROP_HPEL_OUT_OF_SPACE_ACTION = "hpelOutOfSpaceAction";

    /**
     * Indicates the format of the log files to be generated (String value).
     * @since 8.6
     */
    public final String PROP_HPEL_OUTPUT_FORMAT = "hpelOutputFormat";

    /**
     * Indicates the Maximum size of files generate in megabytes (long value).
     * @since 8.6
     */
    public final String PROP_HPEL_MAX_REPOSITORY_SIZE = "hpelMaxRepositorySize";

    /**
     * Indicates the maximum retention time to hold files (long value).
     * @since 8.6
     */
    public final String PROP_HPEL_MAX_RETENTION_TIME = "hpelMaxRetentionTime";

    /**
     * Indicates the file switch hour (int value).
     * @since 8.6
     */
    public final String PROP_HPEL_FILE_SWITCH_HOUR = "hpelFileSwitchHour";

    /**
     * Indicates the file repository path (int value).
     * @since 8.6
     */
    public final String PROP_HPEL_REPOSITORY_PATH = "hpelRepositoryPath";

    ////////////////////////////////////////////////////////////////////////////

    // Syslogd variables ///////////////////////////////////////////////////////
    /**
     * Indicates whether to enable the transmission (over UDP) of log messages 
     * to a remote syslogd server. 
     * 
     * Possible values are true for enabled, false for disabled.
     *  
     * Syslog is disabled by default.
     * @since 8.6
     */
    public final String PROP_SYSLOG_ENABLED = "syslogEnabled";

    /**
     * Indicates the receiving syslogd server fully-qualified domain name 
     * or IP address.
     * @since 8.6
     */
    public final String PROP_SYSLOG_HOST_NAME = "syslogHostName";

    /**
     * Indicates the syslogd server port number on which to receive syslog messages.
     * @since 8.6
     */
    public final String PROP_SYSLOG_HOST_PORT = "syslogHostPort";

    /**
     * Indicates the syslog facility with which to send out syslog messages
     * @since 8.6
     */
    public final String PROP_SYSLOG_FACILITY = "syslogFacility";

    /**
     * Indicates the log level threshold of messages to trasmit to the 
     * configured syslogd server 
     *
     * Possible values: {warning, error}
     * @since 8.6
     */
    public final String PROP_SYSLOG_THRESHOLD = "syslogThreshold";

    ///////////////////////////////////////////////////////////////////////////

    // STATS_TO_CSV ////////////////////////////////////////////////////////////

    /**
     * When set to true, enables log data for the JVM to be written to a CSV file.
     *
     * @since 7.1.1
     */
    public final String PROP_JVM_STATS_ENABLED = "jvmStatsLoggingEnabled";

    /**
     * Indicates the maximum number of CSV statistics files that are generated for
     * the JVM Stats.
     *
     * @since 7.1.1
     */
    public final String PROP_MAX_JVM_STATS_FILES = "maxJVMStatsFiles";

    /**
     * Indicates the maximum file size, in megabytes, of the CSV statistics files
     * for the JVM.
     *
     * @since 7.1.1
     */
    public final String PROP_MAX_JVM_STATS_FILE_SIZE = "maxJVMStatsFileSize";

    /**
     * Specifies the file name of the CSV statistics file for the JVM.
     *
     * @since 7.1.1
     */
    public final String PROP_JVM_STATS_FILE_NAME = "jvmStatsFileName";

    /**
     * Specifies the write rate of the CSV statistics files for the JVM in seconds.
     *
     * @since 7.1.1
     */
    public final String PROP_JVM_STATS_WRITE_RATE = "jvmStatsWriteRate";


    /**
     * When set to true, enables log data for the Map to be written to a CSV file.
     *
     * @since 7.1.1
     */
    public final String PROP_MAP_STATS_ENABLED = "mapStatsLoggingEnabled";

    /**
     * Indicates the maximum number of CSV statistics files that are generated for the Map Stats.
     *
     * @since 7.1.1
     */
    public final String PROP_MAX_MAP_STATS_FILES = "maxMapStatsFiles";

    /**
     * Indicates the maximum file size, in megabytes, of the CSV statistics files for the Map Stats.
     *
     * @since 7.1.1
     */
    public final String PROP_MAX_MAP_STATS_FILE_SIZE = "maxMapStatsFileSize";

    /**
     * Specifies the file name of the CSV statistics file for the Map Stats.
     *
     * @since 7.1.1
     */
    public final String PROP_MAP_STATS_FILE_NAME = "mapStatsFileName";

    /**
     * Specifies the write rate of the CSV statistics files for the Map Stats in seconds.
     *
     * @since 7.1.1
     */
    public final String PROP_MAP_STATS_WRITE_RATE = "mapStatsWriteRate";


    /**
     * When set to true, enables log data for the Map to be written to a CSV file.
     *
     * @since 7.1.1
     */
    public final String PROP_OG_STATS_ENABLED = "ogStatsLoggingEnabled";

    /**
     * Indicates the maximum number of CSV statistics files that are generated for the Object Grid Stats.
     *
     * @since 7.1.1
     */
    public final String PROP_MAX_OG_STATS_FILES = "maxOGStatsFiles";

    /**
     * Indicates the maximum file size, in megabytes, of the CSV statistics files for the Object Grid Stats.
     *
     * @since 7.1.1
     */
    public final String PROP_MAX_OG_STATS_FILE_SIZE = "maxOGStatsFileSize";

    /**
     * Specifies the file name of the CSV statistics file for the Object Grid Stats.
     *
     * @since 7.1.1
     */
    public final String PROP_OG_STATS_FILE_NAME = "ogStatsFileName";

    /**
     * Specifies the write rate of the CSV statistics files for the Object Grid Stats in seconds.
     *
     * @since 7.1.1
     */
    public final String PROP_OG_STATS_WRITE_RATE = "ogStatsWriteRate";

	/** Defines the root directory to use when disk overflow is enabled.
	 *
	 * @see #setDiskStoragePath(String)
	 * @since 8.6
	 */
	public static final String PROP_DISK_STORAGE_PATH = "diskStoragePath";

	/** Defines maximum number of bytes that can be used by this JVM for overflowing
	 * content to disk.
	 *
	 * @see #setDiskOverflowCapBytes(long)
	 * @since 8.6
	 */
	public static final String PROP_DISK_OVERFLOW_CAP_BYTES = "diskOverflowCapBytes";

	/** Defines minimum number of bytes that must remain free on disk in the
	 * partition named by PROP_DISK_STORAGE_PATH.
	 *
	 * @see #getDiskOverflowMinDiskSpaceBytes
	 * @since 8.6
	 */
	public static final String PROP_DISK_OVERFLOW_MIN_DISK_SPACE_BYTES = "diskOverflowMinDiskSpaceBytes";

	/**
	 * Defines whether the capability of overflowing data from main-memory to
	 * disk is enabled.
	 *
	 * @see #setDiskOverflowEnabled(boolean)
	 * @since 8.6
	*/
	public static final String PROP_DISK_OVERFLOW_ENABLED = "diskOverflowEnabled";

	/**
	 * Sets the timeout for server requests using the XIO transport. The timeout is set in seconds.
	 * The default xioTimeout for server requests is set to 30 seconds. The valid range is timeout >= 1.
	 *
	 * @since 8.6
	 */
	public static final String PROP_XIO_TIMEOUT ="xioTimeout";


  /**
   * Used by users of the embedded server API in WebSphere Application Server, when they want to stop the
   * eXtreme Scale containers manually using the {@linkplain com.ibm.websphere.objectgrid.server.Container#terminate} API.<p>
   *
   * The default is true, which allows the eXtreme Scale runtime to automatically stop the container servers when the application
   * server is shut down.
   * @see #setWASManagedContainerShutdownEnabled(boolean)
   * @since 7.1.1.1 PM67857
   */
  public final String PROP_WAS_MANAGED_CONTAINER_SHUTDOWN_ENABLED = "wasManagedContainerShutdownEnabled";

    //////////////////////////////////////////////////////////////////////////////



    /**
     * Loads the properties from <code>input</code> into this property object.
     *
     * @param input
     *            A stream to load properties from.
     * @throws IOException
     *             If the properties input has an invalid format.
     */
    public void load(InputStream input) throws IOException;

    /**
     * Writes the properties from this object into <code>output</code>.
     *
     * @param output
     *            A stream to write out the properties to.
     * @throws IOException
     *             If the properties input has an invalid format.
     */
    public void store(OutputStream output) throws IOException;

    /**
     * Answers whether or not this process is configured to be an ObjectGrid server process.
     *
     * @return Whether or not this process is a server process.
     */
    public boolean isServer();

    /**
     * Retrieves the name used in identifying this server.
     *
     * @return The name of this server process.
     */
    public String getServerName();

    /**
     * Sets the name used to identify this process.
     * <p>
     *
     * This value is optional. For WebSphere Application Server processes, the server name match the WebSphere
     * Application Server process name. For other processes, the name is automatically generated in the form:
     *
     * <pre>
     * ogserver-[index]
     * </pre>
     *
     * For example, the first server name will be: ogserver-0, the second server: ogserver-1.
     *
     * @param serverName
     *            The name of the server process.
     */
    public void setServerName(String serverName);

    /**
     * Retrieve the zone name.
     *
     * @return The name of the zone that this server belongs to.
     * @see #DEFAULT_ZONE
     */
    public String getZoneName();

    /**
     * Sets the zone that this server belongs to. This can only be set initially, once the server has connected to the
     * catalog service the zone is set and can not be modified.
     *
     * @param zoneName
     *            The name of the zone that this server belongs to.
     * @see #DEFAULT_ZONE
     */
    public void setZoneName(String zoneName);

    /**
     * Retrieves the host to be used by the ORB. The listener host property defaults to 'localhost'.
     *
     * @return The host that the ORB is going to be bound to.
     */
    public String getListenerHost();

    /**
     * Assigns the host that the ORB is to bind to.
     *
     * @param listenerHost
     *            The host name to be used when creating the ORB.
     */
    public void setListenerHost(String listenerHost);

    /**
     * Retrieves the inbound port used by the dominant transport. If the ORB transport is used, then this is the ORB
     * non-secure listener port. If the XIO transport is used, then then is the non-secure inbound port.
     *
     * @return The inbound port in use by the dominant transport.
     */
    public int getListenerPort();

    /**
     * Assigns the port that the ORB is to bind to.
     *
     * @param listenerPort
     *            The port to be used when creating the ORB.
     */
    public void setListenerPort(int listenerPort);

    /**
     * Retrieves the catalog service corbaloc string.
     *
     * @return The catalog service corbaloc string.
     */
    public String getCatalogServiceBootstrap();

    /**
     * Sets the catalog service grid addresses in the form [host:port,host:port,...]. This should generally be enough
     * entries that redundancy is sufficient but does not need to be the complete set.
     *
     * @param catalogServiceAddress
     *            The addresses of all the catalog services that can be used to bootstrap to.
     */
    public void setCatalogServiceBootstrap(String catalogServiceAddress);

    /**
     * Retrieves the port to be used by the JMX connection. The JMX service port defaults to the ORB listener port.
     *
     * @return The port that the JMX connection is to be bound to.
     * @see #getJMXConnectorPort()
     */
    public int getJMXServicePort();

    /**
     * Assigns the port that the JMX service is to bind to. If this is not set the JMX service is bound to the ORB port.
     *
     * @param jmxServicePort
     *            The port that the JMX service is to bind to.
     * @see #setJMXConnectorPort(int)
     */
    public void setJMXServicePort(int jmxServicePort);

    /**
     * Retrieves the JMX connector port that the JMX service is to bind to.
     * <p>
     * If the JMX service port is specified and this port is not specified, the port that is chosen depends on different
     * criteria. If SSL is not enabled, the JMXServicePort is used for the JMXConnectorPort is possible. If that fails,
     * an ephemeral port is used. With SSL enabled, an ephemeral port is always used if the JMXConnectorPort is not
     * provided.
     * <p>
     * The format of the JMX URL is the following format when a JMXServicePort is provided:
     *   service:jmx:rmi://listenerHost:JMXConnectorPort/jndi/rmi://listenerHost:JMXServicePort/objectgrid/MBeanServer
     * <p>
     * If the JMX service port is not specified, this property is ignored and the ORB is used for both TCP and
     * SSL configurations.
     *
     * @return The connector port that the JMX connection is to bind to.
     * @since 7.1.0.3
     */
    public int getJMXConnectorPort();

    /**
     * Assigns the JMX connector port that the JMX service is to bind to.
     * <p>
     * If the JMX service port is specified and this port is not specified, the port that is chosen depends on different
     * criteria. If SSL is not enabled, the JMXServicePort is used for the JMXConnectorPort is possible. If that fails,
     * an ephemeral port is used. With SSL enabled, an ephemeral port is always used if the JMXConnectorPort is not
     * provided.
     * <p>
     * The format of the JMX URL is the following format when a JMXServicePort is provided:
     *   service:jmx:rmi://listenerHost:JMXConnectorPort/jndi/rmi://listenerHost:JMXServicePort/objectgrid/MBeanServer
     * <p>
     *   The rmiHost and namingHost in the URL uses the listenerHost property.
     * <p>
     * If the JMX service port is not specified, this property is ignored and the ORB is used for both TCP and
     * SSL configurations.
     *
     * @param jmxConnectorPort
     *            The connector port that the JMX connection is to bind to.
     * @since 7.1.0.3
     * @see #setJMXServicePort(int)
     */
    public void setJMXConnectorPort(int jmxConnectorPort);

    /**
     * Sets the regular expression filter to be applied to log records messages that are forwarded
     * as JMX notifications by the ObjectGrid server through the LoggerListenerJMXNotifier
     * @param regExFilter
     *            The initial regular expression filter for log record messages.
     * @since 8.6
     */
    public void setLogNotificationFilter(String regExFilter);

    /**
     * Retrieves the initial trace specification the server should use.
     *
     * @return The initial trace specification.
     */
    public String getTraceSpecification();

    /**
     * Assigns the specification that the server should use on initialization.
     *
     * @param traceSpecification
     *            The initial trace specification for the server to use.
     */
    public void setTraceSpecification(String traceSpecification);

    /**
     * Retrieves a reference to the file name the server will use for trace.
     *
     * @return The trace file name.
     */
    public String getTraceFileName();

    /**
     * Assigns the file to use for logging trace.
     *
     * @param traceFileName
     *            The file name to use for logging trace.
     */
    public void setTraceFileName(String traceFileName);

    /**
     * Retrieves the statistics specification that this process should start with.
     *
     * @return The initial stats specification.
     */
    public String getStatsSpecification();

    /**
     * Sets the initial statistics specification that the ObjectGrid server should start with.
     *
     * @param statsSpecification
     *            The initial stats specification.
     * @see StatsSpec
     */
    public void setStatsSpecification(String statsSpecification);

    /**
     * Retrieves the regular expression filter to be applied to log records messages that are forwarded
     * as JMX notifications by the ObjectGrid server through the LoggerListenerJMXNotifier
     * @return
     *            The current regular expression filter for log record messages.
     * @since 8.6
     */
    public String getLogNotificationFilter();


    /**
     * Retrieves whether or not the MBeans are going to be registered.
     *
     * @return Whether or not the ObjectGrid MBeans will be registered.
     */
    public boolean isMBeanEnabled();

    /**
     * Enables or disables ObjectGrid MBean registration for this process.
     *
     * @param enableMBeans
     *            Whether or not MBeans should be registered.
     */
    public void setMBeansEnabled(boolean enableMBeans);

    /**
     * Retrieves the working directory for the ObjectGrid server runtime.
     *
     * @return The ObjectGrid server runtime working directory.
     */
    public String getWorkingDirectory();

    /**
     * Sets the working directory for the ObjectGrid server runtime to use.
     *
     * @param workingDirectory
     *            The directory that the ObjectGrid will default activity to.
     */
    public void setWorkingDirectory(String workingDirectory);

    /**
     * Retrieves the stream to file setting.
     *
     * @return Whether or not the system streams will be written to file.
     */
    public boolean isSystemStreamToFileEnabled();

    /**
     * Sets the trace logic to write SystemOut and SystemErr to file, or not write to file.
     *
     * @param streamsToFile
     *            Whether or not the system streams should be written to file.
     */
    public void setSystemStreamsToFileEnabled(boolean streamsToFile);

    /**
     * Sets the port number for the High Availability Manager (HA Manager).
     *
     * @param port
     *            the port number
     * @since WAS XD 6.1.0.3
     */
    public void setHAManagerPort(int port);

    /**
     * Retrieve the High Availability Manager (HA Manager) port.
     *
     * @return the port
     * @since WAS XD 6.1.0.3
     */
    public int getHAManagerPort();

    /**
     * Sets the memory threshold usage percentage, (set up to 100% ).
     *
     * @param threshold
     *            the memory threshold percentage
     * @throws IllegalArgumentException
     *             if the threshold is out of range.
     * @since WAS XD 6.1.0.3
     */
    public void setMemoryThresholdPercentage(int threshold);

    /**
     * Gets the memory threshold usage percentage, -1 indicates that it was not set.
     *
     * @return the memory threshold percentage
     * @since WAS XD 6.1.0.3
     */
    public int getMemoryThresholdPercentage();

    /**
     * Sets the minimum number of threads in the pool.
     *
     * @param minNumThreads
     *            The minimum number of threads to hold in the pool.
     * @since 7.0.0.0 FIX2
     */
    public void setMinimumThreadPoolSize(int minNumThreads);

    /**
     * Retrieve the minimum number of pooled threads.
     *
     * @return The minimum number of threads to hold in the pool.
     * @since 7.0.0.0 FIX2
     */
    public int getMinimumThreadPoolSize();

    /**
     * Sets the maximum number of threads in the pool.
     *
     * @param maxNumThreads
     *            The maximum number of threads to use.
     * @since 7.0.0.0 FIX2
     */
    public void setMaximumThreadPoolSize(int maxNumThreads);

    /**
     * Retrieve the maximum number of pooled threads.
     *
     * @return The maximum number of threads to use.
     * @since 7.0.0.0 FIX2
     */
    public int getMaximumThreadPoolSize();

    /**
     * Returns if the ChannelFramework setting was enabled for the TransportMode on the IBM ORB properties
     * @return Answers true if the ChannelFramework setting is enabled.
     * @since 7.1
     */
    public boolean isChannelFrameworkEnabled();

    /**
     * Sets the TransportMode on the IBM ORB properties to ChannelFramework
     * @param enableChannelFramework True if the ChannelFramework is enabled.
     * @since 7.1
     */
    public void setChannelFramework(boolean enableChannelFramework);

    /**
     * Sets the channel properties for the eXtremeIO transport, replacing all current
     * channel properties with the specified map of properties.  To add a new property:
     * <pre>
     * ServerProperties serverProps = ServerFactory.getServerProperties();
     * Map&lt;String, Properties&gt; allChannelProps = serverProps.getXIOChannelProps();
     * Properties channelProps = allChannelProps.get("THECHANNEL");
     * channelProps.put("PORT", "9000");
     * serverProps.setXIOChannelProps(channelProps);
     * </pre>
     * @param channelProperties
     *          The map of channel names to the properties associated with the channel.
     * @since 7.1.1
     */
    public void setXIOChannelProps(Map<String, ? extends Properties> channelProperties);

    /**
     * Returns a copy of the map of overriding properties for all channels.
     * @return the map of channel names to the properties associated with the channel.
     * @since 7.1.1
     */
    public Map<String, Properties> getXIOChannelProps();

    /**
     * Enables the eXtremeMemory memory storage mode for the server.  When
     * enabled, all data is stored outside of the Java heap.
     * @since 7.1.1
     */
    public void setEnableXM(boolean enableXM);

    /**
     * Answers true if eXtremeMemory is enabled.
     * @since 7.1.1
     */
    public boolean isEnableXM();

    /**
     * Sets the maximum amount of eXtremeMemory (in megabytes) to allocate when eXtremeMemory is enabled.
     * This is ignored if {@linkplain #isEnableXM()} answers false.
     * @param sizeInMegabytes the amount of eXtremeMemory to allocate in megabytes, or -1 if the maximum size is not set.
     * @since 7.1.1
     */
    public void setMaximumXMSize(int sizeInMegabytes);

    /**
     * Retrieves the maximum amount of eXtremeMemory (in megabytes) to allocate when eXtremeMemory
     * is enabled.
     * @return the amount of eXtremeMemory to allocate in megabytes, or -1 if not set.
     * @since 7.1.1
     */
    public int getMaximumXMSize();

    /**
     * Sets the replicationIdleLevel. The replication idle level tells the replicas
     * how much to back off during idle times. The default is 1.<p>
     *
     * @param frequencyLevel one of three values: -1, 0 or 1 as defined by the
     * constants that begin with name <tt>REPLICATION_IDLE_LEVEL</tt>.
     * @throws IllegalArgumentException if the frequencyLevel is out of range.
     * @since WAS XD 6.1.0.3

     * Valid values include:
     * <ul>
     * <li> {@link #REPLICATION_IDLE_LEVEL_SHORT}
     * <li> {@link #REPLICATION_IDLE_LEVEL_RELAXED}
     * <li> {@link #REPLICATION_IDLE_LEVEL_NONE}
     * </ul>
     *
     * @param replicationIdleLevel one of three values: -1, 0 or 1 as defined by the
     * constants that begin with name <tt>REPLICATION_IDLE_LEVEL</tt>.
     * @throws IllegalArgumentException if the replicationIdleLevel is out of range.
     */
    // public void setReplicationIdleLevel(int replicationIdleLevel);

    /**
     * Retrieves the replicationIdleLevel
     *
     * * @return the replication idle level:  -1, 0 or 1 as defined by the
     * constants that begin with name <tt>REPLICATION_IDLE_LEVEL</tt>.
     */
    // public int getReplicationIdleLevel();

    // HPEL ////////////////////////////////////////////////////////////////
    /**
     * Indicates if the HPEL feature is to be enable or not: true for enabled
     * false for disabled. HPEL is disabled by default.
     * @return The value of true or false indicating if HPEL is enabled or
     * disabled
     * @since 8.6
     */
    public boolean isHPELEnabled();

    /**
     * When set to true, enables HPEL logging.
     * @param hpelEnabled
     * 	      enables log data to be handled by HPEL.
     * @since 8.6
     */
    public void setHPELEnabled(boolean hpelEnabled);

    /**
     * Indicates if the HPEL feature purge by size feature enable or not: true for enabled
     * false for disabled. Purge by size is enabled by default.
     * @return The value of true or false indicating if HPEL is enabled or
     * disabled
     * @since 8.6
     */
    public boolean isHPELPurgeBySizeEnabled();

    /**
     * When set to true, enables HPEL purge by size.
     * @param hpelPurgeBySizeEnabled
     * 	      enables purge by size.
     * @since 8.6
     */
    public void setHPELPurgeBySizeEnabled(boolean hpelPurgeBySizeEnabled);

    /**
     * Indicates if the HPEL feature purge by size is to be enable or not: true for enabled
     * false for disabled. HPEL purge by time is enabled by default.
     * @return The value of true or false indicating if HPEL purge by size is enabled or
     * disabled
     * @since 8.6
     */
    public boolean isHPELPurgeByTimeEnabled();

    /**
     * When set to true, enables HPEL purge by time.
     * @param hpelPurgeByTimeEnabled
     * 	      enables purge by time.
     * @since 8.6
     */
    public void setHPELPurgeByTimeEnabled(boolean hpelPurgeByTimeEnabled);

    /**
     * Indicates if the HPEL file switch feature is to be enable or not: true for enabled
     * false for disabled. HPEL file switch is enabled by default.
     * @return The value of true or false indicating if HPEL purge by time is enabled or
     * disabled
     * @since 8.6
     */
    public boolean isHPELFileSwitchEnabled();

    /**
     * When set to true, enables HPEL file switch.
     * @param hpelFileSwitchEnabled
     * 	      enables file switch.
     * @since 8.6
     */
    public void setHPELFileSwitchEnabled(boolean hpelFileSwitchEnabled);

    /**
     * Indicates if the HPEL file switch feature is to be enable or not: true for enabled
     * false for disabled. HPEL buffering is disabled by default.
     * @return The value of true or false indicating if HPEL buffering is enabled or
     * disabled
     * @since 8.6
     */
    public boolean isHPELBufferingEnabled();

    /**
     * When set to true, enables HPEL buffering.
     * @param hpelBufferingEnabled
     * 	      enables buffering.
     * @since 8.6
     */
    public void setHPELBufferingEnabled(boolean hpelBufferingEnabled);

    /**
     * Indicates if the HPEL include trace feature is to be enable or not: true for enabled
     * false for disabled. HPEL include trace is disabled by default.
     * @return The value of true or false indicating if HPEL include trace is enabled or
     * disabled
     * @since 8.6
     */
    public boolean isHPELIncludeTrace();

    /**
     * When set to true, enables HPEL include trace.
     * @param hpelIncludeTrace
     * 	      enables include trace.
     * @since 8.6
     */
    public void setHPELIncludeTrace(boolean hpelIncludeTrace);

    /**
     * Specifies the HPEL Maximum Repository Size
     * @param hpelMaxRepositorySize
     * 			The HPEL Maximum Repository Size
     * @since 8.6
     */
    public void setHPELMaxRepositorySize(long hpelMaxRepositorySize);

    /**
     * Indicates the HPEL Maximum Repository Size
     * @return The HPEL Maximum Repository Size
     * @since 8.6
     */
    public long getHPELMaxRepositorySize();

    /**
     * Specifies the HPEL Maximum Retention Time
     * @param hpelMaxRetentionTime
     * 			The HPEL Retention Time
     * @since 8.6
     */
    public void setHPELMaxRetentionTime(long hpelMaxRetentionTime);

    /**
     * Indicates the HPEL Maximum Retention Time
     * @return The HPEL Maximum Retention Time
     * @since 8.6
     */
    public long getHPELMaxRetentionTime();

    /**
     * Specifies the HPEL File Switch Hour
     * @param hpelFileSwitchHour
     * 			The HPEL File Switch Hour
     * @since 8.6
     */
    public void setHPELFileSwitchHour(int hpelFileSwitchHour);

    /**
     * Indicates the HPEL File Switch Hour
     * @return The HPEL File Switch Hour
     * @since 8.6
     */
    public int getHPELFileSwitchHour();

    /**
     * Specifies the HPEL Out Of SpaceAction
     * @param hpelOutOfSpaceAction
     * 			The HPEL Out Of SpaceAction
     * @since 8.6
     */
    public void setHPELOutOfSpaceAction(String hpelOutOfSpaceAction);

    /**
     * Indicates the HPEL Out Of Space Action
     * @return The HPEL Out Of Space Action
     * @since 8.6
     */
    public String getHPELOutOfSpaceAction();

    /**
     * Specifies the HPEL Output Format
     * @param hpelOutputFormat
     * 			The HPEL Output Format
     * @since 8.6
     */
    public void setHPELOutputFormat(String hpelOutputFormat);

    /**
     * Indicates the HPEL Output Format
     * @return The HPEL Output Format
     * @since 8.6
     */
    public String getHPELOutputFormat();

    /**
     * Specifies the HPEL Repository Path
     * @param hpelRepositoryPath
     * 			The HPEL Repository Path
     * @since 8.6
     */
    public void setHPELRepositoryPath(String hpelRepositoryPath);

    /**
     * Indicates the HPEL Repository Path
     * @return The HPEL Repository Path
     * @since 8.6
     */
    public String getHPELRepositoryPath();

    ///////////////////////////////////////////////////////////////////////////


    // Syslogd ////////////////////////////////////////////////////////////////
    /**
     * Indicates if the Syslogd feature is to be enable or not: true for enabled
     * false for disabled. Syslog is disabled by default.
     * @return The value of true or false indicating if Syslog is enabled or
     * disabled
     * @since 8.6
     */
    public boolean isSyslogEnabled();

    /**
     * When set to true, enables Syslog logging.
     * @param syslogEnabled
     * 	      enables log data to be sent to Syslog server.
     * @since 8.6
     */
    public void setSyslogEnabled(boolean syslogEnabled);

    /**
     * Indicates the Syslogd Server host full name or IP address
     * @return The Syslogd Server host full name or IP address
     * @since 8.6
     */
    public String getSyslogHostName();

    /**
     * Specifies the Syslogd Server host full name or IP address
     * @param syslogHostName
     * 			The Syslogd Server host full name or IP address
     * @since 8.6
     */
    public void setSyslogHostName(String syslogHostName);

    /**
     * Indicates the Syslogd Server Port Number
     * @return The Syslogd Server Port Number
     * @since 8.6
     */
    public int getSyslogHostPort();

    /**
     * Specifies the Syslogd Server host port number
     * @param syslogHostPort
     * 			The Syslogd Server host Port Number
     * @since 8.6
     */
    public void setSyslogHostPort(int syslogHostPort);

    /**
     * Indicates the Syslogd Facility
     * @return The Syslogd Facility
     * @since 8.6
     */
    public String getSyslogFacility();

    /**
     * Specifies the Syslogd Facility
     * @param syslogFacility
     * 			The Syslogd Server Facility
     * @since 8.6
     */
    public void setSyslogFacility(String syslogFacility);

    /**
     * Indicates the Syslogd Threshold.  Possible value include warning and error
     * @return The Syslogd Threshold
     * @since 8.6
     */
    public String getSyslogThreshold();

    /**
     * Specifies the Syslogd Threshold
     * @param syslogThreshold
     * 			The Syslogd Server Threshold
     * @since 8.6
     */
    public void setSyslogThreshold(String syslogThreshold);
    ///////////////////////////////////////////////////////////////////////////


    // STATS_TO_CSV ////////////////////////////////////////////////////////////////
    /**
     * Indicates the maximum number of CSV statistics files that are generated for
     * the JVM Stats.
     * @return The maximum number of CSV statistics files that are generated for
     * the JVM Stats.
     * @since 7.1.1
     */
    public int getMaximumJVMStatsFiles();

    /**
     * Gets whether of not log data for the JVM will to be written to a CSV file.
     * @return When true, log data for the JVM will be written to a CSV file.
     * @since 7.1.1
     */
    public boolean isJvmStatsLoggingEnabled();

    /**
     * When set to true, enables log data for the JVM to be written to a CSV file.
     * @param jvmStatsLoggingEnabled
     * 	      enables log data for the JVM to be written to a CSV file.
     * @since 7.1.1
     */
    public void setJvmStatsLoggingEnabled(boolean jvmStatsLoggingEnabled);

    /**
     * Sets the maximum number of CSV statistics files that are generated for
     * the JVM Stats.
     * @param maximumJVMStatsFiles
     * 			 The maximum number of CSV statistics files that are generated for
     *                   the JVM Stats.
     * @since 7.1.1
     */
    public void setMaximumJVMStatsFiles(int maximumJVMStatsFiles);

    /**
     * Gets the maximum file size, in megabytes, of the CSV statistics files
     * for the JVM.
     * @return  The maximum file size, in megabytes, of the CSV statistics files
     *          for the JVM.
     * @since 7.1.1
     */
    public int getMaximumJVMStatsFileSize();

    /**
     * Sets the maximum file size, in megabytes, of the CSV statistics files
     * for the JVM.
     * @param maximumJVMStatsFileSize
     * 			The maximum file size, in megabytes, of the CSV statistics files
     *                  for the JVM.
     * @since 7.1.1
     */
    public void setMaximumJVMStatsFileSize(int maximumJVMStatsFileSize);

    /**
     * Gets JVM Stats file name
     * @return  The value of the associated property to set.
     * @since 7.1.1
     */
    public String getJvmStatsFileName();

    /**
     * Specifies the file name of the CSV statistics file for the JVM.
     * @param jvmStatsFileName
     * 			The file name of the CSV statistics file for the JVM.
     * @since 7.1.1
     */
    public void setJvmStatsFileName(String jvmStatsFileName);

    /**
     * Gets the write rate of the CSV statistics files for the JVM in seconds.
     * @return  The write rate of the CSV statistics files for the JVM in seconds.
     * @since 7.1.1
     */
    public int getJvmStatsWriteRate();

    /**
     * Sets the write rate of the CSV statistics files for the JVM in seconds.
     * @param jvmStatsWriteRate
     * 			The write rate of the CSV statistics files for the JVM in seconds.
     * @since 7.1.1
     */
    public void setJvmStatsWriteRate(int jvmStatsWriteRate);






    /**
     * Indicates the maximum number of CSV statistics files that are generated for
     * the Map Stats.
     * @return The maximum number of CSV statistics files that are generated for
     * the Map Stats.
     * @since 7.1.1
     */
    public int getMaximumMapStatsFiles();

    /**
     * Gets whether of not log data for the Map will to be written to a CSV file.
     * @return When true, log data for the Map will be written to a CSV file.
     * @since 7.1.1
     */
    public boolean isMapStatsLoggingEnabled();

    /**
     * When set to true, enables log data for the Map to be written to a CSV file.
     * @param mapStatsLoggingEnabled
     * 	      enables log data for the Map to be written to a CSV file.
     * @since 7.1.1
     */
    public void setMapStatsLoggingEnabled(boolean mapStatsLoggingEnabled);

    /**
     * Sets the maximum number of CSV statistics files that are generated for
     * the Map Stats.
     * @param maximumMapStatsFiles
     * 			 The maximum number of CSV statistics files that are generated for
     *                   the Map Stats.
     * @since 7.1.1
     */
    public void setMaximumMapStatsFiles(int maximumMapStatsFiles);

    /**
     * Gets the maximum file size, in megabytes, of the CSV statistics files
     * for the Map.
     * @return  The maximum file size, in megabytes, of the CSV statistics files
     *          for the Map.
     * @since 7.1.1
     */
    public int getMaximumMapStatsFileSize();

    /**
     * Sets the maximum file size, in megabytes, of the CSV statistics files
     * for the Map.
     * @param maximumMapStatsFileSize
     * 			The maximum file size, in megabytes, of the CSV statistics files
     *                  for the Map.
     * @since 7.1.1
     */
    public void setMaximumMapStatsFileSize(int maximumMapStatsFileSize);

    /**
     * Gets the file name of the CSV statistics file for the Map.
     * @return  The file name of the CSV statistics file for the Map.
     * @since 7.1.1
     */
    public String getMapStatsFileName();

    /**
     * Specifies the file name of the CSV statistics file for the Map.
     * @param mapStatsFileName
     * 			The file name of the CSV statistics file for the Map.
     * @since 7.1.1
     */
    public void setMapStatsFileName(String mapStatsFileName);

    /**
     * Gets the write rate of the CSV statistics files for the Map in seconds.
     * @return  The write rate of the CSV statistics files for the Map in seconds.
     * @since 7.1.1
     */
    public int getMapStatsWriteRate();

    /**
     * Sets the write rate of the CSV statistics files for the Map in seconds.
     * @param mapStatsWriteRate
     * 			The write rate of the CSV statistics files for the Map in seconds.
     * @since 7.1.1
     */
    public void setMapStatsWriteRate(int mapStatsWriteRate);







    /**
     * Indicates the maximum number of CSV statistics files that are generated for
     * the ObjectGrid Stats.
     * @return The maximum number of CSV statistics files that are generated for
     * the ObjectGrid Stats.
     * @since 7.1.1
     */
    public int getMaximumOGStatsFiles();

    /**
     * Gets whether of not log data for the ObjectGrid will to be written to a CSV file.
     * @return When true, log data for the ObjectGrid will be written to a CSV file.
     * @since 7.1.1
     */
    public boolean isOGStatsLoggingEnabled();

    /**
     * When set to true, enables log data for the ObjectGrid to be written to a CSV file.
     * @param ogStatsLoggingEnabled
     * 	      enables log data for the ObjectGrid to be written to a CSV file.
     * @since 7.1.1
     */
    public void setOGStatsLoggingEnabled(boolean ogStatsLoggingEnabled);

    /**
     * Sets the maximum number of CSV statistics files that are generated for
     * the ObjectGrid Stats.
     * @param maximumOGStatsFiles
     * 			 The maximum number of CSV statistics files that are generated for
     *                   the ObjectGrid Stats.
     * @since 7.1.1
     */
    public void setMaximumOGStatsFiles(int maximumOGStatsFiles);

    /**
     * Gets the maximum file size, in megabytes, of the CSV statistics files
     * for the ObjectGrid.
     * @return  The maximum file size, in megabytes, of the CSV statistics files
     *          for the ObjectGrid.
     * @since 7.1.1
     */
    public int getMaximumOGStatsFileSize();

    /**
     * Sets the maximum file size, in megabytes, of the CSV statistics files
     * for the ObjectGrid.
     * @param maximumOGStatsFileSize
     * 			The maximum file size, in megabytes, of the CSV statistics files
     *                  for the ObjectGrid.
     * @since 7.1.1
     */
    public void setMaximumOGStatsFileSize(int maximumOGStatsFileSize);

    /**
     * Gets the file name of the CSV statistics file for the ObjectGrid.
     * @return  The file name of the CSV statistics file for the ObjectGrid.
     * @since 7.1.1
     */
    public String getOGStatsFileName();

    /**
     * Specifies the file name of the CSV statistics file for the ObjectGrid.
     * @param ogStatsFileName
     * 			The file name of the CSV statistics file for the ObjectGrid.
     * @since 7.1.1
     */
    public void setOGStatsFileName(String ogStatsFileName);

    /**
     * Gets the write rate of the CSV statistics files for the ObjectGrid in seconds.
     * @return  The write rate of the CSV statistics files for the ObjectGrid in seconds.
     * @since 7.1.1
     */
    public int getOGStatsWriteRate();

    /**
     * Sets the write rate of the CSV statistics files for the ObjectGrid in seconds.
     * @param ogStatsWriteRate
     * 			The write rate of the CSV statistics files for the ObjectGrid in seconds.
     * @since 7.1.1
     */
    public void setOGStatsWriteRate(int ogStatsWriteRate);

    /**
     * Retrieves the minimum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @return the mininum number of threads.
     * @since 7.1.1
     */
    public int getMinimumXIOWorkerThreads();

    /**
     * Sets the minimum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @param minThreads the minimum number of threads.
     * @since 7.1.1
     */
    public void setMinimumXIOWorkerThreads(int minThreads);

    /**
     * Sets the maximum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @return the maximum number of threads.
     * @since 7.1.1
     */
    public int getMaximumXIOWorkerThreads();

    /**
     * Retrieves the maximum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @param maxThreads the maximum number of threads.
     * @since 7.1.1
     */
    public void setMaximumXIOWorkerThreads(int maxThreads);

    /**
     * Gets the minimum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @return the minimum number of threads
     * @since 7.1.1
     */
    public int getMinimumXIONetworkThreads();

    /**
     * Sets the minimum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @param minThreads the minimum number of threads
     * @since 7.1.1
     */
    public void setMinimumXIONetworkThreads(int minThreads);

    /**
     * Gets the maximum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @return the maximum number of threads.
     * @since 7.1.1
     */
    public int getMaximumXIONetworkThreads() ;

    /**
     * Sets the maximum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @param maxThreads the maximum number of threads.
     * @since 7.1.1
     */
    public void setMaximumXIONetworkThreads(int maxThreads);

    /**
     * Sets the maximum number of messages waiting to be processed.
     *
     * @param maxMessages a negative number indicates no limit which may cause OutOfMemory error
     * in exceptional cases.
     *
     * @since 7.1.1
     */
    public void setMaximumXIOWaitingMessages(int maxMessages);

    /**
     * Gets the maximum number of messages waiting to be processed.
     *
     * @return maximum number of messages that can wait for processing. A negative number indicates no limit.
     *
     * @since 7.1.1
     */
    public int getMaximumXIOWaitingMessages();


	/** Returns true if and only if the disk-overflow feature is enabled, false otherwise.
	 * If the property is not set, the default value is "false".
	 *
	 * @return true if and only if the disk-overflow feature is enabled, false otherwise.
	 * @since 8.6
	 */
	public boolean isDiskOverflowEnabled();

	/** Sets the disk-overflow enablement feature.  This is ignored if
	 * {@linkplain #isEnableXM()} returns false.
	 *
	 * @param diskOverflowEnabled true if and only if the disk-overflow feature should be
	 * enabled, false otherwise.
	 * @since 8.6
	 */
	public void setDiskOverflowEnabled(boolean diskOverflowEnabled);

	/** Sets the root directory to use for disk overflow.  The effect is to
	 * create a new File instance using diskStoragePath as the parent abstract
	 * pathname and "wxsDiskStore" as the child pathname string.  For example, is
	 * client supplies "/tmp", the system will create a "/tmp/wxsDiskStore"
	 * directory and overflow to it.  All files that reside in this partition are
	 * deleted before data are overflowed to this location.  This is ignored if
	 * either {@linkplain #isEnableXM()} or {@linkplain #isDiskOverflowEnabled()}
	 * return false.
	 *
	 * @param diskStoragePath location of the root directory.
	 * @since 8.6
	 */
	public void setDiskStoragePath(String diskStoragePath);

	/** Returns the root directory used for disk overflow.  It is an error to
	 * access this value without having first set the property: that is, no default
	 * value is assigned to this property.
	 *
	 * @return The root directory used for disk overflow.
	 * @since 8.6
	 */
	public String getDiskStoragePath();

	/** Sets the maximum number of bytes that can be used by this JVM for
	 * overflowing bytes to disk.  This is ignored if either {@linkplain
	 * #isEnableXM()} or {@linkplain #isDiskOverflowEnabled()} return false.
	 *
	 * @param bytes the cap specified in bytes.
	 * @since 8.6
	 */
	public void setDiskOverflowCapBytes(long bytes);

	/** Returns the maximum number of bytes that can be used by this JVM when
	 * overflowing bytes to disk.  If this property is not set, the value
	 * defaults to Long.MAX_LONG.
	 *
	 * @return the maximum number of bytes that can be used by this JVM when
	 * overflowing bytes to disk.
	 * @since 8.6
	 */
	public long getDiskOverflowCapBytes();

	/** Sets the minimum number of bytes that must remain free on disk (in the
	 * partition named by diskStoragePath) when overflowing bytes to disk: a
	 * request to store data on disk will not be granted if the request will
	 * cause this constraint to be violated.  This is ignored if either
	 * {@linkplain #isEnableXM()} or {@linkplain #isDiskOverflowEnabled()} return
	 * false.
	 *
	 * @param bytes the minimum number of bytes that must remain free on disk.
	 * @since 8.6
	 */
	public void setDiskOverflowMinDiskSpaceBytes(long bytes);

	/** Returns the minimum number of bytes that must remain free on disk when
	 * overflowing bytes to disk.  If the property is not set, the default value
	 * is "0" bytes: that is, the system is allowed to fill the partition.
	 *
	 * @return the minimum number of bytes that must remain free on disk when
	 * overflowing bytes to disk.
	 * @since 8.6
	 */
	public long getDiskOverflowMinDiskSpaceBytes();
    /**
    * Answers true if WebSphere Application Server automatically stops the eXtreme Scale
    * container when the hosting application server is stopped and the container was started using
    * the eXtreme Scale
    * {@linkplain com.ibm.websphere.objectgrid.server.Server#createContainer(com.ibm.websphere.objectgrid.deployment.DeploymentPolicy)} API.<p>
    *
    * Answers false, if the application manually stops the container.
    * @return true if the container is stopped automatically, false if the application
    * @since 7.1.1.1 PM67857
    */
   public boolean isWASManagedContainerShutdownEnabled() ;

   /**
    * Set to true if the WebSphere Application Server should automatically stop the eXtreme Scale
    * container when the hosting application server is stopped and the container was started using
    * the eXtreme Scale
    * {@linkplain com.ibm.websphere.objectgrid.server.Server#createContainer(com.ibm.websphere.objectgrid.deployment.DeploymentPolicy)} API.<p>
    *
    * Set to false, if the application manually stops the container using the
    * {@linkplain com.ibm.websphere.objectgrid.server.Container#terminate} API.
    * @since 7.1.1.1 PM67857
    */
   public void setWASManagedContainerShutdownEnabled(boolean enabled) ;

   /**
    * Returns the current timeout for server requests using the XIO transport.
    *
    * @return the current timeout in seconds
    * @since 8.6
    */
   public int getXIOTimeout();

   /**
    * Sets the current timeout for server requests using the XIO transport. The timeout is
    * set in seconds. The valid range is timeout >= 1.
    *
    * @param timeout the timeout in seconds
    * @since 8.6
    */
   public void setXIOTimeout(int timeout);
}
