/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
*/
package com.ibm.websphere.objectgrid.spring;

import java.net.URL;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.core.io.Resource;
import org.springframework.util.Assert;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.ClientClusterContext;
import com.ibm.websphere.objectgrid.ObjectGridManagerFactory;
import com.ibm.websphere.objectgrid.security.config.ClientSecurityConfiguration;
import com.ibm.websphere.objectgrid.security.config.ClientSecurityConfigurationFactory;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.xs.NLSConstants;
import com.ibm.ws.xs.util.Messages;

/**
 * <p>This class is a Spring bean representing a Catalog Service Domain for use with
 * the WebSphere eXtreme Scale implementation of Spring's cache abstraction.</p> 
 * 
 * <p>Users must provide the catalog service endpoints used to connect to the eXtreme Scale cluster/domain. 
 * Users optionally may provide a client override XML and/or a client security configuration.</p>
 * 
 * @since 8.5, XC10
 */
public final class ObjectGridCatalogServiceDomainBean implements InitializingBean {
    
    final static String CLASS_NAME = ObjectGridCatalogServiceDomainBean.class.getName();
    
    final static TraceComponent tc = Tr.register(CLASS_NAME, Constants.TR_SPRING_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);
    
    private String catalogServiceEndpoints = null;
    private Resource clientOverrideXml;
    private Resource clientSecurityConfig;
    private ClientClusterContext ccc = null;
    private ClientSecurityConfiguration clientSC = null;

    /**
     * Sets the catalog service endpoints used to connect to the cluster/domain.
     * @param catalogServiceEndpoints The catalog service endpoints to connect to the cluster/domain
     */
    public void setCatalogServiceEndpoints(String catalogServiceEndpoints) {
        this.catalogServiceEndpoints = catalogServiceEndpoints;
    }

    /**
     * Sets the location of the client override XML.
     * @param clientOverrideXml The location of the client override XML.
     */
    public void setClientOverrideXml(Resource clientOverrideXml) {
        this.clientOverrideXml = clientOverrideXml;
    }

    /**
     * Sets the location of the client security configuration.
     * @param clientSecurityConfiguration The location of the client security configuration.
     */
    public void setClientSecurityConfig(Resource clientSecurityConfiguration) {
        this.clientSecurityConfig = clientSecurityConfiguration;
    }
    
    /**
     * Returns a <code>ClientClusterContext</code> representing the cluster/domain ObjectGrid configuration 
     * to which the client is connected.
     * @return a <code>ClientClusterContext</code> representing the cluster/domain to which the client is connected.
     */
    ClientClusterContext getClientClusterContext() {
        return ccc;
    }
    
    boolean isUsingClientOverrideXml() {
        return clientOverrideXml != null;
    }

    /**
     * Initializes the connection to the Catalog Service Domain. This method is to be called after all setter
     * methods have been called and will throw an {@link IllegalArgumentException} if 
     * {@link #setCatalogServiceEndpoints(String)} have not been called prior.
     * @throws Exception if {@link #setCatalogServiceEndpoints(String)} have not been called prior or an error 
     * occurs initializing the connection.
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        if (tc.isEntryEnabled()) {
            Tr.entry(tc, "afterPropertiesSet");
        }
        
        Assert.hasLength(catalogServiceEndpoints, Messages.getMsg(NLSConstants.INVALID_VALUE_ERROR_CWOBJ0008, 
                new Object[]{catalogServiceEndpoints, "catalogServiceEndpoints"}));
        
        if (tc.isDebugEnabled()) {
            Tr.debug(tc, "catalogServiceEndpoints: " + catalogServiceEndpoints);
            Tr.debug(tc, "clientOverrideXml: " + clientOverrideXml);
            Tr.debug(tc, "clientSecurityConfig: " + clientSecurityConfig);
        }
        
        URL clientOverrideXmlURL = (clientOverrideXml != null ? clientOverrideXml.getURL() : null);
        
        try {
            if (clientSecurityConfig != null) {
                
                clientSC = ClientSecurityConfigurationFactory
                        .getClientSecurityConfiguration(clientSecurityConfig.getFile().getAbsolutePath());
            }
            ccc = ObjectGridManagerFactory.getObjectGridManager().connect(catalogServiceEndpoints, 
                    clientSC, clientOverrideXmlURL);
        } catch (Exception e) {
            FFDCFilter.processException(e, getClass().getName() + ".afterPropertiesSet", "86", this);
            if (tc.isEventEnabled()) {
                Tr.event(tc, "Error making connection", e);
            }
            Tr.error(tc, Messages.getMsg(NLSConstants.CATALOG_SERVICE_DOMAIN_BEAN_INITIALIZATION_FAIL_CWOBJ7602, e.getMessage()));
            throw e; //rethrow
        }
    }
}
