/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
*/
package com.ibm.websphere.objectgrid.spring;

import java.util.List;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.ClientClusterContext;
import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.ObjectGridManager;
import com.ibm.websphere.objectgrid.ObjectGridManagerFactory;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.objectgrid.client.ClientProperties;
import com.ibm.websphere.objectgrid.config.BackingMapConfiguration;
import com.ibm.websphere.objectgrid.config.ObjectGridConfiguration;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.objectgrid.config.ObjectGridConfigurationImpl;
import com.ibm.ws.objectgrid.deployment.policy.ObjectGridDeploymentImpl;
import com.ibm.ws.objectgrid.naming.ILocationService;
import com.ibm.ws.objectgrid.naming.LocationServiceFactory;
import com.ibm.ws.xs.NLSConstants;
import com.ibm.ws.xs.util.Messages;

/**
 * <p>This class is a Spring bean representing an ObjectGrid client instance for use with
 * the WebSphere eXtreme Scale implementation of Spring's cache abstraction.</p>
 * 
 * <p>Users must provide a {@link ObjectGridCatalogServiceDomainBean} to configure this client. The ObjectGrid name is optional
 * when using the provided XML configuration files.</p>
 * 
 * @since 8.5, XC10
 */
public final class ObjectGridClientBean implements InitializingBean {
    
    final static String CLASS_NAME = ObjectGridClientBean.class.getName();
    
    final static TraceComponent tc = Tr.register(CLASS_NAME, Constants.TR_SPRING_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);
    
    /** Grid name for the template map used in the provided objectgrid and deployment xml files */
    private static final String DEFAULT_SPRING_GRID_NAME = "SPRING_REMOTE";
    
    private String gridName = null;
    private ObjectGridCatalogServiceDomainBean csDomain;
    private ObjectGrid grid;

    /**
     * Sets the name of the ObjectGrid for the client. This is optional when using the provided XML configuration files.
     * @param objectGridName The name of the ObjectGrid to connect to.
     */
    public void setObjectGridName(String objectGridName) {
        this.gridName = objectGridName;
    }

    /**
     * Sets the {@link ObjectGridCatalogServiceDomainBean} used by the client.
     * @param catalogServiceDomain The {@link ObjectGridCatalogServiceDomainBean} for the client.
     */
    public void setCatalogServiceDomain(ObjectGridCatalogServiceDomainBean catalogServiceDomain) {
        this.csDomain = catalogServiceDomain;
    }
    
    /**
     * Return the ObjectGrid instance of this client.
     * @return The ObjectGrid instance.
     */
    ObjectGrid getObjectGrid() {
        return grid;
    }

    
    /**
     * Initializes the client bean. This method is to be called after all setter methods have been called and will 
     * throw an {@link IllegalArgumentException} if 
     * {@link #setCatalogServiceDomain(ObjectGridCatalogServiceDomainBean)} has not been called prior.
     * @throws Exception if {@link #setCatalogServiceDomain(ObjectGridCatalogServiceDomainBean)} has not been 
     * called prior or an error occurs retrieving the ObjectGrid.
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        if (tc.isEntryEnabled()) {
            Tr.entry(tc, "afterPropertiesSet");
        }
        Assert.notNull(csDomain, Messages.getMsg(NLSConstants.INVALID_VALUE_ERROR_CWOBJ0008, 
                new Object[]{csDomain, "catalogServiceDomain"}));
        if (!StringUtils.hasLength(gridName)) {
            gridName = DEFAULT_SPRING_GRID_NAME;
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, "No value specified for objectGridName so using " + gridName + " for the object grid name");
            }
        }
        
        if (tc.isDebugEnabled()) {
            Tr.debug(tc, "gridName: " + gridName);
        }
        
        ClientClusterContext clientClusterContext = csDomain.getClientClusterContext();
        try {
            if (csDomain.isUsingClientOverrideXml()) {
                grid = ObjectGridManagerFactory.getObjectGridManager().getObjectGrid(clientClusterContext, gridName);
            } else { //not using a client override objectgrid.xml so disable the near cache
                if (tc.isDebugEnabled()) {
                    Tr.debug(tc, "Disable near cache");
                }
                grid = ObjectGridManagerFactory.getObjectGridManager().getObjectGrid(clientClusterContext, gridName, 
                        getOverrideObjectGridConfig(clientClusterContext, gridName));
            }
        } catch (Exception e) {
            FFDCFilter.processException(e, getClass().getName() + ".afterPropertiesSet", "66", this, new Object[] {});
            if (tc.isEventEnabled()) {
                Tr.event(tc, "Error retrieving Object grid named " + gridName, e);
            }
            Tr.error(tc, Messages.getMsg(NLSConstants.CLIENT_BEAN_INITIALIZATION_FAIL_CWOBJ7603, e.getMessage()));
            throw e;
        }
        // should be impossible, an exception will be thrown above from getObjectGrid()
        Assert.notNull(grid, "Unable to retrieve Websphere eXtreme Scale Object Grid named " + gridName);
    }
    
    private ObjectGridConfiguration getOverrideObjectGridConfig(ClientClusterContext ccc, String ogName) {
        String domain = ccc.getClusterName();

        if (tc.isEventEnabled()) {
            Tr.entry(tc, "Overriding the server configuration.");
        }

        ClientProperties clientProps = ccc.getClientProperties(ogName);
        ObjectGridConfiguration serverConfig = getServerObjectGridConfiguration(domain, ogName, clientProps);
        ObjectGridConfiguration clientConfig = null;
        if (serverConfig != null) {
            clientConfig = getObjectGridOverrideConfiguration(serverConfig);
        }

        return clientConfig;
    }
    
    private ObjectGridConfiguration getServerObjectGridConfiguration(String domain, String gridName,
            ClientProperties clientProps) {

        ILocationService locationService = LocationServiceFactory.getRemoteLocationService(domain);
        ObjectGridManager manager = ObjectGridManagerFactory.getObjectGridManager();
        ObjectGridDeploymentImpl deployment = null;
        try {
            deployment = (ObjectGridDeploymentImpl) ((ObjectGridManagerImpl) manager).getRemoteObjectGridDeployment(
                    locationService, gridName, clientProps);
        } catch (ObjectGridRuntimeException ogre) {
            // Thrown when gridName does not exist. Debug log and move on.
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "Caught exception calling getRemoteObjectGridDeployment(): exception=" + ogre);
            }
        }
        ObjectGridConfigurationImpl serverConfiguration = null;
        if (deployment != null) {
            serverConfiguration = (ObjectGridConfigurationImpl) deployment.getObjectGridConfiguration();
        }
        return serverConfiguration;
    }
    
    private ObjectGridConfiguration getObjectGridOverrideConfiguration(ObjectGridConfiguration serverConfig) {
        ObjectGridConfiguration clientConfig = serverConfig;
        List backingMapCfgs = clientConfig.getBackingMapConfigurations();
        // for(BackingMapConfiguration mapConfig : (List<BackingMapConfiguration>)
        // clientConfig.getBackingMapConfigurations()) {
        for (int i = 0; i < backingMapCfgs.size(); i++) {
            BackingMapConfiguration mapConfig = (BackingMapConfiguration) backingMapCfgs.get(i);

            // Disable the near cache
            mapConfig.setNumberOfBuckets(0);
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, "Disabled near cache for backing map configuration: " + mapConfig.getName());
            }
        }

        return clientConfig;
    }
}
