// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  Entity.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b995         6.1       05/22/2006   cdjohnson New part.
// b1591        6.1       07/10/2006   cdjohnson M4 feature additions
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.annotations;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Specifies that this class is an entity.
 *
 * @since WAS XD 6.1
 */
@Target( { TYPE })
@Retention(RUNTIME)
public @interface Entity {

    /**
     * The unqualified name for this entity.  The name is used to
     * refer to the entity in queries.<p>
     *
     * If the name is unspecified, the unqualified name of the entity class
     * will be used.<p>
     *
     * If the name specified is that of an entity that already exists, this
     * entity must include a subset of the existing entity.  An entity sub-set
     * provides the entire key and/or a partial value of the parent entity and is
     * most useful in client/server environments to reduce the amount of data that
     * is transported across the network transport layer.
     * @return the name of the entity.
     */
    public String name() default "";

//    /**
//     * If specified, this entity inherits from the parent entity.  Entity inheritance is much
//     * like object-oriented programming inheritance.  The child entity inherits all of the
//     * attributes and associations of the parent entity and can add new attributes and
//     * associations.<p>
//     *
//     * Example:
//     * <pre>
//     * <literal>@Entity</literal>(name="Person")
//     * public class Person {
//     *   <literal>@Id</literal> long taxId;
//     *   String name;
//     *   <literal>@ManyToOne</literal> Address homeAddress;
//     * }
//     *
//     * <literal>@Entity</literal>(name="Employee", parentEntity="Person")
//     * public class Employee extends Person {
//     *   <literal>@ManyToOne</literal> Department department;
//     *   long serialNumber;
//     * }
//     *
//     * </pre>
//     * @return the name of the parent entity.
//     */
//    public String parentEntity() default "";

    /**
     * Identifies the class in which this entity is a subset.  Subset entities are smaller
     * views of the actual entity stored in a TupleStore.  The most common use of a subset
     * entity is in client/server environments where the server's entity may have many
     * fields, but a client request need only see a few of those fields.<p>
     *
     * Subset entities have the following properties:
     * <ul>
     * <li>Same name as the superset entity.
     * <li>One or more attributes or associations of the superset entity.
     * <li>Requires the superset entity's full key attributes (if a composite key)
     * <li>If read-only, the key is not required.
     * </ul>
     *
     * Example:
     * <pre>
     * <literal>@Entity</literal>(name="Person")
     * public class Person {
     *   <literal>@Id</literal> long taxId;
     *   <literal>@Id</literal> String countryCode;
     *   String firstName;
     *   String lastName;
     *   String middleName;
     *   <literal>@ManyToOne</literal> Address homeAddress;
     *   String favoriteColor;
     *   String hatSize;
     *   String hairColor;
     *   String eyeColor;
     *   ...
     * }
     *
     * <literal>//</literal> Subset entities have the same name, just a different class.
     * <literal>@Entity</literal>(name="Person")
     * public class PersonReadOnlyView {
     *   String firstName;
     *   String lastName;
     *   String middleName;
     * }
     *
     * <literal>@Entity</literal>(name="Person")
     * public class PersonReadWriteView {
     *   <literal>@Id</literal> long taxId;
     *   <literal>@Id</literal> String countryCode;
     *   String firstName;
     *   String lastName;
     *   String middleName;
     *   <literal>@ManyToOne</literal> Address homeAddress;
     * }
     *
     * </pre>
     * @return that class of the entity that this entity is a subset.
     */
    public Class supersetEntity() default void.class;

    /**
     * If specified, override the access type of this entity.
     * This attribute is not normally required unless the entity has
     * no key.
     * @return the AccessType for this entity.
     */
    public AccessType accessType() default AccessType.DEFAULT;

    /**
     * Specifies that this entity is the root entity for the schema.  Each
     * schema (projector instance) can only have one root.  Entity managers
     * may use the root for routing in partitioned environments.
     * @return true if this is the root entity of the schema.
     */
    public boolean schemaRoot() default false;
}
